library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity pwmgen is
	port (
	clk: in STD_LOGIC;
	refcount: in STD_LOGIC_VECTOR (10 downto 0);
	ibus: in STD_LOGIC_VECTOR (15 downto 0);
	obus: out STD_LOGIC_VECTOR (15 downto 0);
	loadpwmval: in STD_LOGIC;
	readpwmval: in STD_LOGIC;
	clearpwmval: in STD_LOGIC;
	pcrloadcmd: STD_LOGIC;
	pcrreadcmd: STD_LOGIC;	
	pwmout: out STD_LOGIC;
	dirout: out STD_LOGIC;
	enablein: in STD_LOGIC;
	enableout: out STD_LOGIC
	);
end pwmgen;

architecture behavioral of pwmgen is

signal pwmval: STD_LOGIC_VECTOR (10 downto 0);
signal fixedpwmval: STD_LOGIC_VECTOR (10 downto 0);
signal fixedrefcount: STD_LOGIC_VECTOR (10 downto 0);
signal dir: STD_LOGIC;
signal pwm: STD_LOGIC;
signal loadpwmval1: STD_LOGIC;
signal loadpwmval2: STD_LOGIC;
signal localenable: STD_LOGIC;
signal lacedpwm: STD_LOGIC;
signal pcrloadcmd1: STD_LOGIC;
signal pcrloadcmd2: STD_LOGIC;

begin
	apwmgen: process  (clk, 
	 						refcount,
							ibus,
							loadpwmval,
							readpwmval,
							clearpwmval,
							enablein,
							localenable,
							lacedpwm,
							pwmval,
							dir, 
							pcrreadcmd,
						 	pwm
							)
							
	begin
		if clk'event and clk = '1' then
			loadpwmval1 <= loadpwmval;
			loadpwmval2 <= loadpwmval1;
			pcrloadcmd1 <= pcrloadcmd;
			pcrloadcmd2 <= pcrloadcmd1;

			if dir = '1' then
				fixedpwmval <= (not pwmval) +1;		-- convert from signed-magnitude to 2s comp 
			else
				fixedpwmval <= pwmval;
			end if;
			dirout <= dir;

			if lacedpwm = '1' then						-- interlaced (600 kc) output for analog
				fixedrefcount(10) <= refcount(5);
				fixedrefcount(9) <= refcount(6);
				fixedrefcount(8) <= refcount(7);
				fixedrefcount(7) <= refcount(8);
				fixedrefcount(6) <= refcount(9);	
				fixedrefcount(5) <= refcount(10);
				fixedrefcount(4 downto 0) <= refcount(4 downto 0);
			else
				fixedrefcount <= refcount;
			end if;			
			
			if (UNSIGNED(fixedrefcount) < UNSIGNED(fixedpwmval)) and (enablein = '1') and (localenable = '1') then 
				pwm <= '1'; 
			else 
				pwm <= '0';
			end if;
			pwmout <= pwm;
			if (loadpwmval1 = '0') and (loadpwmval2 = '1') then -- trailing edge of command
 		   	pwmval <= ibus(14 downto 4);
				dir <= ibus(15);
			end if;	
			if (pcrloadcmd1 = '0') and (pcrloadcmd2 = '1') then -- trailing edge of command
 		   	lacedpwm <= ibus(1);
				localenable <= ibus(0);
			end if;	
		end if;
		if clearpwmval = '1' then
	 		pwmval <= "00000000000";
			localenable <= '0';
		end if;
		if readpwmval = '1' then
		obus(14 downto 4) <= pwmval;
		obus(15) <= dir;
		obus(3 downto 0) <= "0000";
		else
			obus <= "ZZZZZZZZZZZZZZZZ";
		end if;
		if pcrreadcmd = '1' and readpwmval = '0' then
		obus(2) <= pwm;
		obus(1) <= lacedpwm;
		obus(0) <= localenable;
		obus(15 downto 3) <= "ZZZZZZZZZZZZZ";
		else
			obus <= "ZZZZZZZZZZZZZZZZ";
		end if;		
		enableout <= not localenable;
	end process;
end behavioral;

