program DemoMove;
uses DOS,crt;


{$R-}    {Range checking be off }
{$I-}    {No I/O checking }
{$S-}    {No stack checking}

{$M 8192,0,32768}              { use minimum memory, but a large stack...      }
                               { don't make the heap any smaller or you'll be in heap big trouble }

{$I 8259.pas}
type

LongIntRec = record
  LowWord  : word;
  HighWord : word;
end;

LongIntIntRec = record
  LowWord  : word;
  HighWord : integer;
end;

TraceRec = record
  TRDesPos : longint;
  TRPosErr : longint;
  TRDrive : integer;
end;

Motorx = record
      CurPos : longint;
      DesPos : longint;
      NewPos : longint;
      FinalPos : longint;
      DesFrac : word;
      LastPos : longint;
      PosErr : longint;
      MaxPosErr : integer;
      MinPosErr : integer;
      PosErrForStop : integer;
      IntegralErr : longint;
      Accel : longint;
      MaxVel : longint;
      SignedMaxVel : longint;
      Velocity : longint;
      DesVel : longint;
      ActVel : Integer;
      HalfPos : longint;
      KP : integer;
      KD : integer;
      KI : longint;
      IL : longint;
      KF : integer;
      DeadZone : longint;
      RunProfile : boolean;
      PIDOn : boolean;
      EXPE : boolean;
      end;

MotorP = ^MotorX;

const
TheInterrupt : byte = 10;
BasePort = $220;
LastMotor = 3;
LastActiveMotor = 3;
{$I hostmlow.pas}
TraceMax = 100;

var
StartTraceBuffer : array[1..500] of TraceRec;
EndTraceBuffer :  array[1..500] of TraceRec;
StartTraceCount : word;
EndTraceCount : word;
TraceCount : word;
OldVector : pointer;

Motors : array [0..lastmotor] of Motorx;

IRQDiv : byte;
FooKey : char;
CurChan : byte;
{ interrupt variables }
dummy : byte;
countlow,counthigh : word;
longdrive : longint;
tdespos,tdesvel : longint;
signfix,axt,bxt,cxt,tdesfrac : word;
SSSave : word;
SPSave : word;
NewSS : word;
NewSP : word;
foostr : string;

procedure startup;
begin
  for CurChan := 0 to LastMotor do
  begin
    with Motors[CurChan] do
    begin
      DesPos := 0;
      CurPos := 0;
      Velocity := 0;
      Accel := 0;
      MaxVel := 0;
      PosErrForStop := 500;
      IntegralErr := 0;
      KD := 3500;
      KP := 500;
      KI := 80000;
      IL := 8192*65536; {1/8 FS}
      KF := 900;
      KI := 0;
      IL := 0;
      KF := 0;
      RunProfile := false;
      DeadZone := 1;
      PIDOn := false;
      EXPE := false;
      StartTraceCount := 1;
      EndTraceCount := 1;
      { do per channel setup stuff}
      port[BasePort+IndexReg] := CurChan;
      portw[BasePort + CCReg] := $54; { local Count enable + latch on read + clear counters }
      port[BasePort + PCReg] := $01; { local PWM enable}
    end;
  end;
  port[BasePort + GMReg] := $03; { global PWM enable and Count enable }
  port[BasePort +LEDReg] := $01; { set LED reg to point to channel 1 }
  IrqDiv := 4;
  NewSS := SSeg;
  NewSP := SPtr - 4096;               { we will use the current stack later... }
end;

procedure usage;
begin
  writeln('USAGE MOVE ');
end;

procedure GetParms;
var
retcode : integer;
channel : byte;
begin
  if ParamCount <> 0 then usage;
end;

procedure PID;
begin
  { Do all motors }
  for CurChan := LastActiveMotor downto 0 do
  begin
    with Motors[CurChan] do
    begin;
      { General things done every tic }
      Port[BasePort+IndexReg] := CurChan;        {select channel}
      countlow := portw[BasePort + CountLowReg]; {read current position}
      counthigh := portw[BasePort + CountHighReg];
      CurPos := countlow + (counthigh*65536);
      PosErr := CurPos-DesPos;
      If PosErr > MaxPosErr then MaxPosErr := PosErr;
      If PosErr < MinPosErr then MinPosErr := PosErr;
      ActVel := LastPos-CurPos;
      LastPos := CurPos;

      {PID loop motor drive math }
      if PIDOn then
      begin
        IntegralErr := IntegralErr + PosErr*KI;
        if IntegralErr >  IL then IntegralErr :=  IL;             { bound IntegralErr by IL }
        if IntegralErr < -IL then IntegralErr := -IL;
        { this is the actual PID loop calculation: }
        longdrive := KP*PosErr                                    { Proportional term }
                   + LongIntIntRec(IntegralErr).HighWord          { Integral term }
                   - KD*(ActVel + LongIntIntRec(DesVel).Highword) { Derivative term }
                   - KF*LongIntIntRec(DesVel).Highword;           { Velocity feed forward term }
        if longdrive >  32700 then longdrive :=  32700;
        if longdrive < -32700 then longdrive := -32700;
        if (Velocity = 0) and (abs(longdrive) < DeadZone) then Longdrive := 0;
        portw[BasePort+PWMReg] := word(longdrive);
        if abs(PosErr) > PosErrForStop then
        begin
          portw[BasePort+PWMReg] := 0;  { if we're out of range, set the motor drive to 0 }
          PIDOn := false;   { turn the PID loop off }
          EXPE := true;     { set the excessive position error flag }
          RunProfile := false;   { stop running profile - were not going anywhere }
        end;
      end;

      { profile generator }
      if RunProfile = true then
      begin
        if (CurChan = 1) then
        begin
          if (StartTraceCount < TraceMax) then
          begin
            StartTraceBuffer[StartTraceCount].TRDesPos := DesPos;
            StartTraceBuffer[StartTraceCount].TRPosErr := PosErr;
            StartTraceBuffer[StartTraceCount].TRDrive := word(longdrive);
            StartTraceCount := StartTraceCount + 1;
          end;
          EndTraceBuffer[EndTraceCount].TRDesPos := DesPos;
          EndTraceBuffer[EndTraceCount].TRPosErr := PosErr;
          EndTraceBuffer[EndTraceCount].TRDrive := word(longdrive);
          EndTraceCount := EndTraceCount + 1;
          if EndTraceCount > TraceMax then EndTraceCount := 1;
        end;

        if DesPos < HalfPos then
          Velocity := Velocity + Accel
        else
          Velocity := Velocity - Accel;
        if Velocity = 0 then  { we're done! }
        begin
          FinalPos := DesPos;
          RunProfile := false;
          Accel := 0;
          DesPos := NewPos;
          Velocity := 0;
          DesVel := 0;
        end;
        if abs(Velocity) <= MaxVel then DesVel := Velocity else DesVel := SignedMaxVel;
        if DesVel < 0 then signfix := $FFFF else signfix := $0000;
        tdespos := DesPos;
        tdesfrac := DesFrac;
        tdesvel := DesVel;
        asm
          { get three words}
          mov ax,tdesfrac
          mov bx,LongIntRec(tdespos).LowWord
          mov cx,LongIntRec(tdespos).HighWord
          { do the add }
          add ax,LongIntRec(tdesvel).LowWord
          adc bx,LongIntRec(tdesvel).HighWord
          { last add fixes sign }
          adc cx,signfix
          { restore data (why the casts above dont work here I do not know...}
          mov axt,ax
          mov bxt,bx
          mov cxt,cx
        end;
        { restore values in motor record }
        { doesnt work in asm for some reason }
        DesFrac := axt;
        LongIntRec(DesPos).LowWord := bxt;
        LongIntRec(DesPos).HighWord := cxt;
      end;
   end;
   dummy := port[BasePort + IRQSelReg]; { clear interrupt }
  end;
  IssueEOI2;
end;

procedure ISR;
interrupt;
begin
  PID;
end; { InterruptServiceRoutine }

procedure NEWISR;
interrupt;
{ this is the actual hardware interrupt routine }
{ all it does is create a local stack for us to so we  }
{ do not have to depend on the kindness of strangers...}

begin
  asm
    MOV AX,SS          { save old SS and SP }
    MOV SSSave,AX
    MOV AX,SP
    MOV SPSave,AX
    MOV AX,NewSS
    MOV SS,AX
    MOV AX,NewSP
    MOV SP,AX
  end;
  PID;
  asm
    MOV AX,SSSave      { restore old SS and SP (we hope) }
    MOV SS,AX
    MOV AX,SPSave
    MOV SP,AX
  end;
end; { InterruptServiceRoutine }

procedure Move(chan : byte;nextpos : longint; acc : longint; maxv : longint);
begin
  writeln('Chan ',chan,' Move to ',nextpos);
  with Motors[chan] do
  begin
    DisableInterrupts;
    MaxVel := maxv;
    if (nextpos-DesPos) < 0 then
    begin
      SignedMaxVel := -maxv;
    end
    else
    begin
      SignedMaxVel := maxv;
    end;
    HalfPos := (DesPos + nextpos) div 2;
    NewPos := NextPos;
    LastPos := CurPos;
    Accel := acc;
    Velocity := 0;
    DesVel := 0;
    DesFrac := 32768;
{    IntegralErr := 0;}
    MaxPosErr := 0;
    MinPosErr := 0;
    RunProfile := true;
    EXPE := false;
    PIDOn := true;
    StartTraceCount := 1;
    EndTraceCount := 1;
    EnableInterrupts;
  end;
end;

function ProfileRunning(theMotor : byte) : boolean;
begin
  ProfileRunning := Motors[theMotor].RunProfile;
end;

function MotorEXPE(theMotor : byte) : boolean;
begin
  MotorEXPE := Motors[theMotor].EXPE;
end;

procedure PIDOn(theMotor : byte);
begin
  Motors[theMotor].PIDOn := true;
end;

procedure StartInterrupts;
begin
  DisableInterrupts;
  GetHardIntVec(TheInterrupt,OldVector);
  SetHardIntVec(TheInterrupt,@NEWISR);
  port[BasePort + IRQSelReg] := TheInterrupt + $10; { select IRQ + enable driver }
  port[BasePort + IRQDivReg] := IrqDiv;
  port[BasePort + IRQSelReg] := TheInterrupt + $30; { select IRQ + enable driver + unmask }
  UnmaskInterrupt(TheInterrupt);
  EnableInterrupts;
  writeln('Control loop running');
end;

procedure StopInterrupts;
begin
  DisableInterrupts;
  port[BasePort + GMReg] := $00; { Disable all PWM outputs}
  SetHardIntVec(TheInterrupt,OldVector);
  port[BasePort + IRQSelReg] := 0; { Disable driver }
  port[BasePort + IRQDivReg] := 0;
  UnmaskInterrupt(TheInterrupt);
  EnableInterrupts;
  for CurChan := 0 to LastMotor do
  begin
      port[BasePort+IndexReg] := CurChan;
      port[BasePort + PCReg] := $00; { turn off enable bit on interface}
  end;
  writeln('Control loop shut down');
end;

procedure WaitForMotors;
begin
  while (ProfileRunning(0) or ProfileRunning(1)) do
  begin
    with Motors[1] do
    begin
      writeln('PE= ',PosErr,
      ' AV= ',ActVel,
      ' MSDV= ',LongIntIntRec(DesVel).HighWord,
      ' MSIE= ',LongIntIntRec(IntegralErr).HighWord);
    end;
    if MotorEXPE(0) or MotorEXPE(1) then
    begin
      writeln('Shut down due to excessive position error');
      StopInterrupts;
      halt(2);
    end;
  end;
  writeln('Got to (0,1) ',Motors[0].DesPos,',',Motors[1].DesPos);
  writeln('DDA Got to (0,1) ',Motors[0].FinalPos,',',Motors[1].FinalPos);
  writeln('MaxPosErr = ',Motors[0].MaxPosErr,' MinPosErr = ',Motors[0].MinPosErr);
  writeln('Trace of first TraceMax tics');
  for StartTraceCount := 1 to TraceMax do
  begin
    write
    (StartTraceBuffer[StartTraceCount].TRDesPos,
    ',',StartTraceBuffer[StartTraceCount].TRPosErr,
    ',',StartTraceBuffer[StartTraceCount].TRDrive,'  ');
    if (StartTraceCount mod 5) = 0 then writeln;
  end;
  writeln('Press return to continue...');
  readln(foostr);
  writeln('Trace of Last TraceMax tics');
  { we start this loop in the location in the EndTraceBuffer }
  { at 1 past the last location written to (the oldest location)}
  for TraceCount := 1 to TraceMax do
  begin
    write
    (EndTraceBuffer[EndTraceCount].TRDesPos,
    ',',EndTraceBuffer[EndTraceCount].TRPosErr,
    ',',EndTraceBuffer[EndTraceCount].TRDrive,'  ');
    if (TraceCount mod 5) = 0 then writeln;
    EndTraceCount := EndTraceCount +1;
    if EndTraceCount > TraceMax then EndTraceCount := 1;
  end;
  writeln('Press return to continue...');
  readln(foostr);

end;

begin
  GetParms;
  startup;
  StartInterrupts;
  PIDOn(0);
  PIDOn(1);
  delay(10);
  while not keypressed do
  begin
    delay(50);
    move(0,0,44,250000);
    move(1,0,44,250000);
    WaitForMotors;
    move(0, 2640*3,98,250000);
    move(1,-2640*3,98,250000);
    WaitForMotors;
    move(0,-2640*3,98,250000);
    move(1, 2640*3,98,250000);
    WaitForMotors;
  end;
  FooKey := ReadKey;
{  keep(0); }
  StopInterrupts;
  halt(0);
end.
