program I34LOOP;
uses dos;

{$I hexread}
{$I hprint}
{$I beepy}

const
AllIn  : word = $0000;
AllOut : word = $FFFF;

var
BasePort1,BasePort2 : word;

procedure BadBeeps;
begin
  BeepOn(800);
  BeepDelay(20);
  BeepOn(1500);
  BeepDelay(10);
  BeepOff;
end;

procedure HappyBeeps;
begin
  BeepOn(1000);
  BeepDelay(6);
  BeepOn(1200);
  BeepDelay(3);
  BeepOn(1000);
  BeepDelay(6);
  BeepOn(1200);
  BeepDelay(3);
  BeepOn(1000);
  BeepDelay(6);
  BeepOn(1200);
  BeepDelay(3);
  BeepOff;
end;

procedure SetPPMode(baseport: word; themode : word);
var
conport : word;
begin
  conport := baseport + 8;
  portw[conport] := themode;
  portw[conport+2] := themode;
  {set 2 ports directions at once since we are testing 32 bits at a time}
end;

procedure OutOneZero(baseport : word; thebit : word);
var
dword : word;
bitport : word;
begin                  { set one bit in the 32 bit output field of 4I34 low }
                       { baseport is the base port of the 4I34 }
  portw[baseport]   := $FFFF;
  portw[baseport+2] := $FFFF;
  { first set all bits high }
  bitport := (baseport + (thebit div 8)) and $FFFE;
  dword := 1;
  dword := dword shl (thebit mod 16);
  dword := dword xor $FFFF;
  portw[bitport] := dword
end;

procedure OutOneOne(baseport : word; thebit : word);
var
dword : word;
bitport : word;
begin                  { set one bit in the 32 bit output field of 4I34  high }
                       { baseport is the base port of the 4I34 }
  portw[baseport]   := $0000;
  portw[baseport+2] := $0000;
  bitport := (baseport + (thebit div 8)) and $FFFE;
  dword := 1;
  dword := dword shl (thebit mod 16);
  portw[bitport] := dword
end;

function CheckOneZero(baseport : word; thebit : byte): boolean;
var
dword : word;
tword : word;
pword : word;
bitport : word;
checkport : word;
portok : boolean;
loops : byte;
begin                  { check for one low bit only in the 24 bit output field of our 4I34 }
                       { baseport is the base port of the 4I34 }
  portok := true;
  bitport := (baseport + (thebit div 8)) and $FFFE;
  dword := 1;
  dword := dword shl (thebit mod 16);
  dword := dword xor $FFFF;
  checkport := baseport;
  for loops := 1 to 2 do
  begin
    if checkport = bitport then tword := dword else tword := $FFFF;
    pword := portw[checkport];
    if pword <> tword then
    begin
      portok := false;
      write('Error during rotating zero test, expected ');
      HexPrint(tword,4);
      write(' but found ');
      HexPrint(pword,4);
      write(' at port ');
      HexPrint(CheckPort,4);
      writeln;
    end;
    checkport := checkport + 2;
  end;
  CheckOneZero := portok;
end;

function CheckOneOne(baseport : word; thebit : byte): boolean;
var
dword : word;
tword : word;
pword : word;
bitport : word;
checkport : word;
portok : boolean;
loops : byte;
begin                  { check for one high bit only in the 32 bit output field of our 4I34}
                       { baseport is the base port of the 4I34 card }
  portok := true;
  bitport := (baseport + (thebit div 8)) and $FFFE;
  dword := 1;
  dword := dword shl (thebit mod 16);
  checkport := baseport;
  for loops := 1 to 2 do
  begin
    if checkport = bitport then tword := dword else tword := $0000;
    pword := portw[checkport];
    if pword <> tword then
    begin
      portok := false;
      write('Error during rotating one test, expected ');
      HexPrint(tword,4);
      write(' but found ');
      HexPrint(pword,4);
      write(' at port ');
      HexPrint(CheckPort,4);
      writeln;
    end;
    checkport := checkport + 2;
  end;
  CheckOneOne := portok;
end;

procedure BumOut(s: string);
begin
  BadBeeps;
  writeln(s);
  halt(2);
end;

procedure DoRotBitTest(outport,inport : word);
var
testbit : word;
times : word;

begin

  write('Rotating bit test, output port = ');
  HexPrint(outport,4);
  write(' and input port = ');
  HexPrint(inport,4);
  writeln;
  SetPPMode(inport,AllIn);
  SetPPMode(outport,AllOut);
  for times := 1 to 100 do
  begin
    for testbit := 0 to 31 do
    begin
      OutOneOne(outport,testbit);
      if not CheckOneOne(inport,testbit) then BumOut('4I34 LOOPBACK FAILURE!');
      OutOneZero(outport,testbit);
      if not CheckOneZero(inport,testbit) then BumOut('4I34 LOOPBACK FAILURE!');
    end;
  end;
  { avoid conflicts..}
  SetPPMode(inport,AllIn);
  SetPPMode(outport,AllIn);

end;

procedure TestPorts;
begin
  DoRotBitTest(BasePort1,BasePort2);
  DoRotBitTest(BasePort2,BasePort1);
end;

procedure PrintUsage;
begin
  writeln;
  writeln('4I34LOOP -- Test 4I34 type parallel ports via loopback cable');
  writeln;
  writeln('USAGE -- 4I34LOOP PORT1 PORT2');
  writeln;
  writeln('Where PORT1 and PORT2 are hex base port addresses');
  writeln('Requires single or multiple 4I34s looped back to the');
  writeln('other side or other card with an external cable');
  halt(2);
end;

procedure GetPorts;

begin
  if (ParamCount <> 2) then PrintUsage;
  if not HexWordRead(ParamStr(1),BasePort1) then BumOut('Undecipherable hex number > '+ParamStr(1));
  if not HexWordRead(ParamStr(2),BasePort2) then BumOut('Undecipherable hex number > '+ParamStr(2));;
  if BasePort1 = BasePort2 then BumOut('Ports 1 and 2 must be different!');
end;

begin
  GetPorts;
  TestPorts;
  HappyBeeps;
  Writeln('4I34 PORT TESTS OK!');
end.