program I34MLOOP;
uses dos;

{$I hexread}
{$I hprint}
{$I beepy}

const
AllIn  : word = $0000;
AllOut : word = $FFFF;
LowIn : word = $FF00;
HighIn : word = $00FF;

var
BasePort : word;
Port0,Port1,Port2,DDR0,DDR1,DDR2 : word;

procedure BadBeeps;
begin
  BeepOn(800);
  BeepDelay(20);
  BeepOn(1500);
  BeepDelay(10);
  BeepOff;
end;

procedure HappyBeeps;
begin
  BeepOn(1000);
  BeepDelay(6);
  BeepOn(1200);
  BeepDelay(3);
  BeepOn(1000);
  BeepDelay(6);
  BeepOn(1200);
  BeepDelay(3);
  BeepOn(1000);
  BeepDelay(6);
  BeepOn(1200);
  BeepDelay(3);
  BeepOff;
end;


procedure BumOut(s: string);
begin
  BadBeeps;
  writeln(s);
  halt(2);
end;

procedure Test1;
begin
{ first do a simple sanity check }
  writeln('Simple Readback check...');
{ set all I/O bits to input mode }
  portw[DDR0] := AllIn;
  portw[DDR1] := AllIn;
  portw[DDR2] := AllIn;
{ if all inputs, we expect pullups to make all inputs high }
  if portw[Port0] <> $FFFF then BumOut('Port 0 pullup test error');
  if portw[Port1] <> $FFFF then BumOut('Port 1 pullup test error');
  if portw[Port2] <> $FFFF then BumOut('Port 2 pullup test error');
{ set all the output latches to 0}
  portw[Port0] := $0000;
  portw[Port1] := $0000;
  portw[Port2] := $0000;
{ then enable the output drivers on all bits}
  portw[DDR0] := $FFFF;
  portw[DDR1] := $FFFF;
  portw[DDR2] := $FFFF;
  if portw[Port0] <> $0000 then BumOut('Port 0 readback test error');
  if portw[Port1] <> $0000 then BumOut('Port 1 readback test error');
  if portw[Port2] <> $0000 then BumOut('Port 2 readback test error');

  writeln('Readback check ok');
end;

procedure Test2;
begin
  writeln('loopback through cable test - P2 --> P3');
{ set all I/O bits to input mode so we dont have conflicts }
  portw[DDR0] := AllIn;
  portw[DDR1] := AllIn;
  portw[DDR2] := AllIn;
{ next set the 24 P2 bits to output mode }
  portw[DDR0] := AllOut;
  portw[DDR1] := HighIn;
{ then set the outputs to AAAAAA}
  portw[Port0] := $AAAA;
  portw[Port1] := $AAAA;
{ now check that we can read the outputs back on P3}
  if portw[Port1] <> $AAAA then BumOut('Read at Port 1 Loopback failure');
  if portw[Port2] <> $AAAA then BumOut('Read at Port 2 Loopback failure');
{ then set the outputs to 555555}
  portw[Port0] := $5555;
  portw[Port1] := $5555;
{ now check that we can read the outputs back on P3}
  if portw[Port1] <> $5555 then BumOut('Read at Port 1 Loopback failure');
  if portw[Port2] <> $5555 then BumOut('Read at Port 2 Loopback failure');

  writeln('loopback through cable test - P3 --> P2');
{ set all I/O bits to input mode so we dont have conflicts }
  portw[DDR0] := AllIn;
  portw[DDR1] := AllIn;
  portw[DDR2] := AllIn;
{ next set the 24 P3 bits to output mode }
  portw[DDR1] := LowIn;
  portw[DDR2] := AllOut;
{ then set the outputs to AAAAAA}
  portw[Port1] := $AAAA;
  portw[Port2] := $AAAA;
{ now check that we can read the outputs back on P2}
  if portw[Port0] <> $AAAA then BumOut('Read at Port 0 Loopback failure');
  if portw[Port1] <> $AAAA then BumOut('Read at Port 1 Loopback failure');
{ then set the outputs to 555555}
  portw[Port1] := $5555;
  portw[Port2] := $5555;
{ now check that we can read the outputs back on P3}
  if portw[Port1] <> $5555 then BumOut('Read at Port 0 Loopback failure');
  if portw[Port2] <> $5555 then BumOut('Read at Port 1 Loopback failure');

  writeln('loopback through cable test OK');

end;


procedure SetUpPortAdds;
begin
  Port0 := BasePort+$00;
  Port1 := BasePort+$02;
  Port2 := BasePort+$04;
  DDR0  := BasePort+$08;
  DDR1  := BasePort+$0A;
  DDR2  := BasePort+$0C;
end;


procedure TestPorts;
begin
  Test1;
  Test2;
end;

procedure PrintUsage;
begin
  writeln;
  writeln('4I34MLOP -- Test 4I34M type parallel ports via loopback cable');
  writeln;
  writeln('USAGE -- 4I34MLOP PORT');
  writeln;
  writeln('Where PORT1 is the base port address of the 4I34M');
  writeln('Requires single 4I34M with connectors looped back');
  writeln('to the other side of card with an external cable');
  halt(2);
end;

procedure GetPorts;

begin
  if (ParamCount <> 1) then PrintUsage;
  if not HexWordRead(ParamStr(1),BasePort) then BumOut('Undecipherable hex number > '+ParamStr(1));
end;

begin
  GetPorts;
  SetUpPortAdds;
  TestPorts;
  HappyBeeps;
  Writeln('4I34M PORT TESTS OK!');
end.