library IEEE;
use IEEE.std_logic_1164.all;            -- defines std_logic types

entity quadm is
  port
    (
      -- bus interface signals --
      BALE      : in    std_logic;
      TC        : in    std_logic;
      BOSC      : in    std_logic;
      RSTDRV    : in    std_logic;
      REFRES    : in    std_logic;
      SBHE      : in    std_logic;
      MASTER    : in    std_logic;
      AEN       : in    std_logic;
      SMEMR     : in    std_logic;
      SMEMW     : in    std_logic;
      IORD      : in    std_logic;
      IOWR      : in    std_logic;
--      IOCHRDY: inout STD_LOGIC; 
--      MEMCS16: inout STD_LOGIC; 
      IOCS16    : inout std_logic;
      SA        : in    std_logic_vector (19 downto 0);  -- address bus
      SD        : inout std_logic_vector (15 downto 0);  -- data bus
      IRQ       : out   std_logic_vector (15 downto 3);  -- interrupts
      DACK      : in    std_logic_vector (7 downto 6);   -- dma acknowledges
      DRQ       : out   std_logic_vector (7 downto 6);   -- dma requests
      LBE       : out   std_logic;
      LBDIR     : out   std_logic;
      SynClk    : in    std_logic;
      -- I/O Port bits
      PORTADATA : inout std_logic_vector (11 downto 0);
      PORTBDATA : inout std_logic_vector (11 downto 0);
      -- on card debug LEDS
      LEDS      : out   std_logic_vector(7 downto 0);
      -- configuration decode disable
      DIS       : out   std_logic;
      -- motor control stuff
      A         : in    std_logic_vector (3 downto 0);
      B         : in    std_logic_vector (3 downto 0);
      IDX       : in    std_logic_vector (3 downto 0);
      PWM       : inout std_logic_vector (3 downto 0);
      ENA       : out   std_logic_vector (3 downto 0);
      DIR       : inout std_logic_vector (3 downto 0)
      );



end quadm;

architecture dataflow of quadm is

  constant naxis        : integer                       := 4;
  constant Naxisrb      : std_logic_vector(15 downto 0) := x"0004";
  constant Hardwaretype : std_logic_vector(15 downto 0) := x"01B2";
  constant Revision     : std_logic_vector(15 downto 0) := x"0017"; -- M changed to MAC


--      constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100001";  -- 0x210 hex
  constant decode : std_logic_vector (5 downto 0) := "100010";  -- 0x220 hex
--      constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100011";  -- 0x230 hex       
--      constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100100";  -- 0x240 hex
--      constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100101";  -- 0x250 hex       

-- misc global signals --
  signal CardSelect    : std_logic;     -- card select decode
                                        -- index register
  signal DBus          : std_logic_vector (15 downto 0);  -- internal data bus
  signal PreDBLatch    : std_logic_vector (15 downto 0);  -- latched data bus     
  signal DBLatch       : std_logic_vector (15 downto 0);  -- latched data bus
  signal PreSALatch    : std_logic;     -- latched address sa1      
  signal SALatch       : std_logic;     -- latched address sa1  
  signal ROMWrite      : std_logic;
  signal ISAadd        : std_logic_vector (10 downto 0);
  signal WordAccess    : std_logic;
  signal idISARB       : std_logic_vector (15 downto 0);
  signal ROMAddrRegSel : std_logic;
  signal ROMDataSel    : std_logic;
  signal ROMDatawr1    : std_logic;
  signal ROMDatawr2    : std_logic;
  signal ResetRegSel   : std_logic;
  signal ResetRegwr1   : std_logic;
  signal ResetRegwr2   : std_logic;
  signal DBReset       : std_logic;
  signal addpointwr1   : std_logic;
  signal addpointwr2   : std_logic;
  signal LEDLatch      : std_logic_vector (7 downto 0);
  signal DebugAccumLat : std_logic_vector (15 downto 0);
  signal DebugPCLat    : std_logic_vector (7 downto 0);
  signal DebugInstLat  : std_logic_vector (15 downto 0);
  signal DebugFooLat   : std_logic_vector (7 downto 0);
  signal ia            : std_logic_vector(11 downto 0);
  signal id            : std_logic_vector(15 downto 0);
  signal mra           : std_logic_vector(10 downto 0);
  signal mwa           : std_logic_vector(10 downto 0);

  signal mi                  : std_logic_vector(15 downto 0);
  signal mo                  : std_logic_vector(15 downto 0);
  signal mw                  : std_logic;
  signal mipram              : std_logic_vector(15 downto 0);
  signal migram              : std_logic_vector(15 downto 0);
  signal pwrite              : std_logic;
  signal gwrite              : std_logic;
  signal dbcarry             : std_logic;
  signal ioradd              : std_logic_vector(9 downto 0);
  alias iowsel               : std_logic is mwa(9);
  alias iowdec               : std_logic_vector(7 downto 0) is mwa (7 downto 0);
  alias iorsel               : std_logic is ioradd(9);
  alias iordec               : std_logic_vector(7 downto 0) is ioradd (7 downto 0);
  signal PagedRAMReadSel     : std_logic;
  signal PagedRAMWriteSel    : std_logic;
  signal GlobalRAMReadSel    : std_logic;
  signal GlobalRAMWriteSel   : std_logic;
  signal CommandRegReadSel   : std_logic;
  signal HStatusRegWriteSel  : std_logic;
  signal HStatusRegReadSel   : std_logic;
  signal DataRegReadSel      : std_logic;
  signal DataRegWriteSel     : std_logic;
  signal PageRegReadSel      : std_logic;
  signal PageRegWriteSel     : std_logic;
  signal HostReadCommandSel  : std_logic;
  signal HostWriteCommandSel : std_logic;
  signal HostReadDataSel     : std_logic;
  signal HostWriteDataSel    : std_logic;
  signal AxisRegReadSel      : std_logic;
  signal AxisRegWriteSel     : std_logic;
  signal AxisReg             : std_logic_vector (3 downto 0);
  signal AxisRegRB           : std_logic_vector (3 downto 0);
  signal PagedWriteAddress   : std_logic_vector (9 downto 0);
  signal PagedReadAddress    : std_logic_vector (9 downto 0);
  signal StatusRegReadSel    : std_logic;
  signal StatusRegWriteSel   : std_logic;
  signal MRegWriteSel        : std_logic;
  signal MRegReadSel         : std_logic;
  signal AlignReadSel        : std_logic;
  signal AlignWriteSel       : std_logic;


  signal PWMSel           : std_logic;
  signal DirSel           : std_logic;
  signal Counter0ReadSel  : std_logic;
  signal Counter0WriteSel : std_logic;
  signal CCR0WriteSel     : std_logic;
  signal CCR0ReadSel      : std_logic;
  signal Counter1ReadSel  : std_logic;
  signal Counter1WriteSel : std_logic;
  signal CCR1WriteSel     : std_logic;
  signal CCR1ReadSel      : std_logic;
  signal ENASel           : std_logic;

  signal PWMSelects           : std_logic_vector (naxis -1 downto 0);
  signal DirSelects           : std_logic_vector (naxis -1 downto 0);
  signal Counter0ReadSelects  : std_logic_vector (naxis -1 downto 0);
  signal Counter0WriteSelects : std_logic_vector (naxis -1 downto 0);
  signal CCR0WriteSelects     : std_logic_vector (naxis -1 downto 0);
  signal CCR0ReadSelects      : std_logic_vector (naxis -1 downto 0);
  signal Counter1ReadSelects  : std_logic_vector (naxis -1 downto 0);
  signal Counter1WriteSelects : std_logic_vector (naxis -1 downto 0);
  signal CCR1WriteSelects     : std_logic_vector (naxis -1 downto 0);
  signal CCR1ReadSelects      : std_logic_vector (naxis -1 downto 0);
  signal ENASelects           : std_logic_vector (naxis -1 downto 0);

  signal IrqDriveWriteSel : std_logic;
  signal IrqDriveReadSel  : std_logic;
  signal IRQSetSel        : std_logic;
  signal IRQClrSel        : std_logic;

  signal TimerReadSel  : std_logic;
  signal TimerWriteSel : std_logic;

  signal PortAWriteSel    : std_logic;
  signal PortAReadSel     : std_logic;
  signal PortAWriteDDRSel : std_logic;
  signal PortAReadDDRSel  : std_logic;

  signal PortBWriteSel    : std_logic;
  signal PortBReadSel     : std_logic;
  signal PortBWriteDDRSel : std_logic;
  signal PortBReadDDRSel  : std_logic;

-- trace buffer read abd write selects only for 200 K part

  signal TBReadSel        : std_logic;
  signal TBWriteSel       : std_logic;

  signal StatusReg         : std_logic_vector (1 downto 0);
  signal ClearBusyFlag     : std_logic;
  signal HostBusyFlag      : std_logic;
  signal SampleRate        : std_logic;
  signal RefCountBus       : std_logic_vector (7 downto 0);
  signal PreScaleWriteSel  : std_logic;
  signal PostScaleWriteSel : std_logic;
  signal ClearHostData     : std_logic;

  component Sweet16cpu is
    port (
      clk      : in  std_logic;
      reset    : in  std_logic;
      iabus    : out std_logic_vector(11 downto 0);
      idbus    : in  std_logic_vector(15 downto 0);
      mradd    : out std_logic_vector(10 downto 0);
      mwadd    : out std_logic_vector(10 downto 0);
      mibus    : in  std_logic_vector(15 downto 0);
      mobus    : out std_logic_vector(15 downto 0);
      mwrite   : out std_logic;
      carryflg : out std_logic
      );
  end component Sweet16cpu;

  component dpblockram1024x16 is
    port (
      clk  : in  std_logic;
      we   : in  std_logic;
      adda : in  std_logic_vector(9 downto 0);
      addb : in  std_logic_vector(9 downto 0);
      din  : in  std_logic_vector(15 downto 0);
      outa : out std_logic_vector(15 downto 0);
      outb : out std_logic_vector(15 downto 0));
  end component dpblockram1024x16;

  component dmcram
    port (
      addra : in  std_logic_vector(9 downto 0);
      addrb : in  std_logic_vector(9 downto 0);
      clka  : in  std_logic;
      clkb  : in  std_logic;
      dina  : in  std_logic_vector(15 downto 0);
      dinb  : in  std_logic_vector(15 downto 0);
      douta : out std_logic_vector(15 downto 0);
      doutb : out std_logic_vector(15 downto 0);
      wea   : in  std_logic;
      web   : in  std_logic);
  end component dmcram;

  component dpblockram256x16 is
    port (
      clk  : in  std_logic;
      we   : in  std_logic;
      adda : in  std_logic_vector(7 downto 0);
      addb : in  std_logic_vector(7 downto 0);
      din  : in  std_logic_vector(15 downto 0);
      outa : out std_logic_vector(15 downto 0);
      outb : out std_logic_vector(15 downto 0));
  end component dpblockram256x16;

  component CommandReg is
    port (
      clk              : in  std_logic;
      procibus         : in  std_logic_vector (15 downto 0);
      procobus         : out std_logic_vector (15 downto 0);
      procreadcommand  : in  std_logic;
      procwritestatus  : in  std_logic;
      procreadstatus   : in  std_logic;
      hostibus         : in  std_logic_vector (15 downto 0);
      hostobus         : out std_logic_vector (15 downto 0);
      hostreadcommand  : in  std_logic;
      hostwritecommand : in  std_logic;
      clrbusyflag      : in  std_logic;
      busyflag         : out std_logic
      );
  end component CommandReg;

  component DataReg is
    port (
      clk           : in  std_logic;
      procibus      : in  std_logic_vector (15 downto 0);
      procobus      : out std_logic_vector (15 downto 0);
      procradd      : in  std_logic;
      procwadd      : in  std_logic;
      procreaddata  : in  std_logic;
      procwritedata : in  std_logic;
      hostibus      : in  std_logic_vector (15 downto 0);
      hostobus      : out std_logic_vector (15 downto 0);
      hostreadadd   : in  std_logic;
      hostwriteadd  : in  std_logic;
      hostreaddata  : in  std_logic;
      hostwritedata : in  std_logic;
      hostdataclear : in  std_logic
      );
  end component DataReg;

--      component mul16x32 is
--      Port ( 
--              m: in STD_LOGIC_VECTOR (15 downto 0);
--              p: out STD_LOGIC_VECTOR (15 downto 0);
--              mlatch: in STD_LOGIC;
--              readm: in STD_LOGIC;
--              readp: in STD_LOGIC;
--              selradd: in STD_LOGIC;
--              selwadd: in STD_LOGIC;
--              clk: in STD_LOGIC
--               );
--      end component mul16x32;

--  component mul16s32 is
--    port (
--      m       : in  std_logic_vector (15 downto 0);
--      p       : out std_logic_vector (15 downto 0);
--      mlatch  : in  std_logic;
--      readm   : in  std_logic;
--      selradd : in  std_logic_vector(2 downto 0);
--      selwadd : in  std_logic;
--      clk     : in  std_logic
--      );
--  end component mul16s32;


  component mul16a32 is
    port (
      m       : in  std_logic_vector (15 downto 0);
      p       : out std_logic_vector (15 downto 0);
      mlatch  : in  std_logic;
      readm   : in  std_logic;
      selradd : in  std_logic_vector(2 downto 0);
      selwadd : in  std_logic_vector(2 downto 0);
      clk     : in  std_logic
      );
  end component mul16a32;
  
  component alignbox is
    port (
      din     : in  std_logic_vector (15 downto 0);
      dout    : out std_logic_vector (15 downto 0);
      writes  : in  std_logic;
      reads   : in  std_logic;
      selradd : in  std_logic_vector (1 downto 0);
      selwadd : in  std_logic_vector (1 downto 0);
      clk     : in  std_logic
      );
  end component alignbox;

  component dcounter is
    port (
      obus            : out std_logic_vector (15 downto 0);
      ibus            : in  std_logic_vector (7 downto 0);
      quada           : in  std_logic;
      quadb           : in  std_logic;
      index           : in  std_logic;
      ccrloadcmd      : in  std_logic;
      ccrreadcmd      : in  std_logic;
      countoutreadcmd : in  std_logic;
      countlatchcmd   : in  std_logic;
      clk             : in  std_logic
      );
  end component dcounter;

  component pwmgen is
    port (
      clk        : in  std_logic;
      refcount   : in  std_logic_vector (7 downto 0);
      ibus       : in  std_logic_vector (15 downto 0);
      loadpwmval : in  std_logic;
      loadDirCmd : in  std_logic;
      loadEnaCmd : in  std_logic;
      pwmout     : out std_logic;
      dirout     : out std_logic;
      enaout     : out std_logic
      );
  end component pwmgen;

  component pwmref is
    port (
      clk            : in  std_logic;
      refcount       : out std_logic_vector (7 downto 0);
      samplegen      : out std_logic;
      ibus           : in  std_logic_vector (15 downto 0);
      prescaleload   : in  std_logic;
      postscaleload  : in  std_logic;
      clearsamplegen : in  std_logic
      );
  end component pwmref;

  component twelveport is
    port (
      clear    : in  std_logic;
      clk      : in  std_logic;
      ibus     : in  std_logic_vector (11 downto 0);
      obus     : out std_logic_vector (11 downto 0);
      loadport : in  std_logic;
      loadddr  : in  std_logic;
      readddr  : in  std_logic;
      portdata : out std_logic_vector (11 downto 0)
      );
  end component twelveport;

  component twelveiorb is
    port (
      obus     : out std_logic_vector (11 downto 0);
      readport : in  std_logic;
      portdata : in  std_logic_vector (11 downto 0));
  end component twelveiorb;

  component IrqDrive is
    port (
      clear    : in  std_logic;
      clk      : in  std_logic;
      ibus     : in  std_logic_vector (7 downto 0);
      obus     : out std_logic_vector (7 downto 0);
      loadport : in  std_logic;
      readport : in  std_logic;
      setirq   : in  std_logic;
      clrirq   : in  std_logic;
      irqs     : out std_logic_vector (15 downto 3)
      );
  end component IrqDrive;

  component timer is
    port (
      obus          : out std_logic_vector (15 downto 0);
      timerreadcmd  : in  std_logic;
      timerlatchcmd : in  std_logic;
      timerclearcmd : in  std_logic;
      clk           : in  std_logic
      );
  end component timer;

--    component TraceBuffer1K is
--    Port ( 
--	  clk : in std_logic;
--      ibus : in std_logic_vector(15 downto 0);
--      obus : out std_logic_vector(15 downto 0);
--      selradd : in std_logic_vector(1 downto 0);
--      selwadd : in std_logic_vector(1 downto 0);
--	  readsel : in std_logic;
--	  writesel : in std_logic);
--  end component TraceBuffer1K;

begin  -- the whole ball-o-wax

  DRQ <= "ZZ";

--      DIS <= '1';                     -- uncomment to disable config decode
  DIS <= '0';                           -- uncomment to enable config decode


--      MEMCS16 <= 'Z';
--      IOCHRDY <= 'Z'

-- first the components:

--      ProgROM: dpblockram1024x16 port map (
--      clk => SynClk,
--              we  => RomWrite,
--              adda => ISAadd(9 downto 0),
--              addb => ia(9 downto 0),
--              din => DBLatch,
--              outa => idISARB,
--              outb => id
--              ); 
  
  ProgROM : dmcram port map(
    addra => ISAadd(9 downto 0),
    addrb => ia(9 downto 0),
    clka  => SynClk,
    clkb  => SynClk,
    dina  => DBlatch,
    dinb  => x"0000",
    douta => idISARB,
    doutb => id,
    wea   => RomWrite,
    web   => '0'
    );


  PagedDataRAM : dpblockram1024x16 port map (
    clk  => SynClk,
    we   => pwrite,
    adda => PagedWriteAddress,
    addb => PagedReadAddress,
    din  => mo,
    outb => mipram
    );

  GlobalDataRAM : dpblockram256x16 port map (
    clk  => SynClk,
    we   => gwrite,
    adda => mwa(7 downto 0),
    addb => mra(7 downto 0),
    din  => mo,
    outb => migram
    );  

  CPU : Sweet16cpu port map (
    clk      => SynClk,
    reset    => DBreset,
    iabus    => ia,
    idbus    => id,
    mradd    => mra,
    mwadd    => mwa,
    mibus    => mi,
    mobus    => mo,
    mwrite   => mw,
    carryflg => dbcarry
    );

  TheCommandReg : commandreg port map (
    clk              => synclk,
    procibus         => mo,
    procobus         => mi,
    procreadcommand  => CommandRegReadSel,
    procwritestatus  => HStatusRegWriteSel,
    procreadstatus   => HStatusRegreadSel,
    hostibus         => DBLatch,
    hostobus         => DBus,
    hostreadcommand  => HostReadCommandSel,
    hostwritecommand => HostWriteCommandSel,
    clrbusyflag      => ClearBusyFlag,
    busyflag         => HostBusyFlag
    );

  TheDataReg : Datareg port map (
    clk           => synclk,
    procibus      => mo,
    procobus      => mi,
    procradd      => ioradd(0),
    procwadd      => mwa(0),
    procreaddata  => DataRegReadSel,
    procwritedata => DataRegWriteSel,
    hostibus      => DBLatch,
    hostobus      => DBus,
    hostreadadd   => SA(1),
    hostwriteadd  => SALatch,
    hostreaddata  => HostReadDataSel,
    hostwritedata => HostWriteDataSel,
    hostdataclear => ClearHostData
    );

--      TheMultiplier: mul16x32 port map ( 
--              m => mo,
--              p => mi,
--              mlatch => MregWriteSel,
--              readm => MRegReadSel,
--              readp => PRegReadSel,
--              selradd => ioradd(0),
--              selwadd => mwa(0),
--              clk => synclk
--               );
--  TheMultiplier : mul16s32 port map (
--    m       => mo,
--    p       => mi,
--    mlatch  => MregWriteSel,
--    readm   => MRegReadSel,
--    selradd => ioradd(2 downto 0),
--    selwadd => mwa(0),
--    clk     => synclk
--    );

  TheMultiplier : mul16a32 port map (
    m       => mo,
    p       => mi,
    mlatch  => MregWriteSel,
    readm   => MRegReadSel,
    selradd => ioradd(2 downto 0),
    selwadd => mwa(2 downto 0),
    clk     => synclk
    );

  thealignbox : AlignBox port map (
    din     => mo,
    dout    => mi,
    writes  => AlignWriteSel,
    reads   => AlignReadSel,
    selradd => ioradd(1 downto 0),
    selwadd => mwa(1 downto 0),
    clk     => synclk
    );

  makepcounters : for i in 0 to naxis -1 generate
    counterx : dcounter port map (
      obus            => mi,
      ibus            => mo(7 downto 0),
      quada           => A(i),
      quadb           => B(i),
      index           => Idx(i),
      ccrloadcmd      => CCR0WriteSelects(i),
      ccrreadcmd      => CCR0ReadSelects(i),
      countoutreadcmd => Counter0ReadSelects(i),
      countlatchcmd   => Counter0WriteSelects(i),
      clk             => SynClk
      );        
  end generate;

  counters0 : dcounter port map (
    obus            => mi,
    ibus            => mo(7 downto 0),
    quada           => PortBData(8),
    quadb           => PortBData(9),
    index           => PortBData(6),
    ccrloadcmd      => CCR1WriteSelects(0),
    ccrreadcmd      => CCR1ReadSelects(0),
    countoutreadcmd => Counter1ReadSelects(0),
    countlatchcmd   => Counter1WriteSelects(0),
    clk             => SynClk
    );  
  counters1 : dcounter port map (
    obus            => mi,
    ibus            => mo(7 downto 0),
    quada           => PortBData(10),
    quadb           => PortBData(11),
    index           => PortBData(7),
    ccrloadcmd      => CCR1WriteSelects(1),
    ccrreadcmd      => CCR1ReadSelects(1),
    countoutreadcmd => Counter1ReadSelects(1),
    countlatchcmd   => Counter1WriteSelects(1),
    clk             => SynClk
    );  
  counters2 : dcounter port map (
    obus            => mi,
    ibus            => mo(7 downto 0),
    quada           => PortAData(8),
    quadb           => PortAData(9),
    index           => PortAData(6),
    ccrloadcmd      => CCR1WriteSelects(2),
    ccrreadcmd      => CCR1ReadSelects(2),
    countoutreadcmd => Counter1ReadSelects(2),
    countlatchcmd   => Counter1WriteSelects(2),
    clk             => SynClk
    );  
  counters3 : dcounter port map (
    obus            => mi,
    ibus            => mo(7 downto 0),
    quada           => PortAData(10),
    quadb           => PortAData(11),
    index           => PortAData(7),
    ccrloadcmd      => CCR1WriteSelects(3),
    ccrreadcmd      => CCR1ReadSelects(3),
    countoutreadcmd => Counter1ReadSelects(3),
    countlatchcmd   => Counter1WriteSelects(3),
    clk             => SynClk
    );  


  makepwms : for i in 0 to naxis-1 generate
    pwmgenx : pwmgen port map (
      clk        => SynClk,
      refcount   => RefCountBus,
      ibus       => mo,
      loadpwmval => PWMSelects(i),
      loadDirCmd => DIRSelects(i),
      loadEnaCmd => ENASelects(i),
      pwmout     => PWM(i),
      dirout     => Dir(i),
      enaout     => Ena(i)
      );
  end generate;

  pwmrefcount : pwmref port map (
    clk            => SynClk,
    refcount       => RefCountBus,
    SampleGen      => SampleRate,
    ibus           => mo,
    prescaleload   => PrescaleWriteSel,
    postscaleload  => PostScaleWriteSel,
    clearsampleGen => StatusRegWriteSel
    ); 

  TheIrqDrive : IrqDrive port map (
    clear    => RSTDRV,
    clk      => SynClk,
    ibus     => mo(7 downto 0),
    obus     => mi(7 downto 0),
    loadport => IrqDriveWriteSel,
    readport => IrqDriveReadSel,
    setirq   => IRQSetSel,
    clrirq   => IRQClrSel,
    irqs     => IRQ
    );

  Thecycletimer : timer port map (
    obus          => mi,
    timerreadcmd  => TimerReadSel,
    timerlatchcmd => TimerWriteSel,
    timerclearcmd => SampleRate,
    clk           => SynClk
    );

  porta : twelveport port map (
    clear    => RSTDRV,
    clk      => synclk,
    ibus     => mo(11 downto 0),
    obus     => mi(11 downto 0),
    loadport => PortAWriteSel,
    loadddr  => PortAWriteDDRSel,
    readddr  => PortAReadDDRSel,
    portdata => portAData
    );  

  portarb : twelveiorb port map (
    obus     => mi(11 downto 0),
    readport => PortAReadSel,
    portdata => portAData
    );

  portb : twelveport port map (
    clear    => RSTDRV,
    clk      => synclk,
    ibus     => mo(11 downto 0),
    obus     => mi(11 downto 0),
    loadport => PortBWriteSel,
    loadddr  => PortBWriteDDRSel,
    readddr  => PortBReadDDRSel,
    portdata => portBData
    );  

  portbrb : twelveiorb port map (
    obus     => mi(11 downto 0),
    readport => PortBReadSel,
    portdata => portBData
    );
-- Not enough blockRAMs/room for tarcebuffer in 100K part
--	atracebuffer : TraceBuffer1K port map (
--    clk		=> synclk,
--    ibus    => mo,
--    obus	=> mi,
--    selradd => ioradd(1 downto 0),
--    selwadd => mwa(1 downto 0),   
--	readsel => TBReadSel,
--	writesel => TBWriteSel
--    );
-- Then the processes

-- first the PC104/ISA bus decodes

  CSelect : process (AEN, CardSelect, IORD, IOWR, SA)
  begin
    if SA(9 downto 4) = Decode and AEN = '0' then
      CardSelect <= '1';
    else
      CardSelect <= '0';
    end if;

    if SA(3 downto 1) = "000" then
      WordAccess <= '0';
    else
      WordAccess <= '1';
    end if;

    if SA(3 downto 1) = "001" and CardSelect = '1' and IORD = '0' then
      HostReadCommandSel <= '1';
    else
      HostReadCommandSel <= '0';
    end if;

    if SA(3 downto 1) = "001" and CardSelect = '1' and IOWR = '0' then
      HostWriteCommandSel <= '1';
    else
      HostWriteCommandSel <= '0';
    end if;

    if SA(3 downto 2) = "01" and CardSelect = '1' and IORD = '0' then
      HostReadDataSel <= '1';
    else
      HostReadDataSel <= '0';
    end if;

    if SA(3 downto 2) = "01" and CardSelect = '1' and IOWR = '0' then
      HostWriteDataSel <= '1';
    else
      HostWriteDataSel <= '0';
    end if;

    if SA(3 downto 0) = "1000" and CardSelect = '1' then
      ROMAddrRegSel <= '1';
    else
      ROMAddrRegSel <= '0';
    end if;

    if SA(3 downto 0) = "1010" and CardSelect = '1' then
      ROMDataSel <= '1';
    else
      ROMDataSel <= '0';
    end if;

    if SA(3 downto 0) = "1110" and CardSelect = '1' then
      ResetRegSel <= '1';
    else
      ResetRegSel <= '0';
    end if;
  end process;

  LatchDin : process (SynClk)
  begin
    if SynClk'event and SynClk = '1' then
      if IOWR = '0' and CardSelect = '1' and RSTDrv = '0' then
        DBLatch    <= PreDBLatch;
        PreDBLatch <= SD;
        SALatch    <= PreSALatch;
        PreSALatch <= SA(1);  -- we need to latch SA1 for host data write
        -- as the local write actually happens after
        -- the ISA bus cycle is over            

      end if;
    end if;
  end process;

  GenIOCS : process (CardSelect, WordAccess)
  begin
    if (CardSelect = '1') and WordAccess = '1' then  -- address 4 thru 7 are 16 bit
      IOCS16 <= '0';
    else
      IOCS16 <= 'Z';
    end if;
  end process GenIOCS;

  SDDrivers : process (CardSelect, DBus, IORD, WordAccess)
  begin
    if (CardSelect = '1') and IORD = '0' then
      if WordAccess = '1' then
        SD <= DBus;
      else
        SD(7 downto 0)  <= DBus(7 downto 0);
        SD(15 downto 8) <= "ZZZZZZZZ";
      end if;
    else
      SD <= "ZZZZZZZZZZZZZZZZ";
    end if;
  end process SDDrivers;

  localbuffer : process (CardSelect, IORD)
  begin
    
    if CardSelect = '1' then
      LBE <= '0';
    else
      LBE <= '1';
    end if;

    if (CardSelect = '1') and (IORD = '0') then
      LBDIR <= '0';
    else
      LBDIR <= '1';
    end if;
  end process;

  Addpoint0 : process
    (SynClk)
  begin
    if SynClk'event and SynClk = '1' then
      if (IOWR = '0' and ROMAddrRegSel = '1') then
        addpointwr1 <= '1';
      else addpointwr1 <= '0';
      end if;
      addpointwr2      <= addpointwr1;
      if addpointwr1 = '0' and addpointwr2 = '1' then
        ISAadd <= DBLatch(10 downto 0);
      end if;
    end if;
  end process;

  Businterface : process
    (
      IORD, ROMDataSel, SynClk, idISARB)
  begin
    if SynClk'event and SynClk = '1' then
      if (IOWR = '0' and ROMDataSel = '1') then
        RomDatawr1 <= '1';
      else
        RomDatawr1 <= '0';
      end if;
      ROMDatawr2 <= ROMDatawr1;

      if (IOWR = '0' and ResetRegSel = '1') then
        ResetRegwr1 <= '1';
      else
        ResetRegwr1 <= '0';
      end if;

      ResetRegwr2 <= ResetRegwr1;
      if ResetRegwr1 = '0' and ResetRegwr2 = '1' then
        DBReset <= DBLatch(0);
      end if;

      if ROMDatawr1 = '0' and ROMDatawr2 = '1' then
        ROMWrite <= '1';
      else
        ROMWrite <= '0';
      end if;
      
    end if;


    DBus <= "ZZZZZZZZZZZZZZZZ";
    if (IORD = '0' and ROMDataSel = '1') then
      DBus <= idISARB;
    end if;
    
  end process;


  procdecode : process
    (ioradd, iordec, iorsel, iowdec, iowsel, migram, mipram, mw, mwa, synclk)

  begin
-- I/O read addresses need address latch (to match RAM)
    if synclk'event and synclk = '1' then
      ioradd <= mra(9 downto 0);
    end if;

-- first the paged (0-FFh) and global (100h-1FFh) memory decode
    case ioradd(9 downto 8) is          -- read data mux
      when b"00"  => mi <= mipram;
      when b"01"  => mi <= migram;
      when others => mi <= "ZZZZZZZZZZZZZZZZ";
    end case;

-- first 256 words of mem are paged(by axis) RAM
    if mwa(9 downto 8) = b"00" and mw = '1' then
      pwrite <= '1';
    else
      pwrite <= '0';
    end if;

-- second 256 words of mem are global RAM
    if mwa(9 downto 8) = b"01" and mw = '1' then
      gwrite <= '1';
    else
      gwrite <= '0';
    end if;

-- then the I/O decodes - I/O is from 200h up                   
    if iorsel = '1' and iordec = x"00" then
      CommandRegReadSel <= '1';
    else
      CommandRegReadSel <= '0';
    end if;

    if iowsel = '1' and iowdec = x"01" and mw = '1' then
      ClearBusyFlag <= '1';
    else
      ClearBusyFlag <= '0';
    end if;

    if iorsel = '1' and iordec(7 downto 1) = b"0000001" then
      DataRegReadSel <= '1';
    else
      DataRegReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec(7 downto 1) = b"0000001" and mw = '1' then
      DataRegWriteSel <= '1';
    else
      DataRegWriteSel <= '0';
    end if;

    if iorsel = '1' and iordec = x"04" then
      AxisRegReadSel <= '1';
    else
      AxisRegReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"04" and mw = '1' then
      AxisRegWriteSel <= '1';
    else
      AxisRegWriteSel <= '0';
    end if;

    if iorsel = '1' and iordec = x"05" then
      StatusRegReadSel <= '1';
    else
      StatusRegReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"05" and mw = '1' then
      StatusRegWriteSel <= '1';
    else
      StatusRegWriteSel <= '0';
    end if;



    if iorsel = '1' and iordec = x"06" then
      Counter0ReadSel <= '1';
    else
      Counter0ReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"06" and mw = '1' then
      Counter0WriteSel <= '1';
    else
      Counter0WriteSel <= '0';
    end if;

    if iorsel = '1' and iordec = x"07" then
      CCR0ReadSel <= '1';
    else
      CCR0ReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"07" and mw = '1' then
      CCR0WriteSel <= '1';
    else
      CCR0WriteSel <= '0';
    end if;

    if iorsel = '1' and iordec = x"08" then
      Counter1ReadSel <= '1';
    else
      Counter1ReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"08" and mw = '1' then
      Counter1WriteSel <= '1';
    else
      Counter1WriteSel <= '0';
    end if;

    if iorsel = '1' and iordec = x"09" then
      CCR1ReadSel <= '1';
    else
      CCR1ReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"09" and mw = '1' then
      CCR1WriteSel <= '1';
    else
      CCR1WriteSel <= '0';
    end if;

    if iowsel = '1' and iowdec = x"0C" and mw = '1' then
      PreScaleWriteSel <= '1';
    else
      PreScaleWriteSel <= '0';
    end if;

    if iowsel = '1' and iowdec = x"0D" and mw = '1' then
      PostScaleWriteSel <= '1';
    else
      PostScaleWriteSel <= '0';
    end if;

    if iowsel = '1' and iowdec = x"0E" and mw = '1' then
      PWMSel <= '1';
    else
      PWMSel <= '0';
    end if;

    if iowsel = '1' and iowdec = x"0F" and mw = '1' then
      DirSel <= '1';
    else
      DirSel <= '0';
    end if;

    if iowsel = '1' and iowdec = x"10" and mw = '1' then
      EnaSel <= '1';
    else
      EnaSel <= '0';
    end if;

    if iorsel = '1' and iordec = x"11" then
      TimerReadSel <= '1';
    else
      TimerReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"11" and mw = '1' then
      TimerWriteSel <= '1';
    else
      TimerWriteSel <= '0';
    end if;



    if iorsel = '1' and iordec = x"13" then
      HStatusRegReadSel <= '1';
    else
      HStatusRegReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"13" and mw = '1' then
      HStatusRegWriteSel <= '1';
    else
      HStatusRegWriteSel <= '0';
    end if;

    if iorsel = '1' and iordec(7 downto 2) = b"000101" then
      AlignReadSel <= '1';
    else
      AlignReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec(7 downto 2) = b"000101" and mw = '1' then
      AlignWriteSel <= '1';
    else
      AlignWriteSel <= '0';
    end if;

    if iorsel = '1' and iordec = x"18" then
      PortAReadSel <= '1';
    else
      PortAReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"18" and mw = '1' then
      PortAWriteSel <= '1';
    else
      PortAWriteSel <= '0';
    end if;

    if iorsel = '1' and iordec = x"19" then
      PortAReadDDRSel <= '1';
    else
      PortAReadDDRSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"19" and mw = '1' then
      PortAWriteDDRSel <= '1';
    else
      PortAWriteDDRSel <= '0';
    end if;

    if iorsel = '1' and iordec = x"1A" then
      PortBReadSel <= '1';
    else
      PortBReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"1A" and mw = '1' then
      PortBWriteSel <= '1';
    else
      PortBWriteSel <= '0';
    end if;

    if iorsel = '1' and iordec = x"1B" then
      PortBReadDDRSel <= '1';
    else
      PortBReadDDRSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"1B" and mw = '1' then
      PortBWriteDDRSel <= '1';
    else
      PortBWriteDDRSel <= '0';
    end if;

    if iorsel = '1' and iordec = x"28" then
      IRQDriveReadSel <= '1';
    else
      IRQDriveReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec = x"28" and mw = '1' then
      IRQDriveWriteSel <= '1';
    else
      IRQDriveWriteSel <= '0';
    end if;

    if iowsel = '1' and iowdec = x"29" and mw = '1' then
      IRQSetSel <= '1';
    else
      IRQSetSel <= '0';
    end if;

    if iowsel = '1' and iowdec = x"2A" and mw = '1' then
      IRQClrSel <= '1';
    else
      IRQClrSel <= '0';
    end if;

    if iorsel = '1' and iordec(7 downto 2) = b"001100" then
      TBReadSel <= '1';
    else
      TBReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec(7 downto 2) = b"001100" and mw = '1' then
      TBWriteSel <= '1';
    else
      TBWriteSel <= '0';
    end if;

    if iorsel = '1' and iordec(7 downto 3) = b"01000" then
      MRegReadSel <= '1';
    else
      MRegReadSel <= '0';
    end if;
    if iowsel = '1' and iowdec(7 downto 3) = b"01000" and mw = '1' then
      MRegWriteSel <= '1';
    else
      MRegWriteSel <= '0';
    end if;

  end process procdecode;

  AStatusReg : process (HostBusyFlag, SampleRate, StatusReg, StatusRegReadSel)
  begin
    if StatusRegReadSel = '1' then
      StatusReg(0) <= SampleRate;
      StatusReg(1) <= HostBusyFlag;
    else
      StatusReg <= "ZZ";
    end if;
    mi(1 downto 0) <= StatusReg;
  end process;

  AAxisReg : process (AxisReg, AxisRegRB, AxisRegReadSel, SynClk, mra, mwa)
  begin
    if SynClk'event and SynClk = '1' then
      if AxisRegWriteSel = '1' then
        AxisReg <= mo(3 downto 0);
      end if;
    end if;
    if AxisRegReadSel = '1' then
      AxisRegRB <= AxisReg;
    else
      AxisRegRB <= "ZZZZ";
    end if;
    mi(3 downto 0)                <= AxisRegRB;
    PagedReadAddress(9 downto 8)  <= AxisReg(1 downto 0);
    PagedReadAddress(7 downto 0)  <= mra(7 downto 0);
    PagedWriteAddress(9 downto 8) <= AxisReg(1 downto 0);
    PagedWriteAddress(7 downto 0) <= mwa(7 downto 0);
  end process;
  PagedIOdecode : process
    (
      AxisReg, CCR0ReadSel, CCR0WriteSel, CCR1ReadSel, CCR1WriteSel,
      Counter0ReadSel, Counter0WriteSel, Counter1ReadSel, Counter1WriteSel,
      DirSel, ENASel, PWMSel) 

  begin

    Counter0ReadSelects <= "0000";
    if Counter0ReadSel = '1' then
      case AxisReg(1 downto 0) is
        when "00"   => Counter0ReadSelects <= "0001";
        when "01"   => Counter0ReadSelects <= "0010";
        when "10"   => Counter0ReadSelects <= "0100";
        when "11"   => Counter0ReadSelects <= "1000";
        when others => Counter0ReadSelects <= "0000";
      end case;
    else Counter0ReadSelects <= "0000";
    end if;
    if Counter0WriteSel = '1' then
      case AxisReg(1 downto 0) is
        when "00"   => Counter0WriteSelects <= "0001";
        when "01"   => Counter0WriteSelects <= "0010";
        when "10"   => Counter0WriteSelects <= "0100";
        when "11"   => Counter0WriteSelects <= "1000";
        when others => Counter0WriteSelects <= "0000";
      end case;
    else Counter0WriteSelects <= "0000";
    end if;

    Counter1ReadSelects <= "0000";
    if Counter1ReadSel = '1' then
      case AxisReg(1 downto 0) is
        when "00"   => Counter1ReadSelects <= "0001";
        when "01"   => Counter1ReadSelects <= "0010";
        when "10"   => Counter1ReadSelects <= "0100";
        when "11"   => Counter1ReadSelects <= "1000";
        when others => Counter1ReadSelects <= "0000";
      end case;
    else Counter1ReadSelects <= "0000";
    end if;
    Counter1WriteSelects     <= "0000";
    if Counter1WriteSel = '1' then
      case AxisReg(1 downto 0) is
        when "00"   => Counter1WriteSelects <= "0001";
        when "01"   => Counter1WriteSelects <= "0010";
        when "10"   => Counter1WriteSelects <= "0100";
        when "11"   => Counter1WriteSelects <= "1000";
        when others => Counter1WriteSelects <= "0000";
      end case;
    else Counter1WriteSelects <= "0000";
    end if;

    PWMSelects <= "0000";
    if PWMSel = '1' then
      case AxisReg(1 downto 0) is
        when "00"   => PWMSelects <= "0001";
        when "01"   => PWMSelects <= "0010";
        when "10"   => PWMSelects <= "0100";
        when "11"   => PWMSelects <= "1000";
        when others => PWMSelects <= "0000";
      end case;
    else PWMSelects <= "0000";
    end if;

    DirSelects <= "0000";
    if DirSel = '1' then
      case AxisReg(1 downto 0) is
        when "00"   => DirSelects <= "0001";
        when "01"   => DirSelects <= "0010";
        when "10"   => DirSelects <= "0100";
        when "11"   => DirSelects <= "1000";
        when others => DirSelects <= "0000";
      end case;
    else DirSelects <= "0000";
    end if;

    CCR0ReadSelects <= "0000";
    if CCR0ReadSel = '1' then
      case AxisReg(1 downto 0) is
        when "00"   => CCR0ReadSelects <= "0001";
        when "01"   => CCR0ReadSelects <= "0010";
        when "10"   => CCR0ReadSelects <= "0100";
        when "11"   => CCR0ReadSelects <= "1000";
        when others => CCR0ReadSelects <= "0000";
      end case;
    else CCR0ReadSelects <= "0000";
    end if;

    CCR0WriteSelects <= "0000";
    if CCR0WriteSel = '1' then
      case AxisReg(1 downto 0) is
        when "00"   => CCR0WriteSelects <= "0001";
        when "01"   => CCR0WriteSelects <= "0010";
        when "10"   => CCR0WriteSelects <= "0100";
        when "11"   => CCR0WriteSelects <= "1000";
        when others => CCR0WriteSelects <= "0000";
      end case;
    else CCR0WriteSelects <= "0000";
    end if;

    CCR1ReadSelects <= "0000";
    if CCR1ReadSel = '1' then
      case AxisReg(1 downto 0) is
        when "00"   => CCR1ReadSelects <= "0001";
        when "01"   => CCR1ReadSelects <= "0010";
        when "10"   => CCR1ReadSelects <= "0100";
        when "11"   => CCR1ReadSelects <= "1000";
        when others => CCR1ReadSelects <= "0000";
      end case;
    else CCR1ReadSelects <= "0000";
    end if;

    CCR1WriteSelects <= "0000";
    if CCR1WriteSel = '1' then
      case AxisReg(1 downto 0) is
        when "00"   => CCR1WriteSelects <= "0001";
        when "01"   => CCR1WriteSelects <= "0010";
        when "10"   => CCR1WriteSelects <= "0100";
        when "11"   => CCR1WriteSelects <= "1000";
        when others => CCR1WriteSelects <= "0000";
      end case;
    else CCR1WriteSelects <= "0000";
    end if;

    if ENASel = '1' then
      case AxisReg(1 downto 0) is
        when "00"   => ENASelects <= "0001";
        when "01"   => ENASelects <= "0010";
        when "10"   => ENASelects <= "0100";
        when "11"   => ENASelects <= "1000";
        when others => ENASelects <= "0000";
      end case;
    else ENASelects <= "0000";
    end if;
  end process;

  debugleds : process
    (
      LEDLatch, SynClk)
  begin
    LEDS <= LEDLatch;
    if iorsel = '1' then
      case iordec is
        when x"76"  => mi <= x"4000";
        when x"77"  => mi <= x"2000";
        when x"78"  => mi <= x"0000";
        when x"79"  => mi <= x"0001";
        when x"7A"  => mi <= x"FFFF";
        when x"7B"  => mi <= x"8000";
--		when x"7C"  => mi <= x"";
        when x"7D"  => mi <= Naxisrb;
		when x"7E"  => mi <= Revision;
		when x"7F"  => mi <= Hardwaretype;

        when others => mi <= "ZZZZZZZZZZZZZZZZ";
      end case;
    else
      mi <= "ZZZZZZZZZZZZZZZZ";
    end if;
    if SynClk'event and SynClk = '1' then
      if iowsel = '1' and iowdec = x"70" and mw = '1' then  -- 0140h
        LEDLatch <= not mo(7 downto 0);
      end if;
    end if;
  end process;

end dataflow;
