library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TraceBuffer1K is
    Port ( clk : in std_logic;
           ibus : in std_logic_vector(15 downto 0);
           obus : out std_logic_vector(15 downto 0);
           selradd : in std_logic_vector(1 downto 0);
           selwadd : in std_logic_vector(1 downto 0);
		   readsel : in std_logic;
		   writesel : in std_logic);
end TraceBuffer1K;

architecture Behavioral of TraceBuffer1K is

component dpblockram1024x16 is 
 	port (
 	clk  : in std_logic; 
	we   : in std_logic; 
	adda : in std_logic_vector(9 downto 0); 
 	addb : in std_logic_vector(9 downto 0); 
	din  : in std_logic_vector(15 downto 0); 
 	outa : out std_logic_vector(15 downto 0); 
 	outb : out std_logic_vector(15 downto 0));
end component dpblockram1024x16; 
	
	signal pushadd: std_logic_vector(9 downto 0);
	signal pushdata: std_logic_vector(15 downto 0);
	signal popadd: std_logic_vector(9 downto 0);
	signal popdata: std_logic_vector(15 downto 0);
	signal datacounter: std_logic_vector(10 downto 0);
	signal push: std_logic;  -- the write enable for our dual ported RAM
	signal stf: std_logic;
begin

	fiforam: dpblockram1024x16 port map
	(
 	clk =>  clk,
	we =>   push,
	adda => pushadd,
 	addb => popadd,
 	din =>  ibus,
 	outa => pushdata,
 	outb => popdata
	);	

	atracebuffer: process (clk,selwadd,writesel,selradd,readsel,
				    		pushadd,popadd,popdata,pushdata,datacounter)
	begin
		if clk'event and clk = '1' then
		
			if writesel = '1' and selwadd = b"00" then 	-- a push
				if (datacounter /= 1024) then 			-- always increment the data counter if not full
					datacounter <= datacounter +1;
				end if;
				if stf = '1' then						-- if stop on full, only push when space remains 
					if datacounter /= 1024 then		
						pushadd <= pushadd +'1';
					end if;
				else
					pushadd <= pushadd +1;				-- in circular buffer mode always push
					if datacounter = 1024 then
						popadd <= popadd +1;			-- and do automatic pop if full
					end if;
				end if;
			end if;				
			
			if writesel = '1' and selwadd = b"01" then 	-- a pop
				if (datacounter /= 0) then 					-- always decrement the data counter if not empty
					datacounter <= datacounter -1;
				    popadd <= popadd + '1';
				end if;
			end if;
			
			if writesel = '1' and selwadd = b"10" then 
				stf <= ibus(0);
			end if;	
	  
			if writesel = '1' and selwadd = b"11" then 
				pushadd <= "0000000000";
				popadd  <= "0000000000";
				datacounter <= "00000000000";
			end if;	
	

		end if; -- clk rise
		
		if (writesel = '1' and selwadd = b"00" and stf = '0')
 	    or (writesel = '1' and selwadd = b"00" and datacounter /=1024) then
			push <= '1';
		else
			push <= '0';
		end if;

		if	readsel = '1' then
			case selradd is
		 		when b"00" => obus <= pushdata;		-- read pushdata @0
		 		when b"01" => obus <= popdata;		-- read popdata @1	
		 		when b"10" => 						-- read stop on full mode bit @2
					obus(0) <= stf;
					obus(15 downto 1) <= b"000000000000000";
		 		when b"11" => 						-- read data counter @3					
					obus (10 downto 0) <= datacounter;
 				    obus (15 downto 11) <= b"00000";
				when others => null;
			end case;
		else 
			obus <= "ZZZZZZZZZZZZZZZZ"; 
		end if;

	end process;
end Behavioral;
