program sendconf;

uses dos;
{$I hexread}
{$I hprint}


type configfile = file;

const

BufferSize = 8192;

NCSBit           = 1;
NProgramBit      = 2;
NWriteBit        = 4;
LedBit           = 8;

var

PortBase : word;
FileName : string;
CFile : configfile;
ConfigPort : word;
StatusPort : word;

BlockBuffer: array[1..BufferSize] of byte;

function OpenForRead(var fp: configfile; name: string): boolean;
begin
  Assign(fp,Name);
  {$I-}
  Reset(fp);
  {$I+}
  OpenForRead := IOResult = 0;
end { Open };

procedure Usage;
begin
  writeln;
  writeln(' SENDCONF-C - Writes config data to 4I34');
  writeln;
  writeln(' USAGE - SENDCONF file port ');
  writeln(' Where file is a FPGA configuration file');
  writeln(' and port is a 16 bit hex port address');
  writeln;
  halt(2);
end;

procedure GetParms;
var
theport : word;
thefile : string;
  begin
  if ParamCount < 2 then Usage;
  begin
    thefile := paramStr(1);
  end;
  FileName := thefile;
  if not OpenForRead(CFile,FileName) then
  begin
    writeln('Can''t open configuration file');
    halt(2);
  end;
  if HexwordRead(ParamStr(2),theport) then
  begin
    PortBase := theport;
    if (PortBase < $0100) or (PortBase > $3f0) then
    begin
      writeln;
      HexPrint(PortBase,3);
      writeln(' is an invalid 4I34 port address');
      halt(2);
    end;
  end;
end;

procedure SetupPorts;
begin
  ConfigPort := PortBase;
  StatusPort := PortBase + 1;
end;

procedure StartProgramming;
begin
  port[StatusPort] := LEDBit or NCSBit or NWriteBit;
  { Turn on LED, set program low, no CS or write }

  if (port[StatusPort] and $01) <> $00 then
  begin
    writeln('Problem: done not low at beginning...');
    halt(2);
  end;

  port[StatusPort] := LEDBit or NProgramBit;
  { Leave led on, set program high, enable write and CS }

end;

procedure DoneProgramming;
var
cclks : byte;
wait: longint;
dummy : byte;
begin
  port[StatusPort] := LEDBit or NCSBit or NWriteBit or NProgramBit;
  { Leave LED on, no program, CS or write }

  for wait := 1 to 10 do  dummy := port[StatusPort]; { for first rev with no ext pullup on done }
  if (port[StatusPort] and $01) <> $01 then
  begin
    writeln('Problem: done not high at end...');
    halt(2);
  end;
  port[StatusPort] := NCSBit or NWriteBit or NProgramBit;
  { Turn LED off, no program, CS or write }
  for cclks := 1 to 24 do
  begin
    port[ConfigPort] := $FF; { note: at this time we have lost control }
                             { of the buffer so the data written is dont care}
  end;                       { this is just to generate the config clocks...}
  port[StatusPort] := NCSBit or NWriteBit or NProgramBit;
  { Turn LED off, no program, CS or write }
end;


procedure DoIt;
var
cbyte : byte;
bytesread,index : word;
bytecount : longint;
begin
  StartProgramming;
  writeln;
  if OpenForRead(CFile,FileName) then
  begin
    reset(CFile,1);
    while not EOF(CFile) do
    begin
      bytesread := 0;
      blockread(CFile,BlockBuffer,BufferSize,bytesread);
      write('* ');
      if bytesread > 0 then
      begin
        for index := 1 to bytesread do
        begin
          port[ConfigPort] := BlockBuffer[index];
        end;
      end;
    end;
  end
  else
  begin
    writeln('Can''t open configuration file');
    halt(2);
  end;
  DoneProgramming;
  writeln;
end;

begin
  GetParms;
  SetUpPorts;
  DoIt;
  writeln('Configuration complete!');
  halt(0);
end.
