library IEEE;
use IEEE.std_logic_1164.all;  -- defines std_logic types

entity HostMot is
  	port 
  (
     -- bus interface signals --
	BALE: in STD_LOGIC; 
	TC: in STD_LOGIC; 
	BOSC: in STD_LOGIC; 
	RSTDRV: in STD_LOGIC; 
	REFRES: in STD_LOGIC; 
	SBHE: in STD_LOGIC; 
	MASTER: in STD_LOGIC; 
	AEN: in STD_LOGIC;
	SMEMR: in STD_LOGIC;   
	SMEMW: in STD_LOGIC;  
	IORD: in STD_LOGIC;   
   IOWR: in STD_LOGIC;
--	IOCHRDY: inout STD_LOGIC; 
--	MEMCS16: inout STD_LOGIC; 
	IOCS16: inout STD_LOGIC; 
   SA: in STD_LOGIC_VECTOR (19 downto 0); 			-- address bus
 	SD: inout STD_LOGIC_VECTOR (15 downto 0); 		-- data bus
	IRQ: out STD_LOGIC_VECTOR (15 downto 3); 			-- interrupts
	DACK: in STD_LOGIC_VECTOR (7 downto 6); 			-- dma acknowledges
	DRQ: out STD_LOGIC_VECTOR (7 downto 6); 			-- dma requests
	LBE: out STD_LOGIC;
	LBDIR: out STD_LOGIC;
	SynClk: in STD_LOGIC; 

	-- per channel I/O signals --
	A: in STD_LOGIC_VECTOR (7 downto 0);
	B: in STD_LOGIC_VECTOR (7 downto 0);
	Idx: in STD_LOGIC_VECTOR (7 downto 0);
	PWM: inout STD_LOGIC_VECTOR  (7 downto 0);
	Ena: out STD_LOGIC_VECTOR  (7 downto 0);
	Dir: inout STD_LOGIC_VECTOR  (7 downto 0);
	Outbits: out STD_LOGIC_VECTOR (7 downto 0);			
	Inbits: in STD_LOGIC_VECTOR (7 downto 0);				

	-- led bits
	LEDS: out STD_LOGIC_VECTOR(5 downto 0);

	-- DIS bit: set high to disable config decode
	DIS: out STD_LOGIC

	);
end HostMot;

architecture dataflow of HostMot is

	constant counters :integer := 8;
--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100001"; -- 0x210 hex
	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100010"; -- 0x220 hex
--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100011"; -- 0x230 hex	
--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100100"; -- 0x240 hex
--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100101"; -- 0x250 hex



-- misc global signals --
	signal CardSelect: STD_LOGIC; 											-- card select decode
	signal LEDView: STD_LOGIC_VECTOR (7 downto 0);
	signal Index: STD_LOGIC_VECTOR (7 downto 0);							-- index register
	signal DBus: STD_LOGIC_VECTOR (15 downto 0);							-- internal data bus
	signal DBLatch: STD_LOGIC_VECTOR (15 downto 0);						-- latched data bus
	signal PreDBLatch: STD_LOGIC_VECTOR (15 downto 0);					-- latched data bus	


--	irq related signals
	signal IRQSource: STD_LOGIC; 
	signal IRQLatch: STD_LOGIC;
	signal IRQTSEnable: STD_LOGIC;
	signal IRQMask: STD_LOGIC;
	signal IRQSelLatch: STD_LOGIC_VECTOR (3 downto 0);
	signal MissedIRQ: STD_LOGIC; 
	signal LatchOnIRQ: STD_LOGIC;
	signal StopOnMissedIRQ: STD_LOGIC;
	signal ClearMissedIRQ: STD_LOGIC;
	signal LatchOnInterrupt: STD_LOGIC;

-- index reg signals
	signal ReadIndexCmd: STD_LOGIC; 
	signal LoadIndexCmd: STD_LOGIC; 

-- LEDView reg signals
	signal LoadLEDViewCmd: STD_LOGIC;
	signal ReadLEDViewCmd: STD_LOGIC;

-- irqdiv reg signals
	signal ReadIRQDivCmd: STD_LOGIC; 
	signal LoadIRQDivCmd: STD_LOGIC; 

-- irq sel reg signals
	signal ReadIRQSelCmd: STD_LOGIC; 
	signal LoadIRQSelCmd: STD_LOGIC; 

	signal loadGCRCmd: STD_LOGIC;
	signal LoadGMRCmd: STD_LOGIC;
	signal ReadGMRCmd: STD_LOGIC;
	signal WordAccess: STD_LOGIC;

-- counter signals --
	signal CounterReadLow: STD_LOGIC_VECTOR  (counters-1 downto 0);	-- read low 16 bits
	signal CounterReadHigh: STD_LOGIC_VECTOR (counters-1 downto 0);	-- read high 16 bits
	signal GlobalCounterEnable: STD_LOGIC;										-- enable counting
	signal GlobalCountLatchcmd: STD_LOGIC;										-- command to latch counter value	
	signal GlobalCountLatch: STD_LOGIC;											-- command + irq generated latch count
	signal CCRLoadCmds: STD_LOGIC_VECTOR (counters-1 downto 0);			-- counter control reg loads
	signal CCRReadCmds:	STD_LOGIC_VECTOR (counters-1 downto 0);		-- counter control reg reads
	signal GlobalCounterClear: STD_LOGIC;										-- clear counter

-- pwm generator signals --
	signal RefCountBus: STD_LOGIC_VECTOR (10 downto 0);
	signal LoadPWM: STD_LOGIC_VECTOR  (counters-1 downto 0);
	signal ReadPWM: STD_LOGIC_VECTOR  (counters-1 downto 0);
	signal PCRLoadCmds: STD_LOGIC_VECTOR  (counters-1 downto 0);
	signal PCRReadCmds: STD_LOGIC_VECTOR  (counters-1 downto 0);
	signal GlobalPWMEnable: STD_LOGIC;
	signal GlobalClearPWM: STD_LOGIC;
	signal GlobalClearPWMCmd: STD_LOGIC;
	signal StopPWM: STD_LOGIC;

-- decodes --
	signal LEDViewSel: STD_LOGIC;
	signal IndexSel: STD_LOGIC;	
	signal GCRSel: STD_LOGIC;	
	signal GMRSel: STD_LOGIC;
	signal CCRSel: STD_LOGIC;
	signal PCRSel: STD_LOGIC;	
	signal IRQSel: STD_LOGIC;	
	signal IRQDIVSel: STD_LOGIC;	
	signal PWMValSel: STD_LOGIC;
	signal CounterLowSel: STD_LOGIC;	
	signal CounterHighSel: STD_LOGIC;			
	
	component counter
		port ( 
		obus: out STD_LOGIC_VECTOR (15 downto 0);
		ibus: in STD_LOGIC_VECTOR (15 downto 0);
		quada: in STD_LOGIC;
		quadb: in STD_LOGIC;
		index: in STD_LOGIC;
		inbit: in STD_LOGIC;
		outbit: out STD_LOGIC;
		ccrloadcmd: in STD_LOGIC;
		ccrreadcmd: in STD_LOGIC;
		countoutreadlowcmd: in STD_LOGIC;
		countoutreadhighcmd: in STD_LOGIC;
		countlatchcmd: in STD_LOGIC;
		countclearcmd: in STD_LOGIC;
		countenable: in STD_LOGIC;
		clk: in STD_LOGIC
		);
	end component;

	component pwmgen 
		port (
		clk: in STD_LOGIC;
		refcount: in STD_LOGIC_VECTOR (10 downto 0);
		ibus: in STD_LOGIC_VECTOR (15 downto 0);
		obus: out STD_LOGIC_VECTOR (15 downto 0);
		loadpwmval: in STD_LOGIC;
		readpwmval: in STD_LOGIC;
		clearpwmval: in STD_LOGIC;
		pcrloadcmd: STD_LOGIC;
		pcrreadcmd: STD_LOGIC;
		pwmout: out STD_LOGIC;
		dirout: out STD_LOGIC;
		enablein: in STD_LOGIC;
		enableout: out STD_LOGIC
		);
	end component pwmgen;
		
	component pwmref is
   	port (
		clk: in STD_LOGIC;
		refcount: out STD_LOGIC_VECTOR (10 downto 0);
		irqgen: out STD_LOGIC;
		ibus: in STD_LOGIC_VECTOR (15 downto 0);
		obus: out STD_LOGIC_VECTOR (15 downto 0);
		irqdivload: in STD_LOGIC;
		irqdivread: in STD_LOGIC
		);
	end component pwmref;

 	component indexreg is 
		port (
		clk: in STD_LOGIC;
		ibus: in STD_LOGIC_VECTOR (15 downto 0);
		obus: out STD_LOGIC_VECTOR (15 downto 0);
		loadindex: in STD_LOGIC;
		readindex: in STD_LOGIC;
		index: out STD_LOGIC_VECTOR (7 downto 0)
		);
	end component indexreg;

	component irqselreg is 
		port (
		clk: in STD_LOGIC;
		ibus: in STD_LOGIC_VECTOR (15 downto 0);
		obus: out STD_LOGIC_VECTOR (15 downto 0);
		loadirqsel: in STD_LOGIC;
		readirqsel: in STD_LOGIC;
		irqselreg: out STD_LOGIC_VECTOR (3 downto 0);
		irqtsenablereg: out STD_LOGIC;
		irqmaskreg: out STD_LOGIC
		);
	end component irqselreg;
	
	component globalcontrolreg is
	   port (
		clk: in STD_LOGIC;
		ibus: in STD_LOGIC_VECTOR (15 downto 0);
		reset: in STD_LOGIC;
		loadgcr: in STD_LOGIC;
		ctrclear: out STD_LOGIC;
		ctrlatch: out STD_LOGIC;
		pwmclear: out STD_LOGIC
	);
	end component globalcontrolreg;

	component globalmodereg is
   	port (
		clk: in STD_LOGIC;
		ibus: in STD_LOGIC_VECTOR (15 downto 0);
		obus: out STD_LOGIC_VECTOR (15 downto 0);
		reset: in STD_LOGIC;
		loadglobalmode: in STD_LOGIC;
		readglobalmode: in STD_LOGIC;
		ctrena: out STD_LOGIC;
		pwmena: out STD_LOGIC;
		clearpwmena: in STD_LOGIC;
		loi: out STD_LOGIC;
		som: out STD_LOGIC;
		miout: out STD_LOGIC;
		miin: in STD_LOGIC
	
		);
	end component globalmodereg;
	
	begin

	DRQ <= "ZZ";
--	MEMCS16 <= 'Z';
--	IOCHRDY <= 'Z';

--	DIS <= '1';			-- uncomment to disable config decode
	DIS <= '0';			-- uncomment to enable config decode
 

	makecounters: for i in 0 to (counters -1) generate
		counterx: counter port map ( 		
		obus => DBus,
		ibus => DBLatch,
		quada => A(i),
		quadb => B(i),		
		index => Idx(i),
		inbit => Inbits(i),
		outbit => Outbits(i),
		ccrloadcmd => CCRLoadCmds(i),
		ccrreadcmd =>	CCRReadCmds(i),
		countoutreadlowcmd  => CounterReadLow(i),
		countoutreadhighcmd => CounterReadHigh(i),
		countlatchcmd => GlobalCountLatch,	
		countclearcmd => GlobalCounterClear,
		countenable => GlobalCounterEnable,
		clk => SynClk
		);	
	end generate;

	makepwmgen: for i in 0 to (counters -1) generate
		pwmgenx: pwmgen port map (
		clk => SynClk,
		refcount => RefCountBus,
		ibus => DBLatch,
		obus => DBus,
		loadpwmval => LoadPWM(i),
		readpwmval => ReadPWM(i),
		clearpwmval => GlobalClearPWM,
		pcrloadcmd => PCRLoadCmds(i),
		pcrreadcmd => PCRReadCmds(i),
		pwmout => PWM(i),
		dirout => Dir(i),
		enablein => GlobalPWMEnable,
		enableout =>Ena(i)
	);
	end generate;

	pwmrefcount: pwmref port map (
		clk => SynClk,
		refcount => RefCountBus,
		irqgen => IRQSource,
		ibus => DBlatch,
		obus => DBus,
		irqdivload => LoadIRQDivCmd,
		irqdivread => ReadIRQDivCmd
		);

	gindexreg: indexreg port map (			
		clk => SynClk,
		ibus => DBlatch,
		obus => DBus,
		loadindex => LoadIndexCmd,
		readindex => ReadIndexCmd,
		index => Index
		);

	gLedreg: indexreg port map (			
		clk => SynClk,
		ibus => DBlatch,
		obus => DBus,
		loadindex => LoadLEDViewCmd,
		readindex => ReadLEDViewCmd,
		index => LEDView
		);

	girqselreg: irqselreg port map (			
		clk => SynClk,
		ibus => DBlatch,
		obus => DBus,
		loadirqsel => LoadirqselCmd,
		readirqsel => ReadirqselCmd,
		irqselreg => IrqSelLatch,
		irqtsenablereg => IrqTSEnable,
		irqmaskreg => IRQMask
		);

	ggcontrolreg: globalcontrolreg port map (
		clk => SynClk,
		ibus => DBlatch,
		reset => RstDrv,
		loadgcr => LoadGCRCmd,
		ctrclear => GlobalCounterClear,
		ctrlatch => GlobalCountLatchCmd,
		pwmclear => GlobalClearPWMCmd
		);

	gglobalmodereg: globalmodereg port map (
		clk => SynClk,
		ibus => DBLatch,
		obus => DBus,
		reset => RstDrv,
		loadglobalmode => loadGMRCmd,
		readglobalmode => ReadGMRCmd,
		ctrena => GlobalCounterEnable,
		pwmena => GlobalPWMEnable,
		clearpwmena => StopPWM,
		loi => LatchOnInterrupt,
		som => StopOnMissedIRQ,
		miout	=> ClearMissedIRQ,
		miin => MissedIRQ
		);
			
		
	CSelect: process (SA, AEN, CardSelect) 
	begin 
		if SA(9 downto 4) = Decode and AEN = '0' then 
   		CardSelect <= '1';
		else
			CardSelect <= '0';
		end if;
		
		if SA(3) = '1' then 
			WordAccess <= '1'; 
		else 
			WordAccess <= '0';
		end if;

		if SA(3 downto 0) = "0000" and CardSelect = '1' then 
			LEDViewSel <= '1'; 
		else 
			LEDViewSel <= '0';
		end if;				

		if SA(3 downto 0) = "0010" and CardSelect = '1' then 
			IndexSel <= '1'; 
		else 
			IndexSel <= '0';
		end if;

		if SA(3 downto 0) = "0011" and CardSelect = '1' then 
			GCRSel <= '1'; 
		else 
			GCRSel <= '0';
		end if;
			
		if SA(3 downto 0) = "0100" and CardSelect = '1' then 
			GMRSel <= '1'; 
		else 
			GMRSel <= '0';
		end if;		
		
		if SA(3 downto 0) = "0101" and CardSelect = '1' then 
			PCRSel <= '1'; 
		else 
			PCRSel <= '0';
		end if;		
			
		if SA(3 downto 0) = "0110" and CardSelect = '1' then 
		IRQSel <= '1'; 
		else 
			IRQSel <= '0';
		end if;		
			
		if SA(3 downto 0) = "0111" and CardSelect = '1' then 
			IRQDivSel <= '1'; 
		else 
			IRQDivSel <= '0';
		end if;		
			
		if SA(3 downto 0) = "1000" and CardSelect = '1' then 
			PWMValSel <= '1'; 
		else 
			PWMValSel <= '0';
		end if;		
		
		if SA(3 downto 0) = "1010" and CardSelect = '1' then 
			CCRSel <= '1'; 
		else 
			CCRSel <= '0';
		end if;
								
		if SA(3 downto 0) = "1100" and CardSelect = '1' then 
			CounterLowSel <= '1'; 
		else 
			CounterLowSel <= '0';
		end if;		

		if SA(3 downto 0) = "1110" and CardSelect = '1' then 
			CounterHighSel <= '1'; 
		else 
			CounterHighSel <= '0';
		end if;			
	end process;

	LatchDin: process (SD, IOWR, CardSelect,SynClk) 
	begin 
		if SynClk'event and  SynClk = '1' then
			if IOWR = '0' and CardSelect = '1' then
				DBLatch <= PreDBLatch;
				PreDBLatch <= SD;
			end if;
		end if;	
	end process;
	
	CounterDecode: process (CounterLowSel,CounterHighSel, IORD, Index) 
	begin 
		CounterReadLow <= x"00";
 		if (CounterLowSel = '1')and (IORD = '0') then
			case Index is
				when x"00" => CounterReadLow(0) <= '1';
				when x"01" => CounterReadLow(1) <= '1';
				when x"02" => CounterReadLow(2) <= '1';
				when x"03" => CounterReadLow(3) <= '1';
				when x"04" => CounterReadLow(4) <= '1';
				when x"05" => CounterReadLow(5) <= '1';
				when x"06" => CounterReadLow(6) <= '1';
				when x"07" => CounterReadLow(7) <= '1';
				when others => CounterReadLow <= x"00";
			end case;
		end if;
		CounterReadHigh <= x"00";
 		if (CounterHighSel = '1') and (IORD = '0') then
			case Index is
				when x"00" => CounterReadHigh(0) <= '1';
				when x"01" => CounterReadHigh(1) <= '1';
				when x"02" => CounterReadHigh(2) <= '1';
				when x"03" => CounterReadHigh(3) <= '1';
				when x"04" => CounterReadHigh(4) <= '1';
				when x"05" => CounterReadHigh(5) <= '1';
				when x"06" => CounterReadHigh(6) <= '1';
				when x"07" => CounterReadHigh(7) <= '1';
				when others => CounterReadHigh  <= x"00";
			end case;
		end if;
	end process;	

	pwmvals: process (PWMValSel,IOWR, IORD, Index) 
	begin 
		LoadPWM <= x"00";
 		if (PWMValSel = '1') and (IOWR = '0') then
			case Index is
				when x"00" => LoadPWM(0) <= '1';
				when x"01" => LoadPWM(1) <= '1';
				when x"02" => LoadPWM(2) <= '1';
				when x"03" => LoadPWM(3) <= '1';
				when x"04" => LoadPWM(4) <= '1';
				when x"05" => LoadPWM(5) <= '1';
				when x"06" => LoadPWM(6) <= '1';
				when x"07" => LoadPWM(7) <= '1';
				when others => LoadPWM <= x"00";
			end case;
		end if;
		ReadPWM <= x"00";
  		if (PWMValSel = '1') and (IORD = '0') then
			case Index is
				when x"00" => ReadPWM(0) <= '1';
				when x"01" => ReadPWM(1) <= '1';
				when x"02" => ReadPWM(2) <= '1';
				when x"03" => ReadPWM(3) <= '1';
				when x"04" => ReadPWM(4) <= '1';
				when x"05" => ReadPWM(5) <= '1';
				when x"06" => ReadPWM(6) <= '1';
				when x"07" => ReadPWM(7) <= '1';
				when others => ReadPWM <= x"00";
			end case;
		end if;
	end process;	
	
	CCRegs: process (CCRSel,IOWR, IORD, Index) 
	begin 
		CCRLoadCmds <= x"00";
 		if (CCRSel = '1') and (IOWR = '0') then
			case Index is
				when x"00" => CCRLoadCmds(0) <= '1';
				when x"01" => CCRLoadCmds(1) <= '1';
				when x"02" => CCRLoadCmds(2) <= '1';
				when x"03" => CCRLoadCmds(3) <= '1';
				when x"04" => CCRLoadCmds(4) <= '1';
				when x"05" => CCRLoadCmds(5) <= '1';
				when x"06" => CCRLoadCmds(6) <= '1';
				when x"07" => CCRLoadCmds(7) <= '1';
				when others => CCRLoadCmds <= x"00";
			end case;
		end if;
		CCRReadCmds <= x"00";
  		if (CCRSel = '1') and (IORD = '0') then
			case Index is
				when x"00" => CCRReadCmds(0) <= '1';
				when x"01" => CCRReadCmds(1) <= '1';
				when x"02" => CCRReadCmds(2) <= '1';
				when x"03" => CCRReadCmds(3) <= '1';
				when x"04" => CCRReadCmds(4) <= '1';
				when x"05" => CCRReadCmds(5) <= '1';
				when x"06" => CCRReadCmds(6) <= '1';
				when x"07" => CCRReadCmds(7) <= '1';
				when others => CCRReadCmds <= x"00";
			end case;
		end if;
	end process;	

	PCRegs: process (PCRSel,IOWR, IORD, Index) 
	begin 
		PCRLoadCmds <= x"00";
 		if (PCRSel = '1') and (IOWR = '0') then
			case Index is
				when x"00" => PCRLoadCmds(0) <= '1';
				when x"01" => PCRLoadCmds(1) <= '1';
				when x"02" => PCRLoadCmds(2) <= '1';
				when x"03" => PCRLoadCmds(3) <= '1';
				when x"04" => PCRLoadCmds(4) <= '1';
				when x"05" => PCRLoadCmds(5) <= '1';
				when x"06" => PCRLoadCmds(6) <= '1';
				when x"07" => PCRLoadCmds(7) <= '1';
				when others => PCRLoadCmds <= x"00";
			end case;
		end if;
		PCRReadCmds <= x"00";
  		if (PCRSel = '1') and (IORD = '0') then
			case Index is
				when x"00" => PCRReadCmds(0) <= '1';
				when x"01" => PCRReadCmds(1) <= '1';
				when x"02" => PCRReadCmds(2) <= '1';
				when x"03" => PCRReadCmds(3) <= '1';
				when x"04" => PCRReadCmds(4) <= '1';
				when x"05" => PCRReadCmds(5) <= '1';
				when x"06" => PCRReadCmds(6) <= '1';
				when x"07" => PCRReadCmds(7) <= '1';
				when others => PCRReadCmds <= x"00";
			end case;
		end if;
	end process;	
	
	GenIOCS: process (CardSelect, WordAccess)
	begin
		if (CardSelect = '1') and WordAccess = '1' then  	-- address 4 thru 7 are 16 bit
			IOCS16 <= '0';
		else
			IOCS16 <= 'Z';
		end if;
	end process GenIOCS;	
	
	IndexDecode: process (IndexSel,IORD,IOWR)
	begin
		if IndexSel = '1' and IOWR = '0' then 
			LoadIndexCmd <= '1';
		else 
			LoadIndexCmd <= '0';
		end if;
		if IndexSel = '1' and IORD = '0' then 
			ReadIndexCmd <= '1';
		else 
			ReadIndexCmd <= '0';
		end if;
	end process IndexDecode;

	LEDViewDecode: process (IndexSel,IORD,IOWR)
	begin
		if LEDViewSel = '1' and IOWR = '0' then 
			LoadLEDViewCmd <= '1';
		else 
			LoadLEDViewCmd <= '0';
		end if;
		if LEDViewSel = '1' and IORD = '0' then 
			ReadLEDViewCmd <= '1';
		else 
			ReadLEDViewCmd <= '0';
		end if;
	end process LEDViewDecode;

	IRQSelDecode: process (IRQSel,IORD,IOWR)
	begin
		if IRQSel = '1' and IOWR = '0' then 
			LoadIRQSelCmd <= '1';
		else 
			LoadIRQSelCmd <= '0';
		end if;
		if IRQSel = '1' and IORD = '0' then 
			ReadIRQSelCmd <= '1';
		else 
			ReadIRQSelCmd <= '0';
		end if;
	end process IrqSelDecode;
	
	IRQDivDecode: process (IRQDivSel,IORD,IOWR)
	begin
		if IRQDivSel = '1' and IOWR = '0' then 
			LoadIRQDivCmd <= '1';
		else 
			LoadIRQDivCmd <= '0';
		end if;
		if IRQDivSel = '1' and IORD = '0' then 
			ReadIRQDivCmd <= '1';
		else 
			ReadIRQDivCmd <= '0';
		end if;
	end process IrqDivDecode;

	GCRDecode: process (GCRSel,IOWR)
	begin
		if GCRSel = '1' and IOWR = '0' then 
			LoadGCRCmd <= '1';
		else 
			LoadGCRCmd <= '0';
		end if;
	end process GCRDecode;

	GMRDecode: process (GMRSel,IORD,IOWR)
	begin
		if GMRSel = '1' and IOWR = '0' then 
			LoadGMRCmd <= '1';
		else 
			LoadGMRCmd <= '0';
		end if;
		if GMRSel = '1' and IORD = '0' then 
			ReadGMRCmd <= '1';
		else 
			ReadGMRCmd <= '0';
		end if;
	end process GMRDecode;

	SDDrivers: process (CardSelect, WordAccess, DBus, IORD)
	begin 
		if (CardSelect = '1') and IORD = '0' then
			if WordAccess = '1' then
				SD <= DBus;
			else 
				SD(7 downto 0) <= DBus(7 downto 0);
				SD(15 downto 8) <= "ZZZZZZZZ";
		 	end if;
		else
			SD <= "ZZZZZZZZZZZZZZZZ";			
		end if;
	end process SDDrivers;

	localbuffer: process (CardSelect, IORD)
	begin
 		
		if CardSelect = '1' then 
			LBE <= '0'; 
		else 
			LBE <= '1'; 
		end if;
		
		if (CardSelect = '1') and (IORD = '0') then
	 		LBDIR <= '0';
		else 
			LBDIR <= '1'; 
		end if;
	end process;

	irqlogic: process (CardSelect, 
							SA, 
							IRQSource, 
							IrqTSEnable, 
							IrqSelLatch, 
							IrqLatch, 
							IORD,
 						   IRQSel,
							ClearMissedIRQ,
							MissedIRQ,
							StopOnMissedIRQ,
							LatchOnInterrupt,
							GlobalCountLatchCmd,
							GlobalClearPWMCmd)
	begin		
 		if IrqSource'event and IRQsource = '1' then
			IRQLatch <= IRQMask;
			if IRQLatch = '1' then 	-- if IRQLatch is set and we get the next interrupt
				MissedIRQ <= '1';		-- set Missed IRQ latch	
			end if;			
		end if;

		if ((IRQLatch = '1') and (LatchOnInterrupt = '1')) or (GlobalCountLatchCmd = '1') then
			GlobalCountLatch <= '1';
		else
			GlobalCountLatch <= '0';
		end if;

		if ClearMissedIRQ = '1' then 
			MissedIRQ <= '0';
		end if;

		if (MissedIRQ = '1') and (StopOnMissedIRQ = '1') then
			StopPWM <= '1';
		else 
			StopPWM <= '0';
		end if;
		
		if (StopPWM = '1') or (GlobalClearPWMCmd = '1') then  -- either stop on pwm or global clear reset pwm gens
			GlobalClearPWM <= '1';
		else
			GlobalClearPWM <= '0';
		end if;
			
		if IRQSel = '1'  and IORD = '0' then  -- read to IRQSelReg clears IRQ
 	   	IRQLatch <= '0';
		end if;

		IRQ <= "ZZZZZZZZZZZZZ";
		if IRQTSEnable = '1' then
	 		case IRQSelLatch is
				when x"3" => IRQ(3) <= IRQLatch;
				when x"4" => IRQ(4) <= IRQLatch;
				when x"5" => IRQ(5) <= IRQLatch;
				when x"6" => IRQ(6) <= IRQLatch;
				when x"7" => IRQ(7) <= IRQLatch;
				when x"9" => IRQ(9) <= IRQLatch;
				when x"A" => IRQ(10) <= IRQLatch;
				when x"B" => IRQ(11) <= IRQLatch;
				when x"C" => IRQ(12) <= IRQLatch;
				when x"D" => IRQ(13) <= IRQLatch;
				when x"E" => IRQ(14) <= IRQLatch;
				when x"F" => IRQ(15) <= IRQLatch;
				when others => IRQ <= "ZZZZZZZZZZZZZ";			
			end case;
		end if;
	end process;

	LEDDrive: process (A,B,Idx,Dir,PWM,IRQLatch,LedView) 
	begin 
	
		if   LEDView = x"00" then
			LEDS(5) <= not IRQLatch;
			LEDS(4) <= not A(0);
			LEDS(3) <= not B(0);
			LEDS(2) <= not Idx(0);
			LEDS(1) <= not Dir(0);
			LEDS(0) <= not PWM(0);
		elsif LEDView = x"01" then
			LEDS(5) <= not IRQLatch;
			LEDS(4) <= not A(1);
			LEDS(3) <= not B(1);
			LEDS(2) <= not Idx(1);
			LEDS(1) <= not Dir(1);
			LEDS(0) <= not PWM(1);
		elsif LEDView = x"02" then
			LEDS(5) <= not IRQLatch;
			LEDS(4) <= not A(2);
			LEDS(3) <= not B(2);
			LEDS(2) <= not Idx(2);
			LEDS(1) <= not Dir(2);
			LEDS(0) <= not PWM(2);
		elsif LEDView = x"03" then
			LEDS(5) <= not IRQLatch;
			LEDS(4) <= not A(3);
			LEDS(3) <= not B(3);
			LEDS(2) <= not Idx(3);
			LEDS(1) <= not Dir(3);
			LEDS(0) <= not PWM(3);
		elsif LEDView = x"04" then
			LEDS(5) <= not IRQLatch;
			LEDS(4) <= not A(4);
			LEDS(3) <= not B(4);
			LEDS(2) <= not Idx(4);
			LEDS(1) <= not Dir(4);
			LEDS(0) <= not PWM(4);
		elsif LEDView = x"05" then
			LEDS(5) <= not IRQLatch;
			LEDS(4) <= not A(5);
			LEDS(3) <= not B(5);
			LEDS(2) <= not Idx(5);
			LEDS(1) <= not Dir(5);
			LEDS(0) <= not PWM(5);
		elsif LEDView = x"06" then
			LEDS(5) <= not IRQLatch;
			LEDS(4) <= not A(6);
			LEDS(3) <= not B(6);
			LEDS(2) <= not Idx(6);
			LEDS(1) <= not Dir(6);
			LEDS(0) <= not PWM(6);
		elsif LEDView = x"07" then
			LEDS(5) <= not IRQLatch;
			LEDS(4) <= not A(7);
			LEDS(3) <= not B(7);
			LEDS(2) <= not Idx(7);
			LEDS(1) <= not Dir(7);
			LEDS(0) <= not PWM(7);
		end if;
	end process leddrive;

end dataflow;

  