library IEEE;
use IEEE.std_logic_1164.all;  -- defines std_logic types

entity I34Loop is
  	port 
  (
     -- bus interface signals --
	BALE: in STD_LOGIC; 
	TC: in STD_LOGIC; 
	BOSC: in STD_LOGIC; 
	RSTDRV: in STD_LOGIC; 
	REFRES: in STD_LOGIC; 
	SBHE: in STD_LOGIC; 
	MASTER: in STD_LOGIC; 
	AEN: in STD_LOGIC;
	SMEMR: in STD_LOGIC;   
	SMEMW: in STD_LOGIC;  
	IORD: in STD_LOGIC;   
   IOWR: in STD_LOGIC;
--	IOCHRDY: inout STD_LOGIC; 
--	MEMCS16: inout STD_LOGIC; 
	IOCS16: inout STD_LOGIC; 
   SA: in STD_LOGIC_VECTOR (19 downto 0); 			-- address bus
 	SD: inout STD_LOGIC_VECTOR (15 downto 0); 		-- data bus
	IRQ: out STD_LOGIC_VECTOR (15 downto 3); 			-- interrupts
	DACK: in STD_LOGIC_VECTOR (7 downto 6); 			-- dma acknowledges
	DRQ: out STD_LOGIC_VECTOR (7 downto 6); 			-- dma requests
	LBE: out STD_LOGIC;
	LBDIR: out STD_LOGIC;
	SynClk: in STD_LOGIC; 
	DIS: out STD_LOGIC;


	IOBits: inout STD_LOGIC_VECTOR (63 downto 0);			

	-- led bits
	LEDS: out STD_LOGIC_VECTOR(5 downto 0)

	);
end I34Loop;

architecture dataflow of I34Loop is

--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100001"; -- 0x210 hex
	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100010"; -- 0x220 hex
--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100011"; -- 0x230 hex
--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100100"; -- 0x240 hex
--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100101"; -- 0x250 hex

-- misc global signals --
	signal CardSelect: STD_LOGIC; 											-- card select decode				-- index register
	signal DBus: STD_LOGIC_VECTOR (15 downto 0);							-- internal data bus
	signal DBLatch: STD_LOGIC_VECTOR (15 downto 0);						-- latched data bus
	signal PreDBLatch: STD_LOGIC_VECTOR (15 downto 0);					-- latched data bus	
	signal WordAccess: STD_LOGIC;	

-- decodes --
	signal WordSel: STD_LOGIC_VECTOR(3 downto 0);	
	signal LoadPortCmd: STD_LOGIC_VECTOR(3 downto 0);
	signal ReadPortCmd: STD_LOGIC_VECTOR(3 downto 0);
	signal DDRSel: STD_LOGIC_VECTOR(3 downto 0);	
	signal LoadDDRCmd: STD_LOGIC_VECTOR(3 downto 0);
	signal ReadDDRCmd: STD_LOGIC_VECTOR(3 downto 0);		

-- components --	
	component WordPort is 
		port (
		clear: in STD_LOGIC;
		clk: in STD_LOGIC;
		ibus: in STD_LOGIC_VECTOR (15 downto 0);
		obus: out STD_LOGIC_VECTOR (15 downto 0);
		loadport: in STD_LOGIC;
		loadddr: in STD_LOGIC;
		readddr: in STD_LOGIC;
		portdata: out STD_LOGIC_VECTOR (15 downto 0)
		);
	end component WordPort;

	component WordIORB is
    port (			
	 		obus: out STD_LOGIC_VECTOR (15 downto 0);
			readport: in STD_LOGIC;
			portdata: in STD_LOGIC_VECTOR (15 downto 0) );
	end component WordIORB;

	component ledblink is
   	port (
		clk: in STD_LOGIC;
		ledx: out STD_LOGIC_VECTOR (5 downto 0)
		);
	end component ledblink;

	
	begin

	DRQ <= "ZZ";
	IRQ <= "ZZZZZZZZZZZZZ";
--	DIS <= '0';		-- uncomment to leave configuration decode on
	DIS <= '1';		-- uncomment to disable configuration decode

--	MEMCS16 <= 'Z';
--	IOCHRDY <= 'Z';
 
	gledblink: ledblink port map (
		clk => SynClk,
		ledx => LEDS
		);	
	
	makeoports: for i in 0 to 3 generate
		oportx: WordPort port map (
		clear => RSTDrv,
		clk => SynClk,
		ibus => DBLatch,
		obus => DBus,
		loadport => LoadPortCmd(i),
		loadddr => LoadDDRCmd(i),
		readddr => ReadDDRCmd(i),
		portdata => IOBits((((i+1)*16) -1) downto (i*16)) 
		);	
	end generate;

	makeiports: for i in 0 to 3 generate
		iportx: WordIORB port map (
		obus => DBus,
		readport => ReadPortCmd(i),
		portdata => IOBits((((i+1)*16) -1) downto (i*16)) 
		);	
	end generate;

	
	CSelect: process (SA, AEN, CardSelect) 
	begin 
		if SA(9 downto 4) = Decode and AEN = '0' then 
   		CardSelect <= '1';
		else
			CardSelect <= '0';
		end if;		

		WordAccess <= CardSelect; 
				
		if SA(3 downto 0) = "0000" and CardSelect = '1' then 
			WordSel(0) <= '1'; 
		else 
			WordSel(0) <= '0';
		end if;
		
		if SA(3 downto 0) = "0010" and CardSelect = '1' then 
			WordSel(1) <= '1'; 
		else 
			WordSel(1) <= '0';
		end if;
		
		if SA(3 downto 0) = "0100" and CardSelect = '1' then 
			WordSel(2) <= '1'; 
		else 
			WordSel(2) <= '0';
		end if;

		if SA(3 downto 0) = "0110" and CardSelect = '1' then 
			WordSel(3) <= '1'; 
		else 
			WordSel(3) <= '0';
		end if;	

		if SA(3 downto 0) = "1000" and CardSelect = '1' then 
			DDRSel(0) <= '1'; 
		else 
			DDRSel(0) <= '0';
		end if;
		
		if SA(3 downto 0) = "1010" and CardSelect = '1' then 
			DDRSel(1) <= '1'; 
		else 
			DDRSel(1) <= '0';
		end if;
		
		if SA(3 downto 0) = "1100" and CardSelect = '1' then 
			DDRSel(2) <= '1'; 
		else 
			DDRSel(2) <= '0';
		end if;

		if SA(3 downto 0) = "1110" and CardSelect = '1' then 
			DDRSel(3) <= '1'; 
		else 
			DDRSel(3) <= '0';
		end if;	

	end process;

	LatchDin: process (SD, IOWR, IORD, CardSelect,SynClk) 
	begin 
		if SynClk'event and  SynClk = '1' then
			if IOWR = '0' and CardSelect = '1' then
				PreDBLatch <= SD;
				DBLatch <= PreDBLatch;
			end if;
		end if;
	end process;
	

	GenIOCS: process (CardSelect, WordAccess)
	begin
		if (CardSelect = '1') and WordAccess = '1' then  	-- address 4 thru 7 are 16 bit
			IOCS16 <= '0';
		else
			IOCS16 <= 'Z';
		end if;
	end process GenIOCS;	
 
	PortDecode: process (WordSel,DDRSel,IORD,IOWR)
	begin
		for i in 0 to 3 loop                                                                                                                                                                                                                                                                                                                                                                                                                
			if WordSel(i) = '1' and IOWR = '0' then 
    			LoadPortCmd(i) <= '1';
			else 
				LoadPortCmd(i) <= '0';
			end if;
			if WordSel(i) = '1' and IORD = '0' then 
				ReadPortCmd(i) <= '1';
			else 
				ReadPortCmd(i) <= '0';
			end if;
		end loop;
	
		for i in 0 to 3 loop                                                                                                                                                                                                                                                                                                                                                                                                                
			if DDRSel(i) = '1' and IOWR = '0' then 
    			LoadDDRCmd(i) <= '1';
			else 
				LoadDDRCmd(i) <= '0';
			end if;
			if DDRSel(i) = '1' and IORD = '0' then 
				ReadDDRCmd(i) <= '1';
			else 
				ReadDDRCmd(i) <= '0';
			end if;
		end loop;	
	end process PortDecode;


	SDDrivers: process (CardSelect, WordAccess, DBus, IORD)
	begin 
		if (CardSelect = '1') and IORD = '0' then
			if WordAccess = '1' then
				SD <= DBus;
			else 
				SD(7 downto 0) <= DBus(7 downto 0);
				SD(15 downto 8) <= "ZZZZZZZZ";
		 	end if;
		else
			SD <= "ZZZZZZZZZZZZZZZZ";			
		end if;
	end process SDDrivers;

	localbuffer: process (CardSelect, IORD)
	begin
 		
		if CardSelect = '1' then 
			LBE <= '0'; 
		else 
			LBE <= '1'; 
		end if;
		
		if (CardSelect = '1') and (IORD = '0') then
	 		LBDIR <= '0';
		else 
			LBDIR <= '1'; 
		end if;
	end process;
	
end dataflow;

  