program micrdemo;
{$I micrmlow}
{$I general}
const
  BasePort = $220;

  PWMPrescale = 8;
  PWMPostScale = 6;
  SampleRate = SysClk/(PWMPreScale*256*PWMPostScale);

var
Loops : longint;

function RPMtoMicrMot(rpm: real;encounts:longint):longint;
begin

end;

procedure Startup;
begin
  Commandport := BasePort + CommandPortOffset;
  Dataport := BasePort + DataPortOffset;
  writeln('Sample Rate = ',SampleRate:4:0,' Hz');
  writeln('Sample period = ',1e6/SampleRate:3:0,' uS');
end;

procedure SetupSomeReasonableStartupValues;
begin
{ First disable the PID loop in case we were already running... }
  WriteParamWord(0,PIDOnLoc,False);
  WriteParamWord(1,PIDOnLoc,False);
  WriteParamWord(2,PIDOnLoc,False);
  WriteParamWord(3,PIDOnLoc,False);

{ and stop the profile generator }
  WriteParamWord(0,GoLoc,False);
  WriteParamWord(1,GoLoc,False);
  WriteParamWord(2,GoLoc,False);
  WriteParamWord(3,GoLoc,False);

{ initialize counters }
  WriteParamWord(0,CounterControl,CCFilter or CCCntrClr); {quad filter and clear}
  WriteParamWord(1,CounterControl,CCFilter or CCCntrClr); {quad filter and clear}
  WriteParamWord(2,CounterControl,CCFilter or CCCntrClr); {quad filter and clear}
  WriteParamWord(3,CounterControl,CCFilter or CCCntrClr); {quad filter and clear}

  WriteParamWord(0,EnaReg,1); { enable H bridge (inverting driver)}
  WriteParamWord(1,EnaReg,1); { enable H bridge (inverting driver)}
  WriteParamWord(2,EnaReg,1); { enable H bridge (inverting driver)}
  WriteParamWord(3,EnaReg,1); { enable H bridge (inverting driver)}

{ setup debug LEDS }
{ LEDS are a little weird --> you write the }
{ parameter or I/O location you wish to monitor to axis 0}
{ and the axis they should monitor to axis 1 }
  WriteParam(0,LEDLocLoc,PosCounter); {LEDS point to Encoder count}
  WriteParam(1,LEDLocLoc,1); {of axis 1}

{ Setup the samplediv register --> 4 MSBs are prescaler that determine }
{ clock rate into 8 bit PWM generation logic. 50 MHz system clock}
{ is divided by number in 4 MSBs and fed to PWM generator }
{ we use a value of 8 which gives 6.25 MHz to our PWM generator}
{ The PWM generator divides this by 256, giving a PWM rate of ~24.4
{ KHz. We then divide by the number in 4 LSBS of the samplediv}
{ register. We use 6 in this example to set the sample rate to ~4 KHz }

  WriteParamWord(0,SampleDiv,(PWMPostScale or (PWMPreScale SHL 4))); {of axis 0}

{ Set the derivative divider --> for less grainy damping }
{ This sets how many sample periods occur before deltapos is }
{ measured for velocity calculations. Likely to be most useful }
{ at high sample rates - not much use at 4KHz, so just set to 1 }
  WriteParamWord(0,DDivLoc,1);
  WriteParamWord(1,DDivLoc,1);
  WriteParamWord(2,DDivLoc,1);
  WriteParamWord(3,DDivLoc,1);

{ Next set up the filter parameters }
  WriteParamWord(0,KPLoc,500);     { Proportional term }
  WriteParamWord(1,KPLoc,500);
  WriteParamWord(2,KPLoc,500);     { Proportional term }
  WriteParamWord(3,KPLoc,500);

  WriteParamWord(0,KILoc,5);       { Integral Term  }
  WriteParamWord(1,KILoc,5);
  WriteParamWord(2,KILoc,5);       { Integral Term  }
  WriteParamWord(3,KILoc,5);

  WriteParam(0,KILLoc,25000);  { Integral limit -- basically disabled }
  WriteParam(1,KILLoc,25000);  { 10000 would be a more reasonable value }
  WriteParam(2,KILLoc,25000);  { Integral limit  }
  WriteParam(3,KILLoc,25000);

  WriteParamWord(0,KDLoc,2500);    { Derivative term }
  WriteParamWord(1,KDLoc,2500);
  WriteParamWord(2,KDLoc,2500);    { Derivative term }
  WriteParamWord(3,KDLoc,2500);

  WriteParamWord(0,KF1Loc,1000);    { feed-forward term }
  WriteParamWord(1,KF1Loc,1000);
  WriteParamWord(2,KF1Loc,1000);    { feed forward term }
  WriteParamWord(3,KF1Loc,1000);

{ Set our next position to 0 - where we are }
  WriteParam(0,NextPosLoc,0);
  WriteParam(1,NextPosLoc,0);
  WriteParam(2,NextPosLoc,0);
  WriteParam(3,NextPosLoc,0);

{ Set our desired position to 0 - where we are }
  WriteParam(0,DesPosLoc,0);
  WriteParam(1,DesPosLoc,0);
  WriteParam(2,DesPosLoc,0);
  WriteParam(3,DesPosLoc,0);

{ Set excessive position error limit }
  WriteParam(0,ExpErrorLoc,500);
  WriteParam(1,ExpErrorLoc,500);
  WriteParam(2,ExpErrorLoc,5000000); { set high so it will still "run"}
  WriteParam(3,ExpErrorLoc,5000000); { without motors attached to axis 2,3 }

{ Clear excessive position error flag just in case it was set}
  WriteParamWord(0,ExpErrorFlagLoc,0);
  WriteParamWord(1,ExpErrorFlagLoc,0);
  WriteParamWord(2,ExpErrorFlagLoc,0);
  WriteParamWord(3,ExpErrorFlagLoc,0);

{ Last but not least, enable the PID loop }
  writeParamWord(0,PIDOnLoc,True);
  writeParamWord(1,PIDOnLoc,True);
  writeParamWord(2,PIDOnLoc,True);
  writeParamWord(3,PIDOnLoc,True);

end;

begin
  Startup;
  SetupSomeReasonableStartupValues;
  Loops := 0;
  repeat
    Move(0,20000,300,120000);
    Move(1,-10000,600,80000);
    Move(2,20000,300,120000);
    Move(3,-10000,600,80000);
    WaitForMoveDoneWithStats(0);
    WaitForMoveDoneWithStats(1);
    WaitForMoveDone(2);
    WaitForMoveDone(3);
    RecoverFromExpe(0);
    RecoverFromExpe(1);
    Move(0,0,300,60000);
    Move(1,0,600,40000);
    Move(2,0,300,60000);
    Move(3,0,600,40000);
    WaitForMoveDoneWithStats(0);
    WaitForMoveDoneWithStats(1);
    RecoverFromExpe(0);
    RecoverFromExpe(1);
    Loops := Loops +1;
    writeln('End of pass ',Loops);
  until keypressed;
  halt(0);
end.
