program micrdemo;
uses crt;
{$I micrmlow}
{$I hprint}
const
  BasePort = $220;
  Sysclk = 50E6;
  PWMPrescale = 8;
  PWMPostScale = 6;

var
SampleRate : real;
Loops : longint;


procedure Startup;
begin
  Commandport := BasePort + CommandPortOffset;
  Dataport := BasePort + DataPortOffset;
  SampleRate := SysClk/(PWMPreScale*256*PWMPostScale);
  writeln('Sample Rate = ',SampleRate);
end;

procedure SetupSomeReasonableStartupValues;
begin
{ First disable the PID loop in case we were already running... }
  writeParamWord(0,PIDOnLoc,False);
  writeParamWord(1,PIDOnLoc,False);

{initialize counters }
  WriteParamWord(0,CounterControl,CCFilter or CCCntrClr); {quad filter and clear}
  WriteParamWord(1,CounterControl,CCFilter or CCCntrClr); {quad filter and clear}

{ setup debug LEDS }
{ LEDS are a little weird --> you write the }
{ parameter or I/O location you wish to monitor to axis 0}
{ and the axis they should monitor to axis 1 }
  WriteParamWord(0,LEDLocLoc,PosCounter); {LEDS point to Encoder count}
  WriteParamWord(1,LEDLocLoc,0); {of axis 0}

{ Setup the samplediv register --> 4 MSBs are prescaler that determine }
{ clock rate into 8 bit PWM generation logic. 50 MHz system clock}
{ is divided by number in 4 MSBs and fed to PWM generator }
{ we use a value of 8 which gives 6.25 MHz to our PWM generator}
{ The PWM generator divides this by 256, giving a PWM rate of ~24.4
{ KHz. We then divide by the number in 4 LSBS of the samplediv}
{ register. We use 6 in this example to set the sample rate to ~4 KHz }

  WriteParamWord(0,SampleDiv,(PWMPostScale or (PWMPreScale SHL 4))); {of axis 0}

{ Set the derivative divider --> for less grainy damping }
{ This sets how many sample periods occur before deltapos is }
{ measured for velocity calculations. Likely to be most useful }
{ at high sample rates - not much use at 4KHz, so just set to 1 }
  WriteParamWord(0,DDivLoc,1);
  WriteParamWord(1,DDivLoc,1);

{ Next set up the filter parameters }
  WriteParamWord(0,KPLoc,500);     { Proportional term }
  WriteParamWord(1,KPLoc,500);

  WriteParamWord(0,KILoc,0);       { Integral Term -- nop for now }
  WriteParamWord(1,KILoc,0);

  WriteParamWord(0,KDLoc,2500);    { Derivative term }
  WriteParamWord(1,KDLoc,2500);

{ Set our next position to 0 - where we are }
  WriteParam(0,NextPosLoc,0);
  WriteParam(1,NextPosLoc,0);

{ Set excessive position error limit }
  WriteParam(0,ExpErrorLoc,500);
  WriteParam(1,ExpErrorLoc,500);
{ Clear excessive position error flag just in case it was set}
  WriteParam(0,ExpErrorFlagLoc,0);
  WriteParam(1,ExpErrorFlagLoc,0);

{ Last but not least, enable the PID loop and set the go bits }
  writeParamWord(0,PIDOnLoc,True);
  writeParamWord(1,PIDOnLoc,True);

  writeParamWord(0,GoLoc,True);
  writeParamWord(1,GoLoc,True);
end;

procedure Move(axis : word;nextpos : longint; acc : longint; maxv : longint);
begin
  WriteParam(axis,NextPosLoc,nextpos);
  WriteParam(axis,AccelLoc,acc);
  WriteParam(axis,MaxVelLoc,maxv);
  WriteParam(axis,GoLoc,True);
end;

procedure WaitforMoveDone(axis : word);
var
temp : word;
begin
  repeat
    temp := ReadParamWord(axis,MoveLoc);
  until ((temp and $00FF) = 0);  { check low byte for done-a-tude }
end;

procedure SetHomePos(axis:word);
begin
  WriteParamWord(axis,PIDOnLoc,False); { turn off PID loop whilst futzing }
  WriteParamWord(axis,CounterControl,CCFilter or CCCntrClr); {quad filter and clear}
  WriteParam(axis,DesPosLoc,0);  { clear Desired Pos reg }
  WriteParam(axis,NextPosLoc,0); { clear Next Pos reg }
  WriteParam(axis,LastPosLoc,0); { clear Last Pos reg }
  WriteParamWord(axis,ExpErrorFlagLoc,0); { clear any expe }
  WriteParamWord(axis,PIDOnLoc,True); { re-enable PID }
end;

procedure RecoverFromExpe(axis:word);
var
realpos : longint;
begin
  if ReadParamWord(axis,ExpErrorFlagLoc) <> 0 then
  begin
    writeln('Axis ',Axis,' stopped due to position error > ',ReadParam(Axis,ExpErrorLoc));
    RealPos := ReadParam(Axis,MirrorEncLoc); { get current actual position }
    writeln('Setting DesPos = Real Position: ',RealPos:12);
    WriteParam(Axis,DesPosLoc,RealPos);  { store it into desired Pos reg }
    WriteParam(Axis,LastPosLoc,RealPos); { store it into Last Pos reg so we dont  }
                                         { so we dont get a large KD genrated thunk }
                                         { when we re-enable the PID loop }
    writeln('Clearing error and re-enabling PID on stopped channel');
    WriteParamWord(Axis,ExpErrorFlagLoc,0);
    WriteParamWord(Axis,PIDOnLoc,1);
  end;
end;

begin
  Startup;
  SetupSomeReasonableStartupValues;
  SetHomePos(0);
  SetHomePos(1);
  Loops := 0;
  repeat
    Move(0,20000,3000,120000);
    Move(1,-10000,600,80000);
    WaitForMoveDone(0);
    WaitForMoveDone(1);
    RecoverFromExpe(0);
    RecoverFromExpe(1);
    Move(0,0,3000,60000);
    Move(1,0,600,40000);
    WaitForMoveDone(0);
    WaitForMoveDone(1);
    RecoverFromExpe(0);
    RecoverFromExpe(1);
    Loops := Loops +1;
    writeln('End of pass ',Loops);
  until keypressed;
  halt(0);
end.
