var
Commandport : word;
Dataport : word;

const

WriteFlag : byte = $80;
CommandPortOffset = 2;
DataPortOffset = 4;
BusyMask : word = $8000;
True  = $FF;
False = $00;
Sysclk = 50E6;
CPUCycleTime = 2/Sysclk;


{ General motor controller parameters }

NextPosLoc      = 0;       { Next position for move: 32 bit}
MaxVelLoc	= 1;       { Maximum (slew) velocity for move: 32 bit }
AccelLoc	= 2;       { accelleration value: 32 bit}
ExPErrorLoc	= 3;       { Maximum Pos Error for shutdown: 32 bit }
ExPErrorFlagLoc	= 4;       { Position error flag: 16 bit }
MaxNegErr	= 5;       { maximum negative position error: 32 bit }
MaxPosErr	= 6;       { maximum positive position error: 32 bit }
PIDOnLoc	= 7;       { PID on switch: 16 bit }
GoLoc		= 8;       { Go Switch: 16 bit }
VelocityLoc     = 9;       { Commanded velocity for velocity mode: 32 bit }
HomeLoc	        = 10;      { home flag for seek home mode:16 bit }
DesPosLoc       = 12;      { PID loops desired position: 32 bit }
LastPosLoc	= 13;      { Previous sample measured position: 32 bit }
BreakPointLoc	= 14;      { break point value: 32 bit }
MirrorEncLoc	= 15;      { copy of encoder count: 32 bit }
IntMaskLoc	= 16;      { interrupt mask for this axis: 16 bit }
IntCauseLoc	= 17;      { bit field for interrupt causes: 16 bit }
DirLoc          = 18;      { sets system direction - default = 00: 16 bit}
LEDLocLoc	= 19;      { sets up debug LEDs: 16 bit }
IntegralLoc	= 32;      { accumulated error }

{ filter setup parameters }

KPLoc           = 20;     { KP -- Proportional constant: 16 bit }
KILoc		= 21;     { KI -- Integral constant: 16 bit }
KDLoc		= 22;     { KD -- Derivative constant: 16 bit }
KIlLoc          = 23;     { IL -- Integral Limit: 32 bit }
KF1Loc		= 24;     { Velocity feed forward term : 16 bit }
KF2Loc		= 25;
KF3Loc		= 26;
MaxDrive        = 27;     { Maximum PWM drive during move: 16 bit }
                          { this is for calculating headroom or maintanence }
DDivLoc	        = 28;     { Samples used for calculating KD: 16 bit }

{ internal use only }

ierrorLoc	= 30;
derrorLoc	= 31;
iidivLoc	= 33;
iddivLoc	= 34;
iaccelLoc	= 36;
smaxvelLoc	= 37;
deltaLoc	= 38;
ivelocityLoc	= 39;
fixupLoc	= 40;
proctimer       = 41;
debug           = 47;

{ User internal I/O locations - all 8 bits }
{ registers marked *IDX are duplicated (1 per axis)}

PosCounter     = 64;  { *IDX 32 bit position counter -- read only }
PWM            = 64;  { *IDX 8 bit pwm register -- write only }
Dir            = 68;  { *IDX 1 bit direction register = bit 7 }

CounterControl = 70;  { *IDX counter control register }
{ Counter control register bit definitions - set operational mode of counter }
CCUDMode       = $80; { U/D mode instead of quadrature mode}
CCFilter       = $40; { enable input digital filter}
CClrdFlag      = $20; { bit set if count has been cleared }
CCCOI          = $10; { Clear counter on index if set }
CCIdxPol       = $08; { polarity of index input - sets clear pol }
CCCntrClr      = $04; { reads index input - writes with this bit set clear counter }
CCInputB       = $02; { read only: reflects B/CDIR input }
CCInputA       = $01; { read only: reflects A/COUNT input }

SampleDiv      = 71;
{ Sample divider - sets PWM and sample rate }
{ 4 MSBs are prescaler before 8 bit PWM }
{ 4 LSBs are postscaler after 8 bit PWM }
{ scalers divide by programmed amount }
{ default used is $86 Hex which divides the 50 Mhz clock }
{ by 8 which is then divided by 256 to give a PWM rate of ~24.4 KHz }
{ this is then divided by 6 to give a sample rate of ~4070 Hz}
{ sample rates of up to 10 KHz should be possible if all 4 channels}
{ are used, higher sample rates are possible if fewer channels}
{ are enabled (PIDon or GO set) }

MiscBits       = 75;
{ These are only available on the 4I34 version. Not the 4I34M }
MiscBitsDDR    = 76;
{ Data direction register for above. 1 bit in register sets
{ corresponding bit in MiscBits to output mode }

EnaReg         = 77;            { *IDX  LSB is enable bit }

IrqReg         = 69;            { IRQ setting register }
{ this register should only be accessed at startup to specify }
{ whether IRQS are used and which IRQ is generated }
{ bits 0..3 specify interrupt }

IRQSelMask     = $0F;
{ bit 4 is TriState enable}
IRQTSEnable    = $10;
IRQMask        = $40;
IRQGen         = $80;   { do not touch this bit - controlled by ucont}

PortA          = 96;
PortADDR       = 97;
{ Data direction register for above. 1 bit in register sets
{ corresponding bit in PORTA to output mode }

PortB          = 98;
PortBDDR       = 99;
{ Data direction register for above. 1 bit in register sets
{ corresponding bit in PORTB to output mode }

PortC          = 100;
PortCDDR       = 101;
{ Data direction register for above. 1 bit in register sets
{ corresponding bit in PORTC to output mode }

{ These are only available on the 4I34 version. Not the 4I34M }
PortD          = 102;
PortDDDR       = 103;
{ Data direction register for above. 1 bit in register sets
{ corresponding bit in PORTD to output mode }

procedure WriteParam(theaxis: byte;theparm: byte;thedata : longint);
{ full 32 bit parameter write - 2 words }
var foo : word;
begin
  portw[DataPort] := word(thedata);
  portw[DataPort+2] := word(thedata shr 16);
  portw[CommandPort] := (theparm or WriteFlag) + (theaxis * 256);
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
end;

procedure WriteParamWord(theaxis: byte;theparm: byte;thedata : word);
{16 bit only parameter write}
var foo : word;
begin
  portw[DataPort] := word(thedata);
  portw[CommandPort] := (theparm or WriteFlag) + (theaxis * 256);
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
end;

function ReadParam(theaxis: byte;theparm: byte):longint;
{full 32 bit parameter read }
var
foo : word;
rdata : longint;
begin
  portw[CommandPort] := (theparm) + (theaxis * 256);
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
  rdata := Portw[DataPort] + (portw[DataPort+2] * 65536);
  ReadParam :=rdata;
end;

function ReadParamWord(theaxis: byte;theparm: byte):word;
{ 16 bit parameter read }
var
foo : word;
rdata : longint;
begin
  portw[CommandPort] := (theparm) + (theaxis * 256);
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
  rdata := Portw[DataPort];
  ReadParamWord :=rdata;
end;

procedure Move(axis : word;nextpos : longint; acc : longint; maxv : longint);
begin
  WriteParam(axis,MaxNegErr,0);
  WriteParam(axis,MaxPosErr,0);
  WriteParam(axis,NextPosLoc,nextpos);
  WriteParam(axis,AccelLoc,acc);
  WriteParam(axis,MaxVelLoc,maxv);
  WriteParam(axis,GoLoc,True);
end;

procedure WaitforMoveDone(axis : word);
var
temp : word;
begin
  repeat
    temp := ReadParamWord(axis,GoLoc);
  until ((temp and $00FF) = 0);  { check low byte for done-a-tude }
end;

procedure WaitforMoveDoneWithStats(axis : word);
var
temp : word;
cyctime,maxcyc : word;
begin
  { wait for motors to be done and sample loop time statistics}
  maxcyc := 0;
  repeat
    cyctime := ReadParamWord(0,ProcTimer);
    if (cyctime > maxcyc) then maxcyc := cyctime;
    temp := ReadParamWord(axis,GoLoc);
  until ((temp and $00FF) = 0);  { check low byte for done-a-tude }
  writeln('Maximum cycle time = ',maxcyc*1e6*CPUCycleTime:4:0,'uS');
  writeln('MaxNegErr = ',ReadParam(axis,MaxNegErr),' MaxPosErr = ',ReadParam(axis,MaxPosErr));

end;

procedure SetHomePos(axis:word);
begin
  WriteParamWord(axis,PIDOnLoc,False); { turn off PID loop whilst futzing }
  WriteParamWord(axis,GoLoc,False);    { turn off PID loop whilst futzing }
  WriteParamWord(axis,CounterControl,CCFilter or CCCntrClr); {quad filter and clear}
  WriteParam(axis,DesPosLoc,0);  { clear Desired Pos reg }
  WriteParam(axis,NextPosLoc,0); { clear Next Pos reg }
  WriteParam(axis,LastPosLoc,0); { clear Last Pos reg }
  WriteParamWord(axis,ExpErrorFlagLoc,0); { clear any expe }
  WriteParamWord(axis,PIDOnLoc,True); { re-enable PID }
end;

procedure SeekHomePos(axis:word;nextpos : longint; acc : longint; maxv : longint);
begin
  WriteParamWord(axis,CounterControl,CCFilter or CCCOI); {quad filter and clear-on-index}
  { You need to set the CCIdxPol bit if you want rising edge }
  { index detection instead of falling edge detection }
  WriteParamWord(axis,ExpErrorFlagLoc,0); { clear any expe }
  WriteParam(axis,ExpErrorLoc,5000000); { set error limit high }
  WriteParamWord(axis,HomeLoc,True); { set the home flag }
  Move(axis,nextpos,acc,maxv);
  WaitForMoveDone(axis);
  WriteParamWord(axis,HomeLoc,False); { now that we're done, reset the home flag }
end;

procedure EmergencyStop(axis:word;acc :longint);
begin
  WriteParam(axis,VelocityLoc,0); { set velocity to 0 }
  WriteParam(axis,AccelLoc,acc);  { set accel up to stop quick }
  WaitForMoveDone(axis);          { WARNING! this only works in position mode }
                                  { in velocity mode, this will fall through }
end;

procedure RecoverFromExpe(axis:word);
var
realpos : longint;
begin
  if ReadParamWord(axis,ExpErrorFlagLoc) <> 0 then
  begin
    writeln('Axis ',Axis,' stopped due to position error > ',ReadParam(Axis,ExpErrorLoc));
    RealPos := ReadParam(Axis,MirrorEncLoc); { get current actual position }
    writeln('Setting DesPos = Real Position: ',RealPos:12);
    WriteParam(Axis,DesPosLoc,RealPos);  { store it into desired Pos reg }
    WriteParam(Axis,LastPosLoc,RealPos); { store it into Last Pos reg so we dont  }
                                         { so we dont get a large KD generated thunk }
                                         { when we re-enable the PID loop }
    WriteParam(Axis,IntegralLoc,0);      { clear accumulated error (anti-thunk) }
    writeln('Clearing error and re-enabling PID on stopped channel');
    WriteParamWord(Axis,ExpErrorFlagLoc,0);
    WriteParamWord(Axis,PIDOnLoc,1);
  end;
end;
