var
Commandport : word;
Dataport : word;

const

WriteFlag : byte = $80;
IOFlag = $800;
CommandPortOffset = 2;
DataPortOffset = 4;
BusyMask : word = $8000;
True  = $FF;
False = $00;

{ General motor controller parameters }

NextPosLoc      = 0;
MaxVelLoc	= 1;
AccelLoc	= 2;
ExPErrorLoc	= 3;
ExPErrorFlagLoc	= 4;
MaxNegErr	= 5;
MaxPosErr	= 6;
PIDOnLoc	= 7;
GoLoc		= 8;
MoveLoc		= 9;
HomeFlagLoc	= 10;
DesPosLoc       = 12;
LastPosLoc	= 13;
BreakPointLoc	= 14;
MirrorEncLoc	= 15;
IntMaskLoc	= 16;
IntCauseLoc	= 17;
LEDLocLoc	= 19;

{ filter setup parameters }

KPLoc           = 20;
KILoc		= 21;
KDLoc		= 22;
KIlLoc          = 23;
KF1Loc		= 24;
KF2Loc		= 25;
KF3Loc		= 26;
IDivLoc	        = 27;
DDivLoc	        = 28;

{ internal use only }

ierrorLoc	= 30;
derrorLoc	= 31;
integralLoc	= 32;
iidivLoc	= 33;
iddivLoc	= 34;
halfdistLoc	= 35;
iaccelLoc	= 36;
smaxvelLoc	= 37;
deltaLoc	= 38;
velocityLoc	= 39;
fixupLoc	= 40;
proctimer       = 41;
debug           = 47;

{ User internal I/O locations - all 8 bits }
{ registers marked *IDX are duplicated (1 per axis)}

PosCounter     = 64;  { *IDX 32 bit position counter -- read only }
PWM_QUAD       = 64;  { *IDX 8 bit pwm register -- write only }
Dir            = 68;  { *IDX 1 bit direction register = bit 7 }

CounterControl = 70;  { *IDX counter control register }
{ Counter control register bit definitions - set operational mode of counter }
CCUDMode       = $80; { U/D mode instead of quadrature mode}
CCFilter       = $40; { enable input digital filter}
CClrdFlag      = $20; { bit set if count has been cleared }
CCCOI          = $10; { Clear counter on index if set }
CCIdxPol       = $08; { polarity of index input - sets clear pol }
CCCntrClr      = $04; { reads index input - writes with this bit set clear counter }
CCInputB       = $02; { read only: reflects B/CDIR input }
CCInputA       = $01; { read only: reflects A/COUNT input }

SampleDiv      = 71;
{ Sample divider - sets PWM and sample rate }
{ 4 MSBs are prescaler before 8 bit PWM }
{ 4 LSBs are postscaler after 8 bit PWM }
{ scalers divide by programmed amount }
{ default used is $86 Hex which divides 50 Mhz clock }
{ by 8 which gives a PWM rate of ~24.4 KHz }
{ and divides this by 6 to give a sample rate of ~4070 Hz}
{ sample rates of up to 10 KHz should be possible if all 4 channels}
{ are used, higher sample rates are possible if fewer channels}
{ are enabled }

MiscBits       = 75;
{ These are only available on the 4I34 version. Not the 4I34M }
MiscBitsDDR    = 76;
{ Data direction register for above. 1 bit in register sets
{ corresponding bit in MiscBits to output mode }

Ena            = 77;            { *IDX no user access needed }

IrqReg         = 69;            { IRQ setting register }
{ this register should only be accessed at startup to specify }
{ whether IRQS are used and which IRQ is generated }
{ bits 0..3 specify interrupt }

IRQSelMask     = $0F;
{ bit 4 is TriState enable}
IRQTSEnable    = $10;
IRQMask        = $40;
IRQGen         = $80;   { do not touch this bit - controlled by ucont}

PortA          = 96;
PortADDR       = 97;
{ Data direction register for above. 1 bit in register sets
{ corresponding bit in PORTA to output mode }
PortB          = 98;
PortBDDR       = 99;
{ Data direction register for above. 1 bit in register sets
{ corresponding bit in PORTB to output mode }
PortC          = 100;
PortCDDR       = 101;
{ Data direction register for above. 1 bit in register sets
{ corresponding bit in PORTC to output mode }

{ These are only available on the 4I34 version. Not the 4I34M }
PortD          = 102;
PortDDDR       = 103;
{ Data direction register for above. 1 bit in register sets
{ corresponding bit in PORTD to output mode }

procedure WriteParam(theaxis: byte;theparm: byte;thedata : longint);
{ full 32 bit parameter write - 2 words }
var foo : word;
begin
  portw[DataPort] := word(thedata);
  portw[DataPort+2] := word(thedata shr 16);
  portw[CommandPort] := (theparm or WriteFlag) + (theaxis * 256);
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
end;

procedure WriteParamWord(theaxis: byte;theparm: byte;thedata : word);
{16 bit only parameter write}
var foo : word;
begin
  portw[DataPort] := word(thedata);
  portw[CommandPort] := (theparm or WriteFlag) + (theaxis * 256);
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
end;

function ReadParam(theaxis: byte;theparm: byte):longint;
{full 32 bit parameter read }
var
foo : word;
rdata : longint;
begin
  portw[CommandPort] := (theparm) + (theaxis * 256);
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
  rdata := Portw[DataPort] + (portw[DataPort+2] * 65536);
  ReadParam :=rdata;
end;

function ReadParamWord(theaxis: byte;theparm: byte):word;
{ 16 bit parameter read }
var
foo : word;
rdata : longint;
begin
  portw[CommandPort] := (theparm) + (theaxis * 256);
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
  rdata := Portw[DataPort];
  ReadParamWord :=rdata;
end;
