library IEEE;
use IEEE.std_logic_1164.all;  -- defines std_logic types


-- latest changes crunch IO into 32 locations - abandoned but relocations stay
-- busy-flag was 24 now 14
-- mul-m was 32 now 20
-- mul-p was 36 now 24
-- hostdata was 20 now 16
-- hostcontrol was 16 now 28
-- irqreg added @ 15
-- proc cycle timer reg added @ 30
-- ram expanded to 192 bytes = 48 dblwords
 
entity MicrMot is
  	port 
  (
     -- bus interface signals --
	BALE: in STD_LOGIC; 
	TC: in STD_LOGIC; 
	BOSC: in STD_LOGIC; 
	RSTDRV: in STD_LOGIC; 
	REFRES: in STD_LOGIC; 
	SBHE: in STD_LOGIC; 
	MASTER: in STD_LOGIC; 
	AEN: in STD_LOGIC;
	SMEMR: in STD_LOGIC;   
	SMEMW: in STD_LOGIC;  
	IORD: in STD_LOGIC;   
   IOWR: in STD_LOGIC;
--	IOCHRDY: inout STD_LOGIC; 
--	MEMCS16: inout STD_LOGIC; 
	IOCS16: inout STD_LOGIC; 
   SA: in STD_LOGIC_VECTOR (19 downto 0); 			-- address bus
 	SD: inout STD_LOGIC_VECTOR (15 downto 0); 		-- data bus
	IRQ: out STD_LOGIC_VECTOR (15 downto 3); 			-- interrupts
	DACK: in STD_LOGIC_VECTOR (7 downto 6); 			-- dma acknowledges
	DRQ: out STD_LOGIC_VECTOR (7 downto 6); 			-- dma requests
	LBE: out STD_LOGIC;
	LBDIR: out STD_LOGIC;
	SynClk: in STD_LOGIC; 

	-- per channel I/O signals --
	A: in STD_LOGIC_VECTOR (3 downto 0);
	B: in STD_LOGIC_VECTOR (3 downto 0);
	Idx: in STD_LOGIC_VECTOR (3 downto 0);
	PWM: inout STD_LOGIC_VECTOR (3 downto 0);
	Ena: out STD_LOGIC_VECTOR  (3 downto 0);
	Dir: inout STD_LOGIC_VECTOR  (3 downto 0);
	MiscBits: inout STD_LOGIC_VECTOR (7 downto 0);			
			

	-- MiscIO
	IOPortAData: inout STD_LOGIC_VECTOR (7 downto 0);
	IOPortBData: inout STD_LOGIC_VECTOR (7 downto 0);
	IOPortCData: inout STD_LOGIC_VECTOR (7 downto 0);
	IOPortDData: inout STD_LOGIC_VECTOR (7 downto 0);
	

	-- led bits
	LEDS: out STD_LOGIC_VECTOR(5 downto 0);
	-- configuration decode disable
	DIS:out STD_LOGIC
	);
end MicrMot;

architecture dataflow of MicrMot is

--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100001"; -- 0x210 hex
	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100010"; -- 0x220 hex
--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100011"; -- 0x230 hex	
--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100100"; -- 0x240 hex
--	constant decode : STD_LOGIC_VECTOR (5 downto 0) := "100101"; -- 0x250 hex	

-- misc global signals --
	signal CardSelect: STD_LOGIC; 											-- card select decode
	signal Index: STD_LOGIC_VECTOR (7 downto 0);							-- index register
	signal DBus: STD_LOGIC_VECTOR (15 downto 0);							-- internal data bus
	signal DBLatch: STD_LOGIC_VECTOR (15 downto 0);						-- latched data bus
	signal PreDBLatch: STD_LOGIC_VECTOR (15 downto 0);
	signal SALatch: STD_LOGIC;
	signal PreSALatch: STD_LOGIC; 
   
-- kcpsm0 related signals
 	signal 	proc0add: STD_LOGIC_VECTOR (7 downto 0);					-- kcpsm address
 	signal 	isa0add: STD_LOGIC_VECTOR (9 downto 0);					
 	signal 	Proc0Data: STD_LOGIC_VECTOR (15 downto 0);
	signal 	proc0ibus: STD_LOGIC_VECTOR (7 downto 0);
	signal 	proc0obus: STD_LOGIC_VECTOR (7 downto 0);
	signal 	proc0IOAdd: STD_LOGIC_VECTOR (7 downto 0);
 	signal 	proc0IORD: STD_LOGIC;
 	signal 	proc0IOWR: STD_LOGIC;
	signal	ROMAddrReg0Sel: STD_LOGIC;
	signal	ROMData0Sel: STD_LOGIC;
	signal 	addpoint0wr1: STD_LOGIC;
	signal 	addpoint0wr2: STD_LOGIC;
	signal 	RomData0wr1: STD_LOGIC;
	signal	RomData0wr2: STD_LOGIC; 
	signal	Rom0Write: STD_LOGIC;
	signal	IOReadSelect0: STD_LOGIC;
	signal	IOWriteSelect0: STD_LOGIC; 
	signal 	Proc0DataRB: STD_LOGIC_VECTOR (15 downto 0);
	signal 	Proc0DataISARB: STD_LOGIC_VECTOR (15 downto 0);
	signal 	PagedProc0Add: STD_LOGIC_VECTOR (9 downto 0);
	signal 	Proc0CommandReadSelect: STD_LOGIC;
	signal 	Proc0CommandWriteSelect: STD_LOGIC;	
	signal 	Proc0DataWriteSelect: STD_LOGIC;
	signal 	Proc0DataReadSelect: STD_LOGIC;					
	signal 	Proc0ClearBusyFlag: STD_LOGIC;
	signal 	Proc0BusyFlag:	STD_LOGIC;
	signal 	HostWriteCommandSel:	STD_LOGIC;
	signal 	HostReadCommandSel: STD_LOGIC;
	signal 	HostWriteDataSel:	STD_LOGIC;
	signal 	HostReadDataSel: STD_LOGIC;
	signal 	ClearHostData: STD_LOGIC;

	signal	CounterSelect: STD_LOGIC;	
	signal	PWMSelect: STD_LOGIC;	
	signal	DirSelect: STD_LOGIC;	
	signal	CountLatchSelect: STD_LOGIC;	
	signal	CCRWriteSelect: STD_LOGIC;
	signal	CCRReadSelect: STD_LOGIC;

	signal	CounterSelects: STD_LOGIC_VECTOR (3 downto 0);	
	signal	PWMSelects: STD_LOGIC_VECTOR (3 downto 0);	
	signal	DirSelects: STD_LOGIC_VECTOR (3 downto 0);	
	signal	CountLatchSelects: STD_LOGIC_VECTOR (3 downto 0);	
	signal	CCRWriteSelects: STD_LOGIC_VECTOR (3 downto 0);
	signal	CCRReadSelects: STD_LOGIC_VECTOR (3 downto 0);

	signal 	SampleRate: STD_LOGIC;
	signal 	SampleDivSelect: STD_LOGIC;
	signal	StatusReg: STD_LOGIC_VECTOR(7 downto 0);
	signal	LedSelect: STD_LOGIC;
	signal	Proc0RAMRB: STD_LOGIC_VECTOR(7 downto 0);
	signal	Proc0RAMTS: STD_LOGIC_VECTOR(7 downto 0);
	signal 	Fixedproc0IOAdd: STD_LOGIC_VECTOR(9 downto 0);
	signal 	RAM0WriteSelect: STD_LOGIC;
	signal	RAM0ReadSelect: STD_LOGIC;
	signal 	Mul0WriteSelect: STD_LOGIC;
	signal	Mul0ReadPSelect: STD_LOGIC;
	signal	Mul0ReadMSelect: STD_LOGIC;
	signal	KIndex0: STD_LOGIC_VECTOR (2 downto 0);
	signal	KIndex0ReadSelect: STD_LOGIC;
	signal	KIndex0WriteSelect: STD_LOGIC;
	signal	EarlyROMPage0: STD_LOGIC_VECTOR (2 downto 0);
	signal	ROMPage0: STD_LOGIC_VECTOR (2 downto 0);
	signal	ROMPage0ReadSelect: STD_LOGIC;
	signal	ROMPage0WriteSelect: STD_LOGIC;
	signal	ClearROMPage0: STD_LOGIC;
	signal	Menable0WriteSelect: STD_LOGIC;
	signal	Menable0WriteSelects: STD_LOGIC_VECTOR(3 downto 0);
	signal	Nenables: STD_LOGIC_VECTOR(3 downto 0);
	signal	IrqDriveReadSelect: STD_LOGIC;
	signal	IrqDriveWriteSelect: STD_LOGIC;
	signal	TimerReadSelect: STD_LOGIC;
	signal	TimerWriteSelect: STD_LOGIC;
-- IO port related stuff
	signal	IOPortAWriteSelect: STD_LOGIC;
	signal	IOPortAReadSelect: STD_LOGIC;
	signal	IOPortAWriteDDRSelect: STD_LOGIC;
	signal	IOPortAReadDDRSelect: STD_LOGIC;


	signal	IOPortBWriteSelect: STD_LOGIC;
	signal	IOPortBReadSelect: STD_LOGIC;
	signal	IOPortBWriteDDRSelect: STD_LOGIC;
	signal	IOPortBReadDDRSelect: STD_LOGIC;


	signal	IOPortCWriteSelect: STD_LOGIC;
	signal	IOPortCReadSelect: STD_LOGIC;
	signal	IOPortCWriteDDRSelect: STD_LOGIC;
	signal	IOPortCReadDDRSelect: STD_LOGIC;


	signal	IOPortDWriteSelect: STD_LOGIC;
	signal	IOPortDReadSelect: STD_LOGIC;
	signal	IOPortDWriteDDRSelect: STD_LOGIC;
	signal	IOPortDReadDDRSelect: STD_LOGIC;

	signal	MiscBitsWriteSelect: STD_LOGIC;
	signal	MiscBitsReadSelect: STD_LOGIC;
	signal	MiscBitsWriteDDRSelect: STD_LOGIC;
	signal	MiscBitsReadDDRSelect: STD_LOGIC;
		
	signal 	ReadIndexCmd: STD_LOGIC; 
	signal 	LoadIndexCmd: STD_LOGIC; 
	signal 	WordAccess: STD_LOGIC;

-- counter signals --
	signal 	CounterReadLow: STD_LOGIC_VECTOR  (3 downto 0);	-- read low 16 bits
	signal 	CounterReadHigh: STD_LOGIC_VECTOR (3 downto 0);	-- read high 16 bits
	signal 	GlobalCounterEnable: STD_LOGIC;										-- enable counting
	signal 	GlobalCountLatchcmd: STD_LOGIC;										-- command to latch counter value	
	signal 	GlobalCountLatch: STD_LOGIC;											-- command + irq generated latch count
	signal 	CCRLoadCmds: STD_LOGIC_VECTOR (3 downto 0);			-- counter control reg loads
	signal 	CCRReadCmds:	STD_LOGIC_VECTOR (3 downto 0);		-- counter control reg reads
	signal 	GlobalCounterClear: STD_LOGIC;										-- clear counter

-- pwm generator signals --
	signal RefCountBus: STD_LOGIC_VECTOR (7 downto 0);
	signal LoadPWM: STD_LOGIC_VECTOR  (3 downto 0);
	signal ReadPWM: STD_LOGIC_VECTOR  (3 downto 0);
	signal PCRLoadCmds: STD_LOGIC_VECTOR  (3 downto 0);
	signal PCRReadCmds: STD_LOGIC_VECTOR  (3 downto 0);
	signal GlobalPWMEnable: STD_LOGIC;
	signal GlobalClearPWM: STD_LOGIC;
	signal GlobalClearPWMCmd: STD_LOGIC;
	signal StopPWM: STD_LOGIC;

-- decodes --
	signal IndexSel: STD_LOGIC;	
	signal GCRSel: STD_LOGIC;	
	signal GMRSel: STD_LOGIC;
	signal CCRSel: STD_LOGIC;
	signal PCRSel: STD_LOGIC;	
	signal IRQSel: STD_LOGIC;	
	signal IRQDIVSel: STD_LOGIC;	
	signal PWMValSel: STD_LOGIC;
	signal CounterLowSel: STD_LOGIC;	
	signal CounterHighSel: STD_LOGIC;			
	signal Progrom0Read: STD_LOGIC;	
	signal Progrom0Write: STD_LOGIC;
	signal StatusReadSelect: STD_LOGIC;
	signal StatusWriteSelect: STD_LOGIC;
	signal LedLatch: STD_LOGIC_VECTOR (5 downto 0);
		
	component scounter
		port ( 
		obus: out STD_LOGIC_VECTOR (7 downto 0);
		ibus: in STD_LOGIC_VECTOR (7 downto 0);
		quada: in STD_LOGIC;
		quadb: in STD_LOGIC;
		index: in STD_LOGIC;
		ccrloadcmd: in STD_LOGIC;
		ccrreadcmd: in STD_LOGIC;
		countoutreadcmd: in STD_LOGIC;
	   byteselect: in STD_LOGIC_VECTOR (1 downto 0);		
		countlatchcmd: in STD_LOGIC;
		clk: in STD_LOGIC
		);
	end component;

	component pwmgen 
		port (
		clk: in STD_LOGIC;
		refcount: in STD_LOGIC_VECTOR (7 downto 0);
		ibus: in STD_LOGIC_VECTOR (7 downto 0);
		loadpwmval: in STD_LOGIC;
		loadDirCmd: STD_LOGIC;
		pwmout: out STD_LOGIC;
		dirout: out STD_LOGIC
		);
	end component pwmgen;
		
	component pwmref is
   	port (
		clk: in STD_LOGIC;
		refcount: out STD_LOGIC_VECTOR (7 downto 0);
		samplegen: out STD_LOGIC;
		ibus: in STD_LOGIC_VECTOR (7 downto 0);
		sampledivload: in STD_LOGIC;
		clearsampleff: in STD_LOGIC
			);
	end component pwmref;

--	component indexreg is 
--		port (
--		clk: in STD_LOGIC;
--		ibus: in STD_LOGIC_VECTOR (15 downto 0);
--		obus: out STD_LOGIC_VECTOR (15 downto 0);
--		loadindex: in STD_LOGIC;
--		readindex: in STD_LOGIC;
--		index: out STD_LOGIC_VECTOR (7 downto 0)
--		);
--	end component indexreg;


	component dpblockram1024x8 is 
		port	(
		clk  : in std_logic; 
		we   : in std_logic; 
		adda : in std_logic_vector(9 downto 0); 
 		addb : in std_logic_vector(9 downto 0); 
 		din  : in std_logic_vector(7 downto 0); 
 		outa : out std_logic_vector(7 downto 0); 
 		outb : out std_logic_vector(7 downto 0));
 	end component dpblockram1024x8; 

  	component dpblockram1024x16 is 
		port	(
		clk  : in std_logic; 
		we   : in std_logic; 
		adda : in std_logic_vector(9 downto 0); 
 		addb : in std_logic_vector(9 downto 0); 
 		din  : in std_logic_vector(15 downto 0); 
 		outa : out std_logic_vector(15 downto 0); 
 		outb : out std_logic_vector(15 downto 0));
 	end component dpblockram1024x16; 

	component kcpsm is
		port (
		addr			: out std_logic_vector(7 downto 0);
		i				: in std_logic_vector(15 downto 0);
		input			: in std_logic_vector(7 downto 0);
		output		: out std_logic_vector(7 downto 0);
		port_id		: out std_logic_vector(7 downto 0);
 	   read_strobe	: out std_logic;
 		write_strobe: out std_logic;
		interrupt	: in std_logic;
		clk			: in std_logic);
	end component kcpsm;

	component mul16x32 is
   	Port ( 
		m: in STD_LOGIC_VECTOR (7 downto 0);
		p: out STD_LOGIC_VECTOR (7 downto 0);
		mlatch: in STD_LOGIC;
		readm: in STD_LOGIC;
		readprod: in STD_LOGIC;
		seladd: in STD_LOGIC_VECTOR (1 downto 0);
		clk: in STD_LOGIC
		 );
	end component mul16x32;

	component goutport is
   	port (
		clk: in STD_LOGIC;
		ibus0: in STD_LOGIC;
		loadgoutport: in STD_LOGIC;
		goutdata: out STD_LOGIC
		);
	end component goutport;

	component gindexreg is
   	port (
		clk: in STD_LOGIC;
		ibus: in STD_LOGIC_VECTOR (2 downto 0);	
		obus: out STD_LOGIC_VECTOR (2 downto 0);
		loadgindex: in STD_LOGIC;
		readgindex: in STD_LOGIC;
		cleargindex: in STD_LOGIC;
		gindexdata: out STD_LOGIC_VECTOR (2 downto 0)
		);

	end component gindexreg;

	component commandreg is
		port (
	 	clk: in STD_LOGIC;
		procibus: in STD_LOGIC_VECTOR (7 downto 0);
		procobus: out STD_LOGIC_VECTOR (7 downto 0);
		procadd: in STD_LOGIC;
		procreadcommand: in STD_LOGIC;
		procwritecommand: in STD_LOGIC;
		hostibus: in STD_LOGIC_VECTOR (15 downto 0);
		hostobus: out STD_LOGIC_VECTOR (15 downto 0);
		hostreadcommand: in STD_LOGIC;
		hostwritecommand: in STD_LOGIC;
		clrbusyflag: in STD_LOGIC;
		busyflag: out STD_LOGIC
		);
	end component commandreg;
	
	component DataReg is
    	port ( 	
	 	clk: in STD_LOGIC;
		procibus: in STD_LOGIC_VECTOR (7 downto 0);
		procobus: out STD_LOGIC_VECTOR (7 downto 0);
		procadd: in STD_LOGIC_VECTOR (1 downto 0);
		procreaddata: in STD_LOGIC;						
		procwritedata: in STD_LOGIC;
		hostibus: in STD_LOGIC_VECTOR (15 downto 0);
		hostobus: out STD_LOGIC_VECTOR (15 downto 0);
		hostreadadd: in STD_LOGIC;
		hostwriteadd: in STD_LOGIC;
		hostreaddata: in STD_LOGIC;
		hostwritedata: in STD_LOGIC;
		hostdataclear: in STD_LOGIC
		);
	end component DataReg;

	component IrqDrive is
   	port (		
	   clear: in STD_LOGIC;
		clk: in STD_LOGIC;
		ibus: in STD_LOGIC_VECTOR (7 downto 0);
		obus: out STD_LOGIC_VECTOR (7 downto 0);
		loadport: in STD_LOGIC;
      readport: in STD_LOGIC;
		irqs: out STD_LOGIC_VECTOR (15 downto 3)
 		);
	end component IrqDrive;
	
	component timer is
	   port ( 
		obus: out STD_LOGIC_VECTOR (7 downto 0);
		timerreadcmd: in STD_LOGIC;
		byteselect: in STD_LOGIC;
		timerlatchcmd: in STD_LOGIC;
		timerclearcmd: in STD_LOGIC;
		clk: in STD_LOGIC
		);
	end component timer;

	component byteiorb is
	   port (			
	 	obus: out STD_LOGIC_VECTOR (7 downto 0);
		readport: in STD_LOGIC;
		portdata: in STD_LOGIC_VECTOR (7 downto 0) );
	end component byteiorb;	
	
	component byteport is
   	port (		
	 	clear: in STD_LOGIC;
		clk: in STD_LOGIC;
		ibus: in STD_LOGIC_VECTOR (7 downto 0);
		obus: out STD_LOGIC_VECTOR (7 downto 0);
		loadport: in STD_LOGIC;
		loadddr: in STD_LOGIC;
		readddr: in STD_LOGIC;
		portdata: out STD_LOGIC_VECTOR (7 downto 0)
 		);
	end component byteport;

	begin -- the whole ball-o-wax

	DRQ <= "ZZ";

--	DIS <= '1';			-- uncomment to disable config decode
	DIS <= '0';			-- uncomment to enable config decode
 		
		
--	MEMCS16 <= 'Z';
--	IOCHRDY <= 'Z'



	makecounters: for i in 0 to 3 generate
		counterx: scounter port map ( 		
			obus => Proc0Ibus,
			ibus => Proc0Obus,
			quada => A(i),
			quadb => B(i),		
			index => Idx(i),
			ccrloadcmd => CCRWriteSelects(i),
			ccrreadcmd => CCRReadSelects(i),
			countoutreadcmd  => CounterSelects(i),
			byteselect => Proc0IOAdd(1 downto 0),
			countlatchcmd => SampleRate,	
			clk => SynClk
		);	
	end generate;

	makepwms: for i in 0 to 3 generate
		pwmgenx: pwmgen port map (
			clk => SynClk,
			refcount => RefCountBus,
			ibus => Proc0Obus,
			loadpwmval => PWMSelects(i),
			loadDirCmd => DIRSelects(i),
			pwmout => PWM(i),
			dirout => Dir(i)
			);
	end generate;

	pwmrefcount: pwmref port map (
		clk => SynClk,
		refcount => RefCountBus,
		SampleGen => SampleRate,
		ibus => Proc0Obus,
		Sampledivload => SampleDivSelect,
		clearsampleff => StatusWriteSelect
		);


	progROM0: dpblockram1024x16 port map (
     	clk => SynClk,
 		we  => ROM0Write,
		adda => ISA0add(9 downto 0),
 		addb => PagedProc0add,
 		din => DBlatch,
 		outa => Proc0DataISARB,
 	 	outb => Proc0data
		);


	progRAM0: dpblockram1024x8 port map (
     	clk => SynClk,
 		we  => RAM0WriteSelect,
		adda => FixedProc0IOAdd,
 		addb => "0000000000",
 		din => Proc0OBus,
 		outa => Proc0RAMRB
		);
			

	kcpsm0: kcpsm port map
		(
		addr => proc0add,
		i => Proc0Data,
 		input => Proc0Ibus,
		output =>Proc0Obus,
		port_id =>Proc0IOAdd,
 	   read_strobe	=> Proc0IORD,
 		write_strobe => Proc0IOWR,
		interrupt => '0',
		clk => SynClk
		);
		
	mul0: mul16x32 port map (
		m => Proc0Obus,
		p => Proc0Ibus,
		mlatch => Mul0WriteSelect,
		readm => Mul0ReadMSelect,
		readprod => Mul0ReadPSelect,
		seladd => Proc0IOAdd(1 downto 0),
		clk => SynClk
		);

	kindexreg0: gindexreg port map (  
		clk => SynClk,
		ibus => Proc0Obus(2 downto 0),
		obus => Proc0Ibus(2 downto 0),
		loadgindex => Kindex0WriteSelect,
		readgindex => Kindex0ReadSelect,
		cleargindex => '0',
		gindexdata  => Kindex0
		);

	ROMPageReg0: gindexreg port map (  
		clk => SynClk,
		ibus => Proc0Obus(2 downto 0),
		obus => Proc0Ibus(2 downto 0),
		loadgindex => RomPage0WriteSelect,
		readgindex => RomPage0ReadSelect,
		cleargindex => ClearRomPage0,
		gindexdata => EarlyRomPage0
		);


	makemenables:	for i in 0 to 3 generate	
		menablex: goutport port map (
		clk => SynClk,	
		ibus0 => Proc0Obus(0),
		loadgoutport => Menable0WriteSelects(i),
		goutdata => Nenables(i)
		);
	end generate;

	TheCommandReg: commandreg port map (
 		clk => synclk,
		procibus => Proc0Obus,
		procobus => Proc0Ibus,
		procadd => Proc0IOAdd(0),
		procreadcommand => Proc0CommandReadSelect,
		procwritecommand => Proc0CommandWriteSelect,
		hostibus	=> DBLatch,
		hostobus => DBus,
		hostreadcommand => HostReadCommandSel,
		hostwritecommand => HostWriteCommandSel,
		clrbusyflag => Proc0ClearBusyFlag,
		busyflag => Proc0BusyFlag
		);
		
	TheDataReg: Datareg port map (
 		clk => synclk,
		procibus => Proc0Obus,
		procobus => Proc0Ibus,
		procadd => Proc0IOAdd(1 downto 0),
		procreaddata => Proc0DataReadSelect,					
		procwritedata => Proc0DataWriteSelect,
		hostibus	=> DBLatch,
		hostobus => DBus,
		hostreadadd => SA(1),
		hostwriteadd => SALatch,
		hostreaddata => HostReadDataSel,
		hostwritedata =>HostWriteDataSel,
		hostdataclear => ClearHostData
		);

	TheIrqDrive: IrqDrive port map (		
	   clear => RSTDRV,
		clk => SynClk,
		ibus => Proc0OBus,
		obus => Proc0IBus,
		loadport => IrqDriveWriteSelect,
      readport => IrqDriveReadSelect,
		irqs => IRQ
 		);

	Thecycletimer: timer port map (
		obus => Proc0IBus,
		timerreadcmd => TimerReadSelect,
		byteselect => Proc0IOAdd(0),
		timerlatchcmd => TimerWriteSelect,
		timerclearcmd => SampleRate,
		clk => SynClk
		);
		
		IOPortA: byteport port map (
	 	clear => RSTDRV,
		clk => SynClk,
		ibus => Proc0OBus,
		obus => Proc0IBus,
		loadport => IOPortAWriteSelect,
		loadddr  => IOPortAWriteDDRSelect,
		readddr  => IOPortAReadDDRSelect,
		portdata => IOPortAData
		);

		IORBPortA:  byteiorb port map (
		obus => Proc0IBus,
		readport => IOPortAReadSelect,
		portdata => IOPortAData
		);			
		
		IOPortB: byteport port map (
	 	clear => RSTDRV,
		clk => SynClk,
		ibus => Proc0OBus,
		obus => Proc0IBus,
		loadport => IOPortBWriteSelect,
		loadddr  => IOPortBWriteDDRSelect,
		readddr  => IOPortBReadDDRSelect,
		portdata => IOPortBData
		);

		IORBPortB:  byteiorb port map (
		obus => Proc0IBus,
		readport => IOPortBReadSelect,
		portdata => IOPortBData
		);			
		
		IOPortC: byteport port map (
	 	clear => RSTDRV,
		clk => SynClk,
		ibus => Proc0OBus,
		obus => Proc0IBus,
		loadport => IOPortCWriteSelect,
		loadddr  => IOPortCWriteDDRSelect,
		readddr  => IOPortCReadDDRSelect,
		portdata => IOPortCData
		);

		IORBPortC:  byteiorb port map (
		obus => Proc0IBus,
		readport => IOPortCReadSelect,
		portdata => IOPortCData
		);					
		
		IOPortD: byteport port map (
	 	clear => RSTDRV,
		clk => SynClk,
		ibus => Proc0OBus,
		obus => Proc0IBus,
		loadport => IOPortDWriteSelect,
		loadddr  => IOPortDWriteDDRSelect,
		readddr  => IOPortDReadDDRSelect,
		portdata => IOPortDData
		);

		IORBPortD:  byteiorb port map (
		obus => Proc0IBus,
		readport => IOPortDReadSelect,
		portdata => IOPortDData
		);				 	

		MiscBitsPort: byteport port map (
	 	clear => RSTDRV,
		clk => SynClk,
		ibus => Proc0OBus,
		obus => Proc0IBus,
		loadport => MiscBitsWriteSelect,
		loadddr  => MiscBitsWriteDDRSelect,
		readddr  => MiscBitsReadDDRSelect,
		portdata => MiscBits
		);

		MiscBitsIORB:  byteiorb port map (
		obus => Proc0IBus,
		readport => MiscBitsReadSelect,
		portdata => MiscBits
		);				 	


		kcpsm0decode: process
		(
		Proc0Ibus,
		Proc0Obus,
		Proc0IOAdd,
 		Proc0IORD,
 		Proc0IOWR
		) 
	begin -- Gynormous ucontroller decoder...
--		if Proc0IOAdd(7 downto 6)  = "00" and Proc0IORD = '1' then -- IO = bottom 64 bytes		
		if Proc0IOAdd(7 downto 6)  = "00" then -- IO = bottom 64 bytes		
			IOReadSelect0 <= '1';
		else
			IOReadSelect0 <= '0';
		end if;
		
		if Proc0IOAdd(7 downto 6)  = "00" and Proc0IOWR = '1' then -- IO = bottom 64 bytes
   		IOWriteSelect0 <= '1';
		else
			IOWriteSelect0 <= '0';
		end if;

		if Proc0IOAdd(5 downto 2)  = "0000" and IOReadSelect0 = '1' then -- Quadrature counter read
			CounterSelect <= '1'; 
		else 
			CounterSelect <= '0';
		end if;				

		if Proc0IOAdd(5 downto 2)  = "0000" and IOWriteSelect0 = '1' then -- PWM write
			PWMSelect <= '1'; 
		else 
			PWMSelect <= '0';
		end if;	
			
		if Proc0IOAdd(5 downto 0)  = "000100" and IOWriteSelect0 = '1' then -- 4 Dir bits
			DirSelect <= '1'; 
		else 
			DirSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "000101" and IOWriteSelect0 = '1' then -- 4 Counter Latches
			CountLatchSelect <= '1'; 
		else 
			CountLatchSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "000110" and IOWriteSelect0 = '1' then -- 4 Counter control register
			CCRWriteSelect <= '1'; 
		else 
			CCRWriteSelect <= '0';
		end if;	
	
		if Proc0IOAdd(5 downto 0)  = "000110" and IOReadSelect0 = '1' then -- 4 Counter control register
			CCRReadSelect <= '1'; 
		else 
			CCRReadSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "000111" and IOWriteSelect0 = '1' then -- Led latch
			SampleDivSelect <= '1'; 
		else 
	  		SampleDivSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001000" and IOReadSelect0 = '1' then -- Status read
			StatusReadSelect <= '1'; 
		else 
			StatusReadSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001000" and IOWriteSelect0 = '1' then -- Status write clears sampleff
			StatusWriteSelect <= '1'; 
		else 
			StatusWriteSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001001" and IOReadSelect0 = '1' then -- Kindex read
			Kindex0ReadSelect <= '1'; 
		else 
			Kindex0ReadSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001001" and IOWriteSelect0 = '1' then -- Kindex write read
			Kindex0WriteSelect <= '1'; 
		else 
			Kindex0WriteSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001010" and IOReadSelect0 = '1' then -- ROM Page select read
			ROMPage0ReadSelect <= '1'; 
		else 
			ROMpage0ReadSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001010" and IOWriteSelect0 = '1' then -- ROM Page select write			
			ROMPage0WriteSelect <= '1'; 			
		else 
			ROMPage0WriteSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001011" and IOWriteSelect0 = '1' then -- Out bits reg
			MiscBitsWriteSelect <= '1'; 
		else 
			MiscBitsWriteSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001011" and IOReadSelect0 = '1' then -- in bits reg
			MiscBitsReadSelect <= '1'; 
		else 
			MiscBitsReadSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001100" and IOWriteSelect0 = '1' then -- Out bits reg
			MiscBitsWriteDDRSelect <= '1'; 
		else 
			MiscBitsWriteDDRSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001100" and IOReadSelect0 = '1' then -- in bits reg
			MiscBitsReadDDRSelect <= '1'; 
		else 
			MiscBitsReadDDRSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001101" and IOWriteSelect0 = '1' then -- Motor enable bits 
			Menable0WriteSelect <= '1'; 
		else 
			Menable0WriteSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "001110" and IOWriteSelect0 = '1' then -- Motor enable bits 
			Proc0ClearBusyFlag <= '1'; 
		else 
			Proc0ClearBusyFlag <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001111" and IOReadSelect0 = '1' then -- Kindex read
			IrqDriveReadSelect <= '1'; 
		else 
			IrqDriveReadSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 0)  = "001111" and IOWriteSelect0 = '1' then -- Kindex write read
			IrqDriveWriteSelect <= '1'; 
		else 
			IrqDriveWriteSelect <= '0';
		end if;			
		
		if Proc0IOAdd(5 downto 2)  = "0100" and IOReadSelect0 = '1' then 
			Proc0DataReadSelect <= '1'; 
		else 
			Proc0DataReadSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 2)  = "0100" and IOWriteSelect0 = '1' then -- Motor enable bits 
			Proc0DataWriteSelect <= '1'; 
		else 
			Proc0DataWriteSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 2)  = "0101" and IOWriteSelect0 = '1' then -- multiplier M write select
			Mul0WriteSelect <= '1'; 
		else 
			Mul0WriteSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 2)  = "0101" and IOReadSelect0 = '1' then -- multiplier M Read Select
			Mul0ReadMSelect <= '1'; 
		else 
			Mul0ReadMSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 2)  = "0110" and IOReadSelect0 = '1' then -- multiplier P read select
			Mul0ReadPSelect <= '1'; 
		else 
			Mul0ReadPSelect <= '0';
		end if;	

		if Proc0IOAdd(5 downto 1)  = "01110" and IOReadSelect0 = '1' then 
			Proc0CommandReadSelect <= '1'; 
		else 
			Proc0CommandReadSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 1)  = "01110" and IOWriteSelect0 = '1' then  
			Proc0CommandWriteSelect <= '1'; 
		else 
			Proc0CommandWriteSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 1)  = "01111" and IOReadSelect0 = '1' then 
			TimerReadSelect <= '1'; 
		else 
			TimerReadSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 1)  = "01111" and IOWriteSelect0 = '1' then  
			TimerWriteSelect <= '1'; 
		else 
			TimerWriteSelect <= '0';
		end if;			

-- locations 32 --> 39 = I/O ports

-- PortA
		if Proc0IOAdd(5 downto 0)  = "100000" and IOReadSelect0 = '1' then 
			IOPortAReadSelect <= '1'; 
		else 
			IOPortAReadSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100000" and IOWriteSelect0 = '1' then  
			IOPortAWriteSelect <= '1'; 
		else 
			IOPortAWriteSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100001" and IOReadSelect0 = '1' then 
			IOPortAReadDDRSelect <= '1'; 
		else 
			IOPortAReadDDRSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100001" and IOWriteSelect0 = '1' then  
			IOPortAWriteDDRSelect <= '1'; 
		else 
			IOPortAWriteDDRSelect <= '0';
		end if;	

-- PortB		
		if Proc0IOAdd(5 downto 0)  = "100010" and IOReadSelect0 = '1' then 
			IOPortBReadSelect <= '1'; 
		else 
			IOPortBReadSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100010" and IOWriteSelect0 = '1' then  
			IOPortBWriteSelect <= '1'; 
		else 
			IOPortBWriteSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100011" and IOReadSelect0 = '1' then 
			IOPortBReadDDRSelect <= '1'; 
		else 
			IOPortBReadDDRSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100011" and IOWriteSelect0 = '1' then  
			IOPortBWriteDDRSelect <= '1'; 
		else 
			IOPortBWriteDDRSelect <= '0';
		end if;	

-- PortC		
		if Proc0IOAdd(5 downto 0)  = "100100" and IOReadSelect0 = '1' then 
			IOPortCReadSelect <= '1'; 
		else 
			IOPortCReadSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100100" and IOWriteSelect0 = '1' then  
			IOPortCWriteSelect <= '1'; 
		else 
			IOPortCWriteSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100101" and IOReadSelect0 = '1' then 
			IOPortCReadDDRSelect <= '1'; 
		else 
			IOPortCReadDDRSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100101" and IOWriteSelect0 = '1' then  
			IOPortCWriteDDRSelect <= '1'; 
		else 
			IOPortCWriteDDRSelect <= '0';
		end if;	

-- PortD		
		if Proc0IOAdd(5 downto 0)  = "100110" and IOReadSelect0 = '1' then 
			IOPortDReadSelect <= '1'; 
		else 
			IOPortDReadSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100110" and IOWriteSelect0 = '1' then  
			IOPortDWriteSelect <= '1'; 
		else 
			IOPortDWriteSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100111" and IOReadSelect0 = '1' then 
			IOPortDReadDDRSelect <= '1'; 
		else 
			IOPortDReadDDRSelect <= '0';
		end if;			

		if Proc0IOAdd(5 downto 0)  = "100111" and IOWriteSelect0 = '1' then  
			IOPortDWriteDDRSelect <= '1'; 
		else 
			IOPortDWriteDDRSelect <= '0';
		end if;			
	
		if Proc0IOAdd(7 downto 0)  = "00111110" and Proc0IOWR = '1' then -- Clear the host data reg			
			ClearHostData <= '1'; 			
		else 
			ClearHostData <= '0';
		end if;	
	
-- last but not least the LEDS:
		
		
		if Proc0IOAdd(5 downto 0)  = "111111" and IOWriteSelect0 = '1' then -- Led latch
			LedSelect <= '1'; 
		else 
			LedSelect <= '0';
		end if;	

		if ((Proc0IOAdd(7) = '1') or (Proc0IOAdd(6) = '1')) and Proc0IORD = '1' then 
   		RAM0ReadSelect <= '1';
		else
			RAM0ReadSelect <= '0';
		end if;

		if ((Proc0IOAdd(7) = '1') or (Proc0IOAdd(6) = '1')) and Proc0IOWR = '1' then 
   		RAM0WriteSelect <= '1';
		else
			RAM0WriteSelect <= '0';
		end if;
	end process;

	kcpsm0indexdecode: process
	(
		CounterSelect,
		PWMSelect,
		DirSelect,
 		CountLatchSelect,
 		CCRWriteSelect,
		CCRReadSelect,
		Menable0WriteSelect,
		Kindex0
	) 

	begin 

	  	CounterSelects <= "0000";
		if CounterSelect = '1' then
			case Kindex0(2 downto 0) is
				when "000" => CounterSelects <= "0001"; 
				when "001" => CounterSelects <= "0010"; 	
				when "010" => CounterSelects <= "0100"; 
				when "011" => CounterSelects <= "1000";
 				when others => CounterSelects <= "0000";
			end case;
		else CounterSelects <= "0000"; 
		end if;

		PWMSelects <= "0000";
		if PWMSelect = '1' then
			case Kindex0(2 downto 0) is
				when "000" => PWMSelects <= "0001"; 
				when "001" => PWMSelects <= "0010"; 	
				when "010" => PWMSelects <= "0100"; 
				when "011" => PWMSelects <= "1000";
 				when others => PWMSelects <= "0000";
			end case;
		else PWMSelects <= "0000";
		end if;	

		DirSelects <= "0000";
		if DirSelect = '1' then
			case Kindex0(2 downto 0) is
				when "000" => DirSelects <= "0001"; 
				when "001" => DirSelects <= "0010"; 	
				when "010" => DirSelects <= "0100"; 
				when "011" => DirSelects <= "1000";
 				when others => DirSelects <= "0000";
			end case;
		else DirSelects <= "0000";
		end if;	

		CountLatchSelects <= "0000";
		if CountLatchSelect = '1' then
			case Kindex0 (2 downto 0) is
				when "000" => CountLatchSelects <= "0001"; 
				when "001" => CountLatchSelects <= "0010"; 	
				when "010" => CountLatchSelects <= "0100"; 
				when "011" => CountLatchSelects <= "1000";
 				when others => CountLatchSelects <= "0000";
			end case;
		else CountLatchSelects <= "0000";
		end if;

		CCRReadSelects <= "0000";
		if CCRReadSelect = '1' then
			case Kindex0(2 downto 0) is
				when "000" => CCRReadSelects <= "0001"; 
				when "001" => CCRReadSelects <= "0010"; 	
				when "010" => CCRReadSelects <= "0100"; 
				when "011" => CCRReadSelects <= "1000";
 				when others => CCRReadSelects <= "0000";
			end case;
		else CCRReadSelects <= "0000";
		end if;	

		CCRWriteSelects <= "0000";
		if CCRWriteSelect = '1' then
			case Kindex0(2 downto 0) is
				when "000" => CCRWriteSelects <= "0001"; 
				when "001" => CCRWriteSelects <= "0010"; 	
				when "010" => CCRWriteSelects <= "0100"; 
				when "011" => CCRWriteSelects <= "1000";
 				when others => CCRWriteSelects <= "0000";
			end case;
		else CCRWriteSelects <= "0000";
		end if;
	
		if Menable0WriteSelect = '1' then
			case Kindex0(2 downto 0) is
				when "000" => Menable0WriteSelects <= "0001"; 
				when "001" => Menable0WriteSelects <= "0010"; 	
				when "010" => Menable0WriteSelects <= "0100"; 
				when "011" => Menable0WriteSelects <= "1000";
 				when others => Menable0WriteSelects <= "0000";
			end case;
		else Menable0WriteSelects <= "0000";
		end if;	
	end process;

	AStatusReg: process (StatusReadSelect,SampleRate,StatusReg,Proc0BusyFlag)
	begin
		if StatusReadSelect = '1' then
			StatusReg(0) <= SampleRate;
			StatusReg(1) <= Proc0BusyFlag;
			else
			StatusReg <= "ZZZZZZZZ";
		end if;
		Proc0Ibus <= StatusReg;
	end process;		

	ProgRam0Select: process (RAM0ReadSelect,Proc0IOAdd,Proc0RAMRB,Kindex0)
	begin
   	if RAM0ReadSelect = '1' then
			Proc0RAMTS <= Proc0RAMRB;
 		else
			Proc0RAMTS <= "ZZZZZZZZ";
		end if;
		Proc0Ibus <= Proc0RAMTS;
		FixedProc0IOAdd(7 downto 0) <= proc0IOAdd;			
		FixedProc0IOAdd(9 downto 8) <= Kindex0(1 downto 0);		
	end process;

	ProgROM0Select: process (ROMPage0,Proc0Add,synclk)
	begin
		if SynClk'event and SynClk = '1' then
			ROMPage0 <= EarlyROMPage0; -- pipeline delay for ROM page changing
		end if;
		PagedProc0Add(7 downto 0) <= Proc0Add;
		PagedProc0Add(9 downto 8) <= ROMPage0(1 downto 0);
	end process;

	CSelect: process (SA, AEN, CardSelect, IORD, IOWR) 
	begin 
		if SA(9 downto 4) = Decode and AEN = '0' then 
   		CardSelect <= '1';
		else
			CardSelect <= '0';
		end if;
		
		if SA(3 downto 1) = "000" then 
			WordAccess <= '0'; 
		else 
			WordAccess <= '1';
		end if;

		if SA(3 downto 1) = "001" and CardSelect = '1' and IORD = '0' then 
			HostReadCommandSel <= '1'; 
		else 
			HostReadCommandSel <= '0';
		end if;

		if SA(3 downto 1) = "001" and CardSelect = '1' and IOWR = '0' then 
			HostWriteCommandSel <= '1'; 
		else 
			HostWriteCommandSel <= '0';
		end if;

		if SA(3 downto 2) = "01" and CardSelect = '1' and IORD = '0' then 
			HostReadDataSel <= '1'; 
		else 
			HostReadDataSel <= '0';
		end if;					

		if SA(3 downto 2) = "01" and CardSelect = '1' and IOWR = '0' then 
			HostWriteDataSel <= '1'; 
		else 
			HostWriteDataSel <= '0';
		end if;					

		if SA(3 downto 0) = "1000" and CardSelect = '1' then 
			ROMAddrReg0Sel <= '1'; 
		else 
			ROMAddrReg0Sel <= '0';
		end if;		
		
		if SA(3 downto 0) = "1010" and CardSelect = '1' then 
			ROMData0Sel <= '1'; 
		else 
			ROMData0Sel <= '0';
		end if;
	end process;

	LatchDin: process (SD, IOWR, CardSelect,SynClk,RSTdrv) 
	begin
		if SynClk'event and  SynClk = '1' then
			if IOWR = '0' and CardSelect = '1' and RSTDrv = '0' then
				DBLatch <= PreDBLatch;
				PreDBLatch <= SD;
				SALatch <= PreSALatch;
				PreSALatch <= SA(1);
			end if;
		end if;	
	end process;
	
	GenIOCS: process (CardSelect, WordAccess)
	begin
		if (CardSelect = '1') and WordAccess = '1' then  	-- address 4 thru 7 are 16 bit
			IOCS16 <= '0';
		else
			IOCS16 <= 'Z';
		end if;
	end process GenIOCS;	
	
	SDDrivers: process (CardSelect, WordAccess, DBus, IORD)
	begin 
		if (CardSelect = '1') and IORD = '0' then
			if WordAccess = '1' then
				SD <= DBus;
			else 
				SD(7 downto 0) <= DBus(7 downto 0);
				SD(15 downto 8) <= "ZZZZZZZZ";
		 	end if;
		else
			SD <= "ZZZZZZZZZZZZZZZZ";			
		end if;
	end process SDDrivers;

	localbuffer: process (CardSelect, IORD)
	begin
 		
		if CardSelect = '1' then 
			LBE <= '0'; 
		else 
			LBE <= '1'; 
		end if;
		
		if (CardSelect = '1') and (IORD = '0') then
	 		LBDIR <= '0';
		else 
			LBDIR <= '1'; 
		end if;
	end process;

	LEDDrive: process 
   			( 
				SynClk,
				Proc0OBus,
				LedLatch,
				LEDSelect
				)
		begin		
				if SynClk'event and SynClk = '1' then
					if LEDSelect = '1' then LedLatch <= Proc0OBus(5 downto 0);
		  	   	end if;
				LEDS <= not LedLatch;
				end if;

		end process;
	

	EnaDrive: process 
   			( Nenables )
		begin						
			Ena <= not Nenables;
		end process;

	Addpoint0: process 
   			( 
				SynClk,
				DBLatch,
				IOWR,
				ROMAddrReg0Sel
				)
		begin		
				if SynClk'event and SynClk = '1' then
					if (IOWR = '0' and ROMAddrReg0Sel = '1') then
						addpoint0wr1 <= '1';
					else addpoint0wr1 <= '0';
					end if;
					addpoint0wr2 <= addpoint0wr1;
					if addpoint0wr1 = '0' and addpoint0wr2 = '1' then  
						ISA0Add <= DBLatch(9 downto 0);
		  	   	end if;
				end if;
		end process;

Romdata0: process 
   			( 
				SynClk,
				DBLatch,
				IOWR,
				IORD,
				ROMData0Sel,
				Proc0DataISARB
				)
		begin		
				if SynClk'event and SynClk = '1' then
					if (IOWR = '0' and ROMData0Sel = '1') then
						RomData0wr1 <= '1';
					else RomData0wr1 <= '0';
					end if;
					ROMData0wr2 <= ROMData0wr1;
					ClearRomPage0 <= ROMData0wr2;
					if ROMData0wr1 = '0' and ROMData0wr2 = '1' then  
						ROM0Write <= '1';
					else  
					 	ROM0Write <= '0';
					end if;
				end if;
				DBus <= "ZZZZZZZZZZZZZZZZ";
				if (IORD = '0' and ROMData0Sel = '1') then
				 	 DBus <= Proc0DataISARB;
				else DBus <= "ZZZZZZZZZZZZZZZZ";
				end if;
		end process;


end dataflow;

  

































