;MOTOR.ASM
;interrupt on excessive error, breakpoint, or done. read in command reg


;new iomap 10-16-01
;set go to start, done 
;pwm dead zone register
;BUGS


.include "kcpsm.mac"
.MSFIRST
;four rom processor

naxis	equ	4		;four axis

task0		equ	0	;rom page
task1		equ	1
task2		equ	2
task3		equ	3
;************************* SYSTEM CONSTANTS *********************************
nroms		equ	4		;number of roms
clock		equ	50000000
ips		equ	clock/2		;instructions per second
prescale	equ	2048
intrate		equ	clock/prescale

;default paramaters for 50mhz 
divrate		equ	8	;3051Hz	;intrate
ddiv		equ	4;200	;bug if 5 or more

;************************* QUAD COUNTER CONSTANTS ***************************
;cntcntl register

quada		equ	1
quadb		equ	2
index		equ	4	;true to clear counter doesn't stick
indexpol	equ	8
clronindex	equ	16
cntcleared	equ	32	;read count cleared (a write will clear flag)
quadfilter	equ	64	;true for filter
updwnmode	equ	128	;true for updown mode

;irqreq register

whichirq	equ	0	;four least sig bits
irqtsena	equ	16	;true to enable driver

irqmask		equ	64	;true to drive

irq		equ	128	;true for irq

;************************* RAM ***********************************************
;global ram non indexed
ramstart	equ	64
LASTRAM		equ	ramstart

	CHECKRAM

;192 indexed registers 
indexramstart	equ	64

;parameters
nextposq	equ	(0 * q) + indexramstart; where we want to go next
hostmaxvelq	equ	(1 * q) + indexramstart
hostaccelq	equ	(2 * q) + indexramstart
exerrorq	equ	(3 * q) + indexramstart;excessive position error limit
exerrorfq	equ	(4 * q) + indexramstart;excessive position flag
maxnegerrq	equ	(5 * q) + indexramstart;max negative error
maxposerrq	equ	(6 * q) + indexramstart;max positive error
pidonq		equ	(7 * q) + indexramstart;true for pidon
goq		equ	(8 * q) + indexramstart;set non zero to go
moveq		equ	(9 * q) + indexramstart ;true if moving
homeflagq	equ	(10 * q) + indexramstart; homing flag
desposx		equ	(11 * q) + 2 + indexramstart;48 bit position
;desposx	equ	(12 * q) + indexramstart;48 bit position
lastposq	equ	(13 * q) + indexramstart
breakpointq	equ	(14 * q) + indexramstart; breakpoint
mirrorencq	equ	(15 * q) + indexramstart
intmaskq	equ	(16 * q) + indexramstart ;interrupt mask
intcauseq	equ	(17 * q) + indexramstart ;cause of interrupt

hostledq	equ	(19 * q) + indexramstart;0:ledaxis 1:ledpointer

;filter param
kpd		equ	(20 * q) + indexramstart
kid		equ	(21 * q) + indexramstart;all parameters are 32bit
kdd		equ	(22 * q) + indexramstart
kilq		equ	(23 * q) + indexramstart
kf1d		equ	(24 * q) + indexramstart ;velocity ff
kf2d		equ	(25 * q) + indexramstart ;accel ff
kf3d		equ	(26 * q) + indexramstart ;????
hostfree	equ	(27 * q) + indexramstart
hostddivq	equ	(28 * q) + indexramstart;constant

;host move parameters

;internal use only
ierrorq		equ	(30 * q) + indexramstart;i error
derrorq		equ	(31 * q) + indexramstart;d error
integralq	equ	(32 * q) + indexramstart
ifreeq		equ	(33 * q) + indexramstart
ddivq		equ	(34 * q) + indexramstart;constant
halfdistq	equ	(35 * q) + indexramstart
accelq		equ	(36 * q) + indexramstart
smaxvelq	equ	(37 * q) + indexramstart
deltaq		equ	(38 * q) + indexramstart
velocityq	equ	(39 * q) + indexramstart
fixupq		equ	(40 * q) + indexramstart
proctimerq	equ	(41 * q) + indexramstart

debugq		equ	(47 * q) + indexramstart; for testing




LASTINDEXRAM	equ	(47 * q) + q

;	CHECKINDEXRAM
;************************* PORTS **********************************************
;40 to 62 free
;see kcpsm.mac for multiplier ports
;add 64 to get host parameter number


;read write
cntcntl		equ	6
statusreg	equ	8
axisindex	equ	9	;3 bits
rompages	equ	10
irqreg		equ	15
hdataq		equ	16
hcommd		equ	28	;two wasted bytes for simplicity
proctimerd	equ	30	;write to latch then read
porta		equ	32
portaddr	equ	33	;bit is 1 for output
portb		equ	34
portbddr	equ	35	;bit is 1 for output
portc		equ	36
portcddr	equ	37	;bit is 1 for output
portd		equ	38
portdddr	equ	39	;bit is 1 for output
miscbits	equ	11
miscbitsddr	equ	12	;bit is 1 for output

;input only
encposq		equ	0	;per axis, latched by sample

;outputs only
pwm		equ	0	;per axis
dir		equ	4	;per axis
cntl		equ	5	;per axis
samplediv	equ	7
ena		equ	13	;per axis pwm enable
clrbusy		equ	14	;clear host busy flag
ledport		equ	63
;***************************** REGISTERS *************************************
raq   		equ     0 
rbq		equ	4
rcq		equ	8
axiss		equ	12
tempt		equ	13
lastreg		equ	tempt + t

rax		equ	0		;raq,+ half of rbq
rbx		equ	6		;half of rbq + rcq

	CHECKREGS

;************************ MACROS ********************************************
.macro	nexttask	task
	loads	task,raq
	jmpa	1	;skip clr instruction
.endm

.macro	display0	r
	ife0	rcounts	
		display	r
 	endif
.endm

.macro	display1	r
	decs	rcounts
	ifz	
		display	r
 	endif
	incs	rcounts
.endm

.macro	iterate	counter,count
	UNIQUE	; The 'loop' label
	DUP
	loads	count-1,counter
LABEL
.endm

.macro	nextiterate	counter
	decs	counter
	jmpnc	LABEL	;back to for
.endm

;************************ TASK SWITCH ***************************************
;nexttask is in each task file

	org	romsize*0
.include	"tasksw.asm"	;same for all roms
.include	"task0.asm"
	CHECKEOM
	
	org	romsize*1
.include	"tasksw.asm"	;same for all roms
.include	"task1.asm"
	CHECKEOM

	org	romsize*2
.include	"tasksw.asm"	;same for all roms
.include	"task2.asm"
	CHECKEOM

	org	romsize*3
.include	"tasksw.asm"	;same for all roms
.include	"task3.asm"
	CHECKEOM

;************************ REVSIONS ********************************************
;revision       	equ     0


	.END

;renamed curposq to encposq
;added currposq 