library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity mul16x32 is
   Port ( 
	m: in STD_LOGIC_VECTOR (7 downto 0);
	p: out STD_LOGIC_VECTOR (7 downto 0);
	mlatch: in STD_LOGIC;
	readm: in STD_LOGIC;
	readprod: in STD_LOGIC;
	seladd: in STD_LOGIC_VECTOR (1 downto 0);
	clk: in STD_LOGIC
	 );
end mul16x32;


architecture behavioral of mul16x32 is

signal product: STD_LOGIC_VECTOR (31 downto 0);
signal alatch: STD_LOGIC_VECTOR (15 downto 0);
signal blatch: STD_LOGIC_VECTOR (15 downto 0);

begin
	amul16x32: 
	process  
							( clk,
							  m,
							  mlatch,
							  alatch,
							  blatch,
							  product,
							  readm,
							  readprod,
							  seladd
							)
	begin
		if clk'event and clk = '1' then
			if mlatch = '1' and seladd = "00" then 
				alatch(7 downto 0) <= m;
			end if;
			if mlatch = '1' and seladd = "01" then 
				alatch(15 downto 8) <= m;
			end if;
			if mlatch = '1' and seladd = "10" then 
				blatch(7 downto 0) <= m;
			end if;
			if mlatch = '1' and seladd = "11" then 
				blatch(15 downto 8) <= m;
			end if;		
		end if;	
		product <= alatch * blatch;

		p <= "ZZZZZZZZ";

		if readprod = '1' and readm = '0' then
			case seladd is
				when "00" => p <= product(7 downto 0);
				when "01" => p <= product(15 downto 8);
				when "10" => p <= product(23 downto 16);
				when "11" => p <= product(31 downto 24);
				when others =>
			end case;
		end if;		

		if readm = '1' and readprod = '0' then
			case seladd is
				when "00" => p <= alatch(7 downto 0);
				when "01" => p <= alatch(15 downto 8);
				when "10" => p <= blatch(7 downto 0);
				when "11" => p <= blatch(15 downto 8);
				when others =>
			end case;
		end if;
		if readm = '0' and readprod = '0' then 
			p <= "ZZZZZZZZ";
		end if;
		if readm = '1' and readprod = '1' then 
			p <= "ZZZZZZZZ";
		end if;
	end process;

end behavioral;

