{DEMO SOFTWARE - FILTER PARAMETER ADJUST PROGRAM FOR 4I34/7I60/4I65/5I20 MOTOR CONTROLLER }
{$R-}

uses graph,crt,dos;

{$I SWMOTLOW}
{$I parsep}
{$I PCI }
{$I 9030 }
const

NTimes = 8;

type point = record
  X : integer;
  Y : integer;
end;

type box = record
 Start : point;
 Size : point;
end;

type title = string[10];

type labeledbox = record
  TitleBox : box;
  ValueBox : box;
  Title : string;
end;


type menuindextype =  (KSI,KCI,KDFILI,KII,ILI,IKF1,IKF2,IKFF,IKK,SII,STP,VEL,ACC,MTR,TIM);
type menuboxestype = array[KSI..TIM] of labeledbox;
type menutitlestype = array[KSI..TIM] of string[10];
type menuspeed = (DownFast,DownSlow,UpFast,UpSlow,Zero,Max);
timeperdivtype = array [1..NTimes] of word;

type AxisInfo = record
  StepSize : longint;
  Velocity : longint;
  Acceleration : longint;
  KSIN :  word;
  KOFF :  integer;
  KDFIL : word;
  KI :  longint;
  KIL : word;
  KF1 : word;
  KF2 : word;
  KFF : word;
  KK : integer;
  Prescale : word;
  PostScale : byte;
  PWMRate : word;
  tDiv : word;
end;

type AIFile = file of AxisInfo;
type AListFile = text;

const
TheAIFileName = 'Motorst.rec';
TheAListFileName = 'Motorst.lst';
MenuTitles : menutitlestype = ('KSIN','KOFF','KDFIL','KI','IL','KF1','KF2',
                               'KFF','KK','PRE/POST','STEP SIZE','VELOCITY',
                               'ACCEL.','MOTOR','TIME / DIV');
FirstMenuIndex = KSI;
LastMenuIndex = TIM;
Times : timeperdivtype = (1,2,5,10,20,50,100,200);
NTraced = 3; {3 variables traced }
MaxSamples = 6000; { maximum per variable }
DefaultPC104BasePort = $220;
EncoderLines = 500;

DAQFlagLoc = Gevent24Loc; { we use the first entry in global event 24 as our DAQ flag}
TrigVal = Gevent24Loc +1;        { trace buffer/polledmode trigger value to set Go}
SampleCnt = Gevent24Loc +2;      { polled mode internal sample count }

HDivsPerScreen = 10;
VDivsPerScreen = 8;
PolledModeSamplesPerVariable = 200;
PolledDAQFlag = $0AAA;

var
BasePort : word;
StepParameters : array[0..7] of AxisInfo;
SavedParameters : AxisInfo;
Naxis : byte;
Axis : byte;
ScreenBox : labeledbox;
MainBox : labeledbox;
VariablesBox : labeledbox;
ParameterBox : labeledbox;
GraphBox : labeledbox;
MessageBox : labeledbox;

MenuIndex : menuindextype;
MenuBoxes : menuboxestype;
GraphDriver : integer;
BoxColor : word;
TitleColor : word;
ProfileColor : word;
DriveColor : word;
ErrorColor : word;
ValueTextColor : word;
TitleTextColor : word;
ValueColor : word;
TimeDivIndex : word;
ArrowSize : word;
ResponseTable : array[0..MaxSamples] of Integer;
ProfileTable : array[0..MaxSamples] of Integer;
DriveTable : array[0..MaxSamples] of Integer;
GoodSample : array[0..MaxSamples] of boolean;
SamplesPerScreen : word;
TraceBufferSize : word;
SamplesPerVariable : word;
TheAIFile : AIFile;
TheAListFile : AListFile;
HaveTraceBuffer : boolean;

function OpenForRead(var fp: AIFile; name: string): boolean;
begin
  Assign(fp,Name);
  {$I-}
  Reset(fp);
  {$I+}
  OpenForRead := IOResult = 0;
end { Open };

function OpenForWrite(var fp: AIFile; name: string): boolean;
begin
  Assign(fp,Name);
  {$I-}
  Rewrite(fp);
  {$I+}
  OpenForWrite := IOResult = 0;
end { Open };

function OpenForWriteChar(var fp: AListFile; name: string): boolean;
begin
  Assign(fp,Name);
  {$I-}
  Rewrite(fp);
  {$I+}
  OpenForWriteChar := IOResult = 0;
end { Open };


procedure BoxInBox(refBox : labeledbox; var newBox : labeledbox;
                   begX : real; begY : real; endX : real; endY : real);
var
sDelX : integer;
sDelY : integer;
eDelX : integer;
eDelY : integer;
begin
  with refBox.ValueBox do
  begin
    sDelX := round(begX * Size.X);
    sDelY := round(begY * Size.Y);
    eDelX := round((1-endX) * Size.X);
    eDelY := round((1-endY) * Size.Y);
    newBox.ValueBox.Start.X := Start.X + sDelX;
    newBox.ValueBox.Start.Y := Start.Y + sDelY;
    newBox.ValueBox.Size.X := Size.X - (sDelX + eDelX);
    newBox.ValueBox.Size.Y := Size.Y - (sDelY + eDelY);
  end;
end;

procedure FillBox(aBox : box; pattern : word; color : word);
begin
  SetFillStyle(pattern,color);
  with aBox do
  Bar(Start.X +1,Start.Y+ 1,Start.X + Size.X-1,Start.y + Size.y-1);
end;

procedure CenterOfBox(theBox : box; var center : point);
begin
  center.X := theBox.Start.X + (theBox.Size.X div 2);
  center.Y := theBox.Start.Y + (theBox.Size.Y div 2);
end;

procedure DrawBox(theBox : box);
begin
  with theBox do
  begin
    rectangle(Start.X,Start.Y,Start.X+Size.X,Start.Y+Size.Y);
  end;
end;

procedure CenterTextInBox(theBox : box; text : string);
var
center : point;
begin
  CenterOfBox(theBox,center);
  SetTextJustify(CenterText,CenterText);
  OutTextXY(center.X,center.Y+1,text);
end;

procedure WriteTextValue(theBox : labeledBox; value : string);
begin
  FillBox(theBox.ValueBox,SolidFill,ValueColor);
  SetColor(ValueTextColor);
  CenterTextInBox(theBox.ValueBox,value);
end;

procedure ClearValueBox(theBox : labeledBox);
begin
  FillBox(theBox.ValueBox,SolidFill,ValueColor);
end;

procedure WriteTitle(theBox : labeledbox; title : string);
begin
  FillBox(theBox.TitleBox,SolidFill,TitleColor);
  SetColor(TitleTextColor);
  CenterTextInBox(theBox.TitleBox,title);
end;

procedure WriteNumberValue(var theBox : labeledBox; value : longint);
var
textValue : string;
begin
  Str(value,textValue);
  WriteTextValue(theBox,textValue);
end;

procedure MakeLabeledBox(var theBox : labeledBox; theTitle : string);
var
theTextHeight : word;
theMaxTextWidth : word;
textInfo : TextSettingsType;
begin
  GetTextSettings(textInfo);
  theTextHeight := textInfo.CharSize * 8;
  with theBox do
  begin
    TitleBox.Start.X := ValueBox.Start.X;
    TitleBox.Start.Y := ValueBox.Start.Y;
    TitleBox.Size.X := ValueBox.Size.X ;
    TitleBox.Size.Y := theTextHeight + 2;
    ValueBox.Start.Y := ValueBox.Start.Y + TitleBox.Size.Y;
    ValueBox.Size.Y := ValueBox.Size.Y - TitleBox.Size.Y;
  end;
  WriteTitle(theBox,theTitle);
  SetColor(BoxColor);
  DrawBox(theBox.TitleBox);
  DrawBox(theBox.ValueBox);
end;

procedure InitColors;
begin
  if GraphDriver <> HercMono then
  begin
    BoxColor := EGAWhite;
    TitleColor := EGAWhite;
    ValueTextColor := EGAYellow;
    ErrorColor := EGALightMagenta;
    ValueColor := EGABlue;
    TitleTextColor := EGABlack;
    ProfileColor := EGALightGreen;
    DriveColor := EGALightRed;
  end
  else
  begin
    BoxColor := White;
    TitleColor := White;
    ValueTextColor := White;
    ErrorColor := White;
    ValueColor := Black;
    TitleTextColor := Black;
    ProfileColor := White;
    DriveColor := EGALightRed;
  end;
end;

procedure InitMenu;
var
begX,begY,endX,endY,stepY,sizeY : real;
menuIndex : menuIndextype;
begin
  begX := 0;
  endX := 1;
  begY := 0;
  stepY := 0.0678;
  sizeY := 0.060;
  for menuIndex := FirstMenuIndex to LastMenuIndex do
  begin
    BoxInBox(VariablesBox,MenuBoxes[menuIndex],begX,begY,endX,begY+sizeY);
    MakeLabeledBox(MenuBoxes[menuIndex],MenuTitles[menuIndex]);
    begY := begY + stepY
  end;
end;

procedure Arrow(x : word; y : word; color : word);
begin
  Setcolor(color);
  Line(x-ArrowSize,y,x,y);
  Line(x-ArrowSize div 4 ,y+ArrowSize div 4 ,x,y);
  Line(x-ArrowSize div 4 ,y-ArrowSize div 4 ,x,y);
end;

procedure ArrowAtBox(theBox : labeledbox; color : word);
var
xArrow,yArrow : word;
begin
  with theBox.ValueBox do
  begin
    xArrow := Start.X-1;
    yArrow := Start.Y;
  end;
  Arrow(xArrow,yArrow,color);
end;

procedure MarkBox(theBox : Labeledbox);
begin
  ArrowAtBox(theBox,ValueTextColor);
end;

procedure UnMarkBox(theBox : Labeledbox);
begin
  ArrowAtBox(theBox,ValueColor);
end;

procedure MenuIndexUp;
begin
  UnMarkBox(MenuBoxes[MenuIndex]);
  if MenuIndex > FirstMenuIndex then MenuIndex := Pred(MenuIndex);
  MarkBox(MenuBoxes[MenuIndex]);
end;

procedure MenuIndexDown;
begin
  UnMarkBox(MenuBoxes[MenuIndex]);
  if MenuIndex < LastMenuIndex then MenuIndex := Succ(MenuIndex);
  MarkBox(MenuBoxes[MenuIndex]);
end;

procedure InitScreen;
var size : point;
var origin : point;
var aBox : box;
var screentitle : string;
begin
  with ScreenBox.ValueBox do
  begin
    Start.X :=0;
    Start.Y :=0;
    Size.X :=GetMaxX;
    Size.Y :=GetMaxY;
    ArrowSize := Size.X div 25;
  end;
  BoxInBox(ScreenBox,MainBox,0.01,0.01,0.99,0.99);
  MakeLabeledbox(MainBox,'Motor tuning program');
  WriteTextValue(MainBox,' ');
  BoxInBox(MainBox,VariablesBox,0.05,0.02,0.2,0.98);
  BoxInBox(MainBox,GraphBox,0.22,0.02,0.98,0.90);
  MakeLabeledBox(GraphBox,'Graph Box');
  BoxInBox(MainBox,MessageBox,0.22,0.92,0.98,0.99);
  MakeLabeledBox(MessageBox,'Message Box');
  WriteTextValue(MessageBox,'Hello world');
  InitMenu;
  MenuIndex := FirstMenuIndex;
  MenuIndexUp;                                 { forces refresh of arrow }
end;

procedure BumpWordParameter(var parameter : word; speed : menuspeed;limit : word);
var
newparameter : longint;
bump : real;
begin
  case speed of
    DownFast : bump := -0.1;
    DownSlow : bump := -0.01;
    UpFast : bump := 0.1;
    UpSlow : bump := 0.01;
    Zero,Max : bump := 0;
  end; {case}
  newparameter := round(parameter * (1 +bump));
  if parameter = newparameter then             { if it did't change... }
  begin
    if bump >0 then newparameter := parameter + 1;{ increment by one }
    if (bump < 0)  and (parameter > 0) then newparameter := parameter - 1;{ or decrement by one }
  end;
  if newparameter >limit then newparameter := limit;
  if newparameter <0 then newparameter := 0;
  if speed = Max then newparameter := limit;
  if speed = Zero then newparameter := 0;
  parameter := newparameter;
end;

procedure BumpIntParameter(var parameter : integer; speed : menuspeed;limit : integer);
var
newparameter : longint;
bump : real;
begin
  case speed of
    DownFast : bump := -0.1;
    DownSlow : bump := -0.01;
    UpFast : bump := 0.1;
    UpSlow : bump := 0.01;
    Zero,Max : bump := 0;
  end; {case}
  if parameter > 0 then newparameter := round(parameter * (1 +bump));
  if parameter <= 0 then newparameter := round(parameter * (1 -bump));
  if parameter = newparameter then             { if it did't change... }
  begin
    if bump >0 then newparameter := parameter + 1;{ increment by one }
    if (bump < 0)  and (parameter > -32767) then newparameter := parameter - 1;{ or decrement by one }
  end;
  if newparameter >limit then newparameter := limit;
  if newparameter < -32767 then newparameter := -32767;
  if speed = Max then newparameter := limit;
  if speed = Zero then newparameter := 0;
  parameter := newparameter;
end;

procedure BumpLongintParameter(var parameter : longint; speed : menuspeed; limit:longint);
var
newparameter : longint;
bump : real;
begin
  case speed of
    DownFast : bump := -0.1;
    DownSlow : bump := -0.01;
    UpFast : bump := 0.1;
    UpSlow : bump := 0.01;
    Zero,Max : bump := 0;
  end; {case}
  newparameter := round(parameter * (1 +bump));
  if parameter = newparameter then             { if it did't change... }
  begin
    if bump >0 then newparameter := newparameter + 1;{ increment by one }
    if (bump < 0)  and (newparameter > 0) then newparameter := newparameter - 1;{ or decrement by one }
  end;
  if newparameter >limit then newparameter := limit;
  if newparameter <0 then newparameter := 0;
  if speed = Max then newparameter := limit;
  if speed = Zero then newparameter := 0;
  parameter := newparameter;
end;

procedure DisplayTimeDiv;
var
nString : string;
begin
  Str(Times[TimeDivIndex],nString);
  WriteTextValue(MenuBoxes[TIM],nString + ' MS');
end;

function MotorStatusStr : string;
var
tstr : string;
begin
  if ReadParamWord(Axis,ProfileLoc) <> 0 then tstr := 'PRO' else tstr := '   ';
  if ReadParamWord(Axis,PIDLoc) <> 0 then tstr := tstr + ' PID';
  MotorStatusStr := tstr;
end;

procedure TogglePID;
begin
  if ReadParamWord(Axis,PIDLoc) <> 0 then WriteParamWord(Axis,PIDLoc,FalseF)
  else WriteParamWord(Axis,PIDLoc,TrueF);
end;

procedure ToggleProfile;
begin
  if ReadParamWord(Axis,ProfileLoc) <> 0 then WriteParamWord(Axis,ProfileLoc,FalseF)
  else WriteParamWord(Axis,ProfileLoc,TrueF);
end;

procedure RefreshDisplayNumbers;
var prestr,poststr,axisstr: string;
begin
  with StepParameters[Axis] do
  begin
    WriteNumberValue(MenuBoxes[KSI],KSIN);
    WriteNumberValue(MenuBoxes[KCI],KOFF);
    WriteNumberValue(MenuBoxes[KDFILI],KDFIL);
    WriteNumberValue(MenuBoxes[KII],KI);
    WriteNumberValue(MenuBoxes[ILI],KIL);
    WriteNumberValue(MenuBoxes[IKF1],KF1);
    WriteNumberValue(MenuBoxes[IKF2],KF2);
    WriteNumberValue(MenuBoxes[IKFF],KFF);
    WriteNumberValue(MenuBoxes[IKK],KK);
    str(PreScale,prestr);
    str(PostScale,poststr);
    WriteTextValue(MenuBoxes[SII],prestr+'/'+poststr);
    WriteNumberValue(MenuBoxes[STP],StepSize);
    WriteNumberValue(MenuBoxes[VEL],Velocity);
    WriteNumberValue(MenuBoxes[ACC],Acceleration);
    str(Axis,axisstr);
    WriteTextValue(MenuBoxes[MTR],axisstr+' '+MotorStatusStr);

{    WriteNumberValue(MenuBoxes[MTR],Axis);}
    TimeDivIndex := TDiv;
    DisplayTimeDiv;
  end;
end;

procedure BumpFilterParameters(bump : menuspeed);
begin
  with StepParameters[Axis] do
  begin
    case MenuIndex of
      KSI :
      begin
        BumpWordParameter(KSIN,bump,$FFFF);
        LoadKP(Axis,KSIN);
        WriteNumberValue(MenuBoxes[MenuIndex],KSIN);
      end;
      KCI :
      begin
        BumpIntParameter(KOFF,bump,$7FFF);
        LoadKD(Axis,KOFF);
        WriteNumberValue(MenuBoxes[MenuIndex],KOFF);
      end;
      KDFILI :
      begin
        BumpWordParameter(KDFIL,bump,65536-655);
        LoadKDFIL(Axis,KDFIL);
        WriteNumberValue(MenuBoxes[MenuIndex],KDFIL);
      end;
      KII :
      begin
        BumpLongIntParameter(KI,bump,$6FFFFFFF);
        LoadKI(Axis,KI);
        WriteNumberValue(MenuBoxes[MenuIndex],KI);
      end;
      ILI :
      begin
        BumpWordParameter(KIL,bump,$7FFF);
        LoadKIL(Axis,KIL);
        WriteNumberValue(MenuBoxes[MenuIndex],KIL);
      end;
      IKF1 :
      begin
        BumpWordParameter(KF1,bump,$FFFF);
        LoadKF1(Axis,KF1);
        WriteNumberValue(MenuBoxes[MenuIndex],KF1);
      end;
      IKF2 :
      begin
        BumpWordParameter(KF2,bump,$FFFF);
        LoadKF2(Axis,KF2);
        WriteNumberValue(MenuBoxes[MenuIndex],KF2);
      end;
      IKFF :
      begin
        BumpWordParameter(KFF,bump,$7FFF);
        LoadKFF(Axis,KFF);
        WriteNumberValue(MenuBoxes[MenuIndex],KFF);
      end;
      IKK :
      begin
        BumpIntParameter(KK,bump,$7FFF);
        LoadKK(Axis,KK);
        WriteNumberValue(MenuBoxes[MenuIndex],KK);
      end;
    end; {case}
  end; {with}
end;

procedure BumpPre(bump : menuspeed);
var
anumber : word;
psreal : real;
poststr,prestr,pwmstr,sampstr : string;
begin
  with StepParameters[Axis] do
  begin
    anumber := PWMRate;
    case bump of
      DownSlow : if anumber >10 then anumber := anumber -1;  { 1 khz}
      UpSlow : if anumber < 1953 then anumber := anumber +1; { 195 khz max }
    end;{case}
    PWMRate := anumber;
    psreal := PWMRate*65536;
    psreal := 256.0*psreal/(SysClk/100);
    PreScale := word(round(psreal));
    LoadPreScale(PreScale);
    str(PreScale,prestr);
    str(PostScale,postStr);
    WriteTextValue(MenuBoxes[SII],prestr+'/'+poststr);
    str((Sysclk/256)*(PreScale/65536):6:2,pwmstr);
    str((Sysclk/256)*(PreScale/65536)/PostScale:5:2,sampstr);
    WriteTextValue(MessageBox,'PWM rate: '+pwmstr+' Sample rate: '+sampstr);
  end; {with}
end;

procedure BumpPost(bump : menuspeed);
var
anumber : word;
prestr,poststr,pwmstr,sampstr : string;
begin
  with StepParameters[Axis] do
  begin
    anumber := PostScale;
    case bump of
      DownFast : if anumber >1 then anumber := anumber -1;
      UpFast : if anumber <32 then anumber := anumber +1;
    end;{case}
    LoadPostScale(anumber);
    PostScale := anumber;
    str(PreScale,prestr);
    str(PostScale,poststr);
    WriteTextValue(MenuBoxes[SII],prestr+'/'+poststr);
    str((Sysclk/256)*(PreScale/65536):6:2,pwmstr);
    str((Sysclk/256)*(PreScale/65536)/PostScale:5:2,sampstr);
    WriteTextValue(MessageBox,'PWM rate: '+pwmstr+' Sample rate: '+sampstr);
  end; {with}
end;

procedure BumpStep(bump : menuspeed);
begin
  with StepParameters[Axis] do
  begin
    case bump of
      DownSlow,DownFast :
      begin
        case StepSize of
           100 : StepSize := -100;
          -100 : StepSize := -200;
          -200 : StepSize := -500;
          -500 : StepSize := -1000;
          -1000 : StepSize := -2000;
          -2000 : StepSize := -5000;
          -5000 : Stepsize := -10000;
          -10000 : Stepsize := -20000;
          200 : StepSize := 100;
          500 : StepSize := 200;
          1000 : StepSize := 500;
          2000 : StepSize := 1000;
          5000 : StepSize := 2000;
          10000 : StepSize := 5000;
          20000 : Stepsize := 10000;
        end;{case StepSize}
      end;{down}
      UpSlow,UpFast :
      begin
        case StepSize of
          -100 : StepSize := 100;
          -200 : StepSize := -100;
          -500 : StepSize := -200;
          -1000 : StepSize := -500;
          -2000 : StepSize := -1000;
          -5000 : Stepsize := -2000;
          -10000 : Stepsize := -5000;
          -20000 : Stepsize := -10000;
          100 : StepSize := 200;
          200 : StepSize := 500;
          500 : StepSize := 1000;
          1000 : StepSize := 2000;
          2000 : StepSize := 5000;
          5000 : StepSize := 10000;
          10000 : StepSize := 20000;
        end;{case StepSize}
      end;{up}
    end;{case bump}
    WriteNumberValue(MenuBoxes[STP],StepSize);
  end;{with}
end;

procedure BumpVel(bump : menuspeed);
var
countstr,rpmstr,cpsampstr : string;
rate,cps,cpsamp,rps,rpm : real;
begin
  with StepParameters[Axis] do
  begin
    BumpLongintParameter(Velocity,bump,$6FFFFFFF);
    LoadVelocity(Axis,Velocity);
    rate :=(Sysclk/256)*(PreScale/65536)/PostScale;
    cps := rate*(Velocity/$00FFFFFF);
    cpsamp := cps/rate;
    rps := cps/(EncoderLines*4);
    rpm := rps*60;
    str(rpm:4:4,rpmstr);
    str(cps:6:0,countstr);
    str(cpsamp:2:4,cpsampstr);

    WriteTextValue(MessageBox,'C/S = '+countstr+' C/Samp = '+cpsampstr+' RPM = '+rpmstr);

    WriteNumberValue(MenuBoxes[VEL],Velocity);
  end;
end;

procedure BumpAcc(bump : menuspeed);
begin
  with StepParameters[Axis] do
  begin
    BumpLongintParameter(Acceleration,bump,$6FFFFFFF);
    LoadAcceleration(Axis,Acceleration);
    WriteNumberValue(MenuBoxes[ACC],Acceleration);
  end;
end;

procedure DeSelectMotor(motor : word );
begin
{  TurnOffMotor(Axis); }
end;

procedure SelectMotor(motor : word );
begin
{  TurnOnMotor(Axis); }
  LEDs(motor,EncPLoc); { let LEDS monitor the primary encoder}
end;

procedure BumpMtr(bump : menuspeed);
var
newMotor : word;
begin
  newMotor := Axis;
  case bump of
    DownFast : if Axis >0 then newMotor := Axis -1;
    UpFast : if Axis <(Naxis-1) then newMotor := Axis +1;
    DownSlow : TogglePID;
    UpSlow : ToggleProfile;

  end;{case}
  If newMotor <> Axis then
  begin
    DeSelectMotor(Axis);
    SelectMotor(NewMotor);
    Axis := newMotor;
    with StepParameters[Axis] do
    begin
      LoadPreScale(PreScale);
      LoadPostScale(PostScale);
    end;
  end;
  RefreshDisplayNumbers;
end;

procedure UpTimeIndex;
begin
 if TimeDivIndex < 8 then TimeDivIndex := TimeDivIndex + 1;
end;

procedure DownTimeIndex;
begin
  if TimeDivIndex > 1 then TimeDivIndex := TimeDivIndex -1;
end;


procedure BumpTime(bump : menuspeed);
begin
  case bump of
    DownFast,DownSlow : DownTimeIndex;
    UpFast,UpSlow : UpTimeIndex;
  end;{case}
  StepParameters[Axis].TDiv := TimeDivIndex;
  DisplayTimeDiv;
end;

procedure BumpOtherParameters(bump : menuspeed);
begin
  case MenuIndex of
    SII :
        case bump of
          UpFast,DownFast : BumpPost(bump);
          UpSlow,DownSlow : BumpPre(bump);
        end;
    STP :  BumpStep(bump);
    VEL :  BumpVel(bump);
    ACC :  BumpAcc(bump);
    MTR :  BumpMtr(bump);
    TIM :  BumpTime(bump);
  end; {case}
end;

procedure BumpMenuItem(bump : menuspeed);
begin
  case MenuIndex of
    KSI..IKK : BumpFilterParameters(bump);
    SII..TIM : BumpOtherParameters(bump);
  end; {case}
end;

procedure InitFilterParameters(Axis: byte);
var psreal : real;
begin

  with StepParameters[axis] do
  begin
    KSIN := 15000;                                { relatively low current }
    KI := 0;                                      { integrated error }
    KOFF := 0;                                    { offset = 0 initially }
    KDFIL := 0;
    KIL := 0;                                     { low integration limit for now }
    KF1 := 0;
    KF2 := 0;
    KFF := 0;
    KK := 0;
    PWMRate := 1750;  { 175KC }
    psreal := PWMRate*65536;
    psreal := 256.0*psreal/(SysClk/100);
    PreScale := word(round(psreal));
    PostScale := 6;
    TDiv := 1; {for 5ms/div};
    { now that we have loaded the record, update the hardware }
    LoadKP(axis,KSIN);
    LoadKI(axis,KI);
    LoadKD(axis,KOFF);
    LoadKDFIL(axis,KDFIL);
    LoadKIL(axis,KIL);
    LoadKF1(axis,KF1);
    LoadKF2(axis,KF2);
    LoadKFF(axis,KFF);
    LoadKK(axis,KK);

    LoadPreScale(PreScale);
    LoadPostScale(PostScale);
    LoadDirectionFlag(axis,TrueF);
    LoadPosErrForStop(Axis,50000);
    SetHomePos(axis);
    StepSize := 20000;
    Velocity :=     8000000;                    { infinitely quick! }
    Acceleration := 15000;                    { almost infinitely quick! }
    LoadAcceleration(Axis,Acceleration);
    LoadVelocity(Axis,Velocity);
    TurnOnMotor(Axis);
  end; {with}
end;

procedure RefreshHardWare(index : byte);
begin
  with StepParameters[index] do
  begin
    LoadKP(index,KSIN);
    LoadKI(index,KI);
    LoadKD(index,KOFF);
    LoadKDFIL(index,KDFIL);
    LoadKIL(index,KIL);
    LoadKF1(index,KF1);
    LoadKF2(index,KF2);
    LoadKFF(index,KFF);
    LoadKK(index,KK);
    LoadAcceleration(index,Acceleration);
    LoadVelocity(index,Velocity);
    if index = 0 then
    begin
      LoadPreScale(PreScale);
      LoadPostScale(PostScale);
    end;
  end; {with}
end;

function InitGraphics : boolean;
var
GraphMode : integer;
ErrorCode : integer;
x : integer;
y : integer;
begin
  GraphDriver := Detect;
  InitGraph(GraphDriver,GraphMode,'');
  ErrorCode := GraphResult;
  If ErrorCode <> GrOk then
  begin
    writeln('BGI driver missing? ... aborting program');
    halt(2);
  end;
end;

procedure ShowGrid;
var
index: word;
step : real;
istep : word;
scaledData : real;
begin
  Setcolor(TitleColor);
  SetLineStyle(UserBitLn,$8888,NormWidth);
  with GraphBox.ValueBox do
  begin
    SetViewPort(Start.X,Start.Y,Start.X+Size.X,Start.Y+Size.Y,ClipOn);
    for index := 1 to HDivsPerScreen -1 do
    begin
        step := Size.X/HDivsPerScreen * index;
        istep := trunc(step + 0.5);
        Line(istep,1,istep,Size.Y-1);
    end;
    for index := 1 to VDivsPerScreen -1 do
    begin
        step := Size.Y/VDivsPerScreen * index;
        istep := trunc(step + 0.5);
        Line(1,istep,Size.X-1,istep);
    end;
  end;
  SetLineStyle(SolidLn,0,NormWidth);
  SetViewPort(0,0,GetMaxX,GetMaxY,ClipOn);
end;

procedure ClearGraph;
begin
  ClearValueBox(GraphBox);
  ShowGrid;
end;


procedure DisplayGraph;
var
astring : string;
index : word;
sindex : word;
scaledData : real;
scaleFactor : real;
escalefactor : real;
drScaleFactor : real;
xscalefactor : real;
offset : real;
droffset : real;
fixedDrive : integer;
begin
  with GraphBox.ValueBox, StepParameters[Axis] do
  begin
  scaleFactor := (Size.Y / StepSize) *0.80;
  escaleFactor := (Size.Y / (VDivsPerScreen *10));
  drScaleFactor := (Size.Y / 65536) *0.80;
  offset := (Size.Y / 10);
  droffset := (Size.Y / 2);
    xscalefactor := (SamplesPerScreen/(Size.X -2));
    SetViewPort(Start.X,Start.Y,Start.X+Size.X,Start.Y+Size.Y,ClipOn);
    for index := 1 to Size.X -1 do
    begin
      sindex := round(index*xscalefactor);
      scaledData := ResponseTable[sindex] * ScaleFactor;
      scaledData := (Size.Y - round(offset) - scaledData);
      if GoodSample[sindex] then PutPixel(index,round(scaledData),ValueTextColor);
      scaledData := (ProfileTable[sindex] - ResponseTable[sindex]) * escaleFactor;
      scaledData := (Size.Y - round(droffset) - scaledData);
      if GoodSample[sindex] then PutPixel(index,round(scaledData),ErrorColor);
      scaledData := ProfileTable[sindex] * ScaleFactor;
      scaledData := (Size.Y - round(offset) - scaledData);
      if GoodSample[sindex] then PutPixel(index,round(scaledData),ProfileColor);
      FixedDrive := DriveTable[sindex];
      scaledData := fixedDrive * drScaleFactor;
      scaledData := (Size.Y - round(droffset) - scaledData);
      if GoodSample[sindex] then PutPixel(index,round(scaledData),DriveColor);
    end;
  end;
  SetViewPort(0,0,GetMaxX,GetMaxY,ClipOn);
end;

procedure StartRateGen(rate : real);
var
samplerate,phaser : real;
phasek : longint;
begin
  { setup the phase accumulator to generate the desired DAQ rate }
  with StepParameters[Axis] do
  begin
    samplerate := (Sysclk/256)*(PreScale/65536)/PostScale;
  end;
  phaser := rate * TwoToThe31/samplerate;
  phasek := trunc(phaser + 0.5);
  WriteParam(0,GPhaseKLoc,phasek);
end;

procedure StartRateGenMax;
begin
  { if we cant sample at the desired rate }
  { setup the phase accumulator to toggle once per sample }
  WriteParamWord(0,GPhaseKLoc,$0000);
  WriteParamWord(0,GPhaseKLoc+1,$8000);
end;


procedure SetupRateEvents;
var rateevent : Event;
begin
  with rateevent do
  begin
    { 2 global events set DAQFlag on rising and falling edges of MSB of GPhaseA }
    ER_condptr := GPhaseALoc +1;{ MSW of GPhaseA}
    ER_condxor := $0000;        { looking for rising edge }
    ER_condand_ptr := $8000;    { looking at MSB of MSW = bit 31}
    ER_condflag  := $FFFF;      { old data set high so we always wait for edge }
    ER_operptr := DAQFlagLoc or EdgeBit; {  DAQ flag + edge detect mask }
    ER_operxor := $0000;        { dont care what was at our source }
    ER_operand := $0000;        { dont care what was at our source }
    ER_operor_src  := $FFFF;    { Set the flag to $FFFF }
    GlobalEvent(1,rateevent);   { install rising edge detect event (gevent1) }
    ER_condxor := $FFFF;        { looking for falling edge }
    GlobalEvent(2,rateevent);   { install falling edge detect event (gevent2) }
  end;
end;

procedure SetupRateEventsForPolledMode;
var rateevent : Event;
begin
  with rateevent do
  begin
    { 4 global events set DAQFlag/status on rising and falling edges of MSB of GPhaseA }
    ER_condptr := GPhaseALoc +1;{ MSW of GPhaseA}
    ER_condxor := $0000;        { looking for rising edge }
    ER_condand_ptr := $8000;    { looking at MSB of MSW = bit 31}
    ER_condflag  := $FFFF;      { old data set high so we always wait for edge }
    ER_operptr := HCommRWLoc or EdgeBit; {  DAQ flag in host status reg + edge detect mask }
    ER_operxor := $0000;        { dont care what was at our source }
    ER_operand := $0000;        { dont care what was at our source }
    ER_operor_src  := PolledDAQFlag;    { Set the flag = PolledDAQ flag }
    GlobalEvent(1,rateevent);   { install rising edge detect event (gevent1) }
    ER_condxor := $FFFF;        { looking for falling edge }
    GlobalEvent(2,rateevent);   { install falling edge detect event (gevent2) }
    ER_condptr := GPhaseALoc +1;{ MSW of GPhaseA}
    ER_condxor := $0000;        { looking for rising edge }
    ER_condand_ptr := $8000;    { looking at MSB of MSW = bit 31}
    ER_condflag  := $FFFF;      { old data set high so we always wait for edge }
    ER_operptr := DAQFlagLoc or EdgeBit; {  DAQ flag in host status reg + edge detect mask }
    ER_operxor := $0000;        { dont care what was at our source }
    ER_operand := $0000;        { dont care what was at our source }
    ER_operor_src  := $FFFF;    { Set the DAQ flag }
    GlobalEvent(3,rateevent);   { install rising edge detect event (gevent1) }
    ER_condxor := $FFFF;        { looking for falling edge }
    GlobalEvent(4,rateevent);   { install falling edge detect event (gevent2) }
  end;
end;

procedure StopRateGen;
begin
  WriteParam(0,GPhaseKLoc,0);      { set rate to 0 }
  Writeparam(0,GPhaseALoc,0);      { clear the phase acc}
  WriteParamWord(0,DAQFlagLoc,0);  { clear flag }
end;

procedure SetupTraceBufferEvents(laxis : byte;div1trig: word);
var daqevent : Event;
begin
  with daqevent do
  begin
    WriteParamWord(0,TrigVal,div1trig);        { Setup up our first division trigger value }
    { first install 3 events to acquire realpos,despos, drive}
    { when the DAQ sample flag is set }
    ER_condptr := DAQFlagLoc;   { our Data sample flag }
    ER_condxor := $0000;        { looking for true }
    ER_condand_ptr := $FFFF;    { looking for any old bit}
    ER_condflag  := $0000;
    ER_operptr := TBuffPushLoc + copybit;  { data goes to trace Buffer }
    ER_operxor := $0000;        { dont mess with the data }
    ER_operand := $FFFF;        { dont mess with the data }
    ER_operor_src := EncPLoc;   { source data for 1st sample is real pos }
    AxisEvent(laxis,1,daqevent); { install event to push RealPos (axis event 1) }
    ER_operor_src := DesPosLoc; { source data for 2nd sample is DesPos}
    AxisEvent(laxis,2,daqevent); { install event to push DesPos (axis event 2) }
    ER_operor_src := PWMLoc;    { source data for 3rd sample is Drive}
    AxisEvent(laxis,3,daqevent); { install event to push Drive (axis event 3) }
    { Set Go when we have collected one division worth of data in tracebuffer }
    ER_condptr := TBuffCntLoc or CompareBit; { pointer to pointer to A = FIFO count }
    ER_condxor := $FFFF;                   { reverse compare so we do A >= B }
    ER_condand_ptr := TrigVal;             { pointer to trigger value }
    ER_condflag  := $FFFF;                 { set high to avoid false trigger}
    ER_operptr := GoLoc + EdgeBit; { Edge detect compare and set Go if true }
    ER_operxor := $0000;        { Dont Care }
    ER_operand := $0000;        { Dont Care }
    ER_operor_src := $FFFF;     { Set Go on edge of compare }
    AxisEvent(laxis,4,daqevent); { install last event to unconditionally clear DAQ flag }
    { event 5 clears the DAQ sample flag so we dont do any more readings }
    { till the DAQ sample flag gets set again by the GPhaseA event }
    ER_condptr := TrueLoc;      { Always }
    ER_condxor := $0000;        { looking for true }
    ER_condand_ptr := $FFFF;    { looking for any old bit}
    ER_condflag := $0000;
    ER_operptr := DAQFlagLoc;   { after we've acquired the data, clear the DAQ flag }
    ER_operxor := $0000;        { Dont Care }
    ER_operand := $0000;        { zero it }
    ER_operor_src := $0000;     { zero it }
    AxisEvent(laxis,5,daqevent); { install last event to unconditionally clear DAQ flag }
  end;
end;

procedure SetupTraceBufferEventsForPolledMode(laxis : byte;div1trig: word);
{ not used yet }
var daqevent : Event;
begin
  WriteParamWord(0,TrigVal,div1trig);        { Setup up our first division trigger value }
  with daqevent do
  begin
    { setup a counter for DAQ events }
    ER_condptr := DAQFlagLoc;            { our DAQ flag }
    ER_condxor := $0000;                   { looking for rising edge }
    ER_condand_ptr := $0FFF;               { mask off unrelated bits }
    ER_condflag  := $FFFF;                 { set high to avoid false trigger}
    ER_operptr := SampleCnt + AddBit;      { if DAQ flag set, inc samplecnt if true }
    ER_operxor := $0001;        { add 1 }
    ER_operand := $FFFF;        { dont mess with count }
    ER_operor_src := $0000;     { dont mess with count }
    AxisEvent(laxis,1,daqevent); { install counter event }

    { Set Go when we have gotten to division worth of data }
    ER_condptr := SampleCnt + CompareBit; { pointer to pointer to A = SampleCnt count }
    ER_condxor := $FFFF;                   { reverse compare so we do A >= B }
    ER_condand_ptr := TrigVal;             { pointer B = trigger value }
    ER_condflag  := $FFFF;                 { set high to avoid false trigger}
    ER_operptr := GoLoc + EdgeBit; { Edge detect compare and set Go if true }
    ER_operxor := $0000;        { Dont Care }
    ER_operand := $0000;        { Dont Care }
    ER_operor_src := $FFFF;     { Set Go on edge of compare }
    AxisEvent(laxis,2,daqevent); { install trigger event }
    ER_condptr := TrueLoc;      { Always }
    ER_condxor := $0000;        { looking for true }
    ER_condand_ptr := $FFFF;    { looking for any old bit}
    ER_condflag := $0000;
    ER_operptr := DAQFlagLoc;   { after we've acquired the data, clear the DAQ flag }
    ER_operxor := $0000;        { Dont Care }
    ER_operand := $0000;        { zero it }
    ER_operor_src := $0000;     { zero it }
    AxisEvent(laxis,3,daqevent); { install last event to unconditionally clear DAQ flag }
  end;
end;

procedure DisableTraceBufferEvents(axis : byte);
begin
  WriteParamWord(axis,EventsLoc,0);
end;

procedure AcquireStepResponseData;
var
index : word;
polledsample : word;
rpt: real;
samplerate,daqrate,timeperscreen : real;
samplesperdiv : word;
perrstr,nerrstr,aString,bString,cString,dString,rString,ptstr,tostr: string;
begin
  StopRateGen; { just in case it happens to be on }
  with StepParameters[Axis] do
  begin
    samplerate := (Sysclk/256)*(PreScale/65536)/PostScale;
    timeperscreen := Times[TDiv]/1000 * HDivsPerScreen;  { in seconds }
  end;
  daqrate := SamplesPerVariable/timeperscreen;
  LoadProcTimer(0);
  LoadTimeOut(0);
  ClearPosErrs(Axis);
  LoadPosition(Axis,StepParameters[Axis].StepSize);
  if HaveTraceBuffer then SetupRateEvents else SetupRateEventsForPolledMode;
  if daqrate > samplerate then
  begin
    { if the desired DAQ sample rate is greater than the DMC sample rate   }
    { best we can do is to sample at the DMC sample rate: So calculate the }
    { number of samples per screen accordingly }
    SamplesPerScreen := trunc(samplerate*timeperscreen);
  end
  else
  begin
    SamplesPerScreen := trunc(daqrate*timeperscreen);
  end;
  samplesperdiv := SamplesPerScreen div HDivsPerScreen;
  if HaveTraceBuffer then
  begin
    SetupTraceBufferEvents(Axis,samplesperdiv*NTraced);
    { We have set up the data aquisition events, and set the GO trigger }
    { to the value the trace buffer count will be at the first display division }
    delay(10);
    ClearTraceBuffer; {just in case we triggered some event whilst messing about }
  end
  else
  begin
    SetupTraceBufferEventsForPolledMode(Axis,samplesperdiv);
    delay(10);
    WriteParamWord(0,SampleCnt,0);
    for index := 1 to SamplesPerScreen do { in polled mode we may lose a few samples }
    begin                                 { so we set up an array to mark the good ones }
      GoodSample[index] := false;
    end;
  end;
  WriteTextValue(MessageBox,'Acquiring data');
  if daqrate > samplerate then
  begin
    daqrate := samplerate;
    StartRateGenMax;
    { if we cant sample fast enough, set the rate generator to one }
    { DAQ sample per DMC sample, start the rate generator, and away we go! }
  end
  else
  begin
    StartRateGen(daqrate);
    { start the rate generator at the desired rate and away we go! }
  end;
  if HaveTraceBuffer then
  begin
    while ReadTraceBuffCount < SamplesPerScreen * NTraced do;
    {wait till we have acquired the needed number of samples }
    {then shut down the DAQ events }
  end
  else
  begin
    for index := 1 to SamplesPerScreen do
    begin
      while (BusReadStatus and $0FFF) <> PolledDAQFlag do;
      { wait for DAQ flag in status reg }
      { then collect our data: }
      polledsample := ReadParamWord(Axis,SampleCnt);
      ResponseTable[polledsample] := ReadRealPositionShort(Axis);
      ProfileTable[polledsample] :=  ReadDesPositionShort(Axis);
      DriveTable[polledsample] :=  ReadDrive(Axis);
      { Lastly, clear our DAQ flag}
      WriteParamWord(0,HCommRWLoc,0);
      GoodSample[polledsample] := true;
    end;
  end;
  DisableTraceBufferEvents(Axis);
  StopRateGen;
  str(ReadPosError(Axis),perrstr);
  str(ReadNegError(Axis),nerrstr);
  WriteTextValue(MessageBox,'Stopping..........');
  Stop(Axis);
  WriteTextValue(MessageBox,'Stopped!');
  if HaveTraceBuffer then
  begin
    for index := 1 to SamplesPerScreen do
    begin
      ResponseTable[index] := ReadTraceBuffer;
      ProfileTable[index] :=  ReadTraceBuffer;
      DriveTable[index] :=  ReadTraceBuffer;
      GoodSample[index] := true;
    end;
  end;
  str(ReadRealPositionShort(Axis),rString);
  str(ReadDesPositionShort(Axis),dString);
  rpt := 1e6*ReadProcTimer*2/SysClk;
  str(rpt:2:2,ptstr);
  str(ReadTimeOut,tostr);
  ShowGrid;
  DisplayGraph;
  LoadPosition(Axis,0);          { return home... }
  WriteTextValue(MessageBox,'Returning home ');
  StartTrajectory(Axis);
  WaitForMoveDone(Axis);
  str(ReadParamWord(Axis,60),astring);
  str(ReadParamWord(Axis,61),bstring);
  str(ReadParamWord(Axis,62),cstring);
  str(ReadParamWord(Axis,63),dstring);

  WriteTextValue(MessageBox,'A= '+astring+' B= '+ bstring+' C= '+cstring+' D= '+dstring+' PT= '+ptstr+' uS, TO= '+tostr);
end;



procedure StepGraph;
begin
  if ProfileOnQ(Axis) then AcquireStepResponseData
  else
  WriteTextValue(MessageBox,'Profile disabled - no motion possible');

end;

procedure ReInitParameters;
begin
  InitFilterParameters(Axis);
  RefreshDisplayNumbers;
end;


procedure ExportSettings;
var index : byte;
begin
  If OpenForWrite(TheAIFile,TheAIFileName) then
  begin
    for index := 0 to Naxis -1 do
    begin
      write(TheAIFile,StepParameters[index]);
    end;
    close(TheAIFile);
    WriteTextValue(MessageBox,'Settings exported successfully');
  end
  else
  begin
    WriteTextValue(MessageBox,'Unable to export settings');
  end;
end;

procedure UnListSettings;
begin
end;

procedure ListSettings;
var index : byte;
begin
  If OpenForWriteChar(TheAlistFile,TheAlistFileName) then
  begin
    for index := 0 to Naxis -1 do
    begin
      writeln;
      writeln(TheAListFile,'************************************************');
      writeln(TheAlistFile,'Axis ',index);
      with StepParameters[index] do
      begin
        writeln(TheAlistFile,'StepSize ',StepSize);
        writeln(TheAlistFile,'Velocity ',Velocity);
        writeln(TheAlistFile,'Acceleration ',Acceleration);
        writeln(TheAlistFile,'KSI ',KSIN);
        writeln(TheAlistFile,'KOF ',KOFF);
        writeln(TheAlistFile,'KDFIL ',KDFIL);
        writeln(TheAlistFile,'KI ',KI);
        writeln(TheAlistFile,'KIL ',KIL);
        writeln(TheAlistFile,'KF1 ',KF1);
        writeln(TheAlistFile,'KF2 ',KF2);
        writeln(TheAlistFile,'KFF ',KFF);
        writeln(TheAlistFile,'KK ',KK);
        writeln(TheAlistFile,'Prescale ',Prescale);
        writeln(TheAlistFile,'PostScale ',PostScale);
      end;
    end;
    close(TheAListFile);
    WriteTextValue(MessageBox,'Settings printed to '+TheAlistFileName);
  end
  else
  begin
    WriteTextValue(MessageBox,'Unable to list settings');
  end;
end;

procedure ImportSettings;
var index : byte;
begin
  If OpenForRead(TheAIFile,TheAIFileName) then
  begin
    for index := 0 to Naxis -1 do
    begin
      read(TheAIFile,StepParameters[index]);
    end;
    close(TheAIFile);
    RefreshDisplayNumbers;
    for index := 0 to Naxis -1 do
    begin
      RefreshHardWare(index);
    end;
    WriteTextValue(MessageBox,'Settings imported successfully');
  end
  else
  begin
    WriteTextValue(MessageBox,'Unable to import file!');
  end;
end;

procedure SaveSettings;
begin
  SavedParameters := StepParameters[Axis];
  WriteTextValue(MessageBox,'Saving settings');
end;

procedure RecallSettings;
begin
  StepParameters[Axis] := SavedParameters;
  WriteTextValue(MessageBox,'Recalling saved settings');
  RefreshDisplayNumbers;
  RefreshHardWare(axis);
end;

procedure KeyLoop;
var
keyChar : char;
done : boolean;

begin
  done := false;
  repeat
    KeyChar := ReadKey;
    case KeyChar of
      'R','r'   : RecallSettings;
      'S','s'   : SaveSettings;
      'E','e'   : ExportSettings;
      'I','i'   : ImportSettings;
      'L','l'   : ListSettings;
      'U','u'   : UnlistSettings;
      'Z','z'   : BumpMenuItem(Zero);
      'M','m'   : BumpMenuItem(Max);
      'Q','q'   : done := true;
      #0 :
      begin
        KeyChar := ReadKey;
        case KeyChar of
          #75  : BumpMenuItem(DownFast);
          #79  : BumpMenuItem(DownSlow);
          #81  : BumpMenuItem(UpSlow);
          #77  : BumpMenuItem(UpFast);
          #82  : StepGraph;
          #83  : ClearGraph;
          #119 : ReInitParameters;
{          #73  : ShowGrid;}
          #72  : MenuIndexUp;
          #80  : MenuIndexDown;
          #45  : done := true;
        end; {function key case}
      end; {any key case}
    end;
  until done = true;
end;

procedure ShutDown;
var axis: word;
begin
  for axis := 0 to Naxis -1 do
  begin
    TurnOffMotor(axis);
  end;
  CloseGraph;
end;

function GetParms : boolean;
var
retcode : integer;
tempstr : string;
templong : longint;
tempword : word;
begin
  GetParms := false;
  if length(ParamStr(1)) <> 0 then
  begin
    tempstr := ParamStr(1);
    tempstr := UpString(tempstr);
    if not HexLongRead(tempstr,templong) then
    begin
      TheComPort := 0;
      if tempstr = 'COM1' then TheComPort := COM1;
      if tempstr = 'COM2' then TheComPort := COM2;
      if tempstr = 'COM3' then TheComPort := COM3;
      if tempstr = 'COM4' then TheComPort := COM4;
      if TheComPort = 0 then
      begin
        WriteTextValue(MessageBox,'Undecipherable COM port - bail out!');
        delay(3000);
        CloseGraph;
        halt(2);
      end;
    end
    else
    begin
      if (templong <$100) or (templong >$3F8) then
      begin
        WriteTextValue(MessageBox,'Out of range COM port address - bail out!');
        delay(3000);
        CloseGraph;
        halt(2);
      end;
      if (templong and 7) <> 0 then
      begin
        WriteTextValue(MessageBox,'Illegal COM port address - bail out!');
        delay(3000);
        CloseGraph;
        halt(2);
      end;
      TheComPort := word(templong);
    end;
  end;
  if length(ParamStr(2)) <> 0 then
  begin
    tempstr := ParamStr(2);
    val(tempstr,templong,retcode);
    if (templong <300) or (templong >115200) or (retcode <> 0) then
    begin
        WriteTextValue(MessageBox,'Illegal Baud rate - bail out!');
        delay(3000);
        CloseGraph;
        halt(2);
    end
    else
    begin
      TheBaudRate := templong;
      GetParms := true
    end;
  end;
end;

function SetupPCI : boolean;
var
busnum,devnum: byte;
configbase : word;
hstring : string;
begin
  SetupPCI := false;
  if PCIBIOSPresent then
  begin
    if FindPCIDevice(N030DeviceID,N030VendorID,0,busnum,devnum) then
    begin
      if ReadPCIConfigWord(busnum,devnum,N030PCIBaseAddress1,ConfigBase) then
      begin
        ConfigBase := ConfigBase and $FFFE;
      end;
      if ReadPCIConfigWord(busnum,devnum,N030PCIBaseAddress2,ConfigBase) then
      begin
        ConfigBase := ConfigBase and $FFFE;
        BasePort := ConfigBase;
        SetupPCI := true;
        hstring := HexString(BasePort,4);
        WriteTextValue(MessageBox,'Found PCI card -- base address is: 0x'+hstring);
      end;
    end
    else
    begin
      WriteTextValue(MessageBox,'Didn''t find PLX9030');
    end;
  end
  else
  begin
    WriteTextValue(MessageBox,'Didn''t find PCI BIOS');
  end;
  Delay(2000);
end;


procedure InitializeMot;
var
nString : string;
screentitle,hrev,majrev,minrev,naxstr,tbstr: string;
hrevnum,majrevnum,minrevnum: word;
test: word;
foodata : longint;
begin
  if BusIntfc = true then
  begin
    If not SetupPCI then BasePort := DefaultPC104Baseport;
    CommandPort := BasePort+CommandPortOffset;
    DataPort :=    BasePort+DataPortOffset;
    ROMAddPort := BasePort+ROMAddOffset;
    ROMDataPort := BasePort+ROMDataOffset;
  end;
  if not HardwareThere then
  begin
    WriteTextValue(MessageBox,'No Hardware signature... Try Again!');
    delay(3000);
  end;
  if not HardwareThere then
  begin
    WriteTextValue(MessageBox,'No Hardware signature... Bail out!');
    delay(3000);
    CloseGraph;
    halt(2);
  end;
  WriteTextValue(MessageBox,'Controller reset... proceeding with set-up');
  delay(1000);
  majrevnum := ReadMajorSWVersion;
  minrevnum := ReadMinorSWVersion;
  hrevnum := ReadHWVersion;
  Str(majrevnum,majrev);
  Str(minrevnum,minrev);
  Str(hrevnum,hrev);
  screentitle := 'SCode Rev. '+ majrev+'.'+minrev+' HW rev. '+ hrev;
  WriteTitle(MainBox,screentitle);
  delay(1000);
  if hrevnum < 20 then
  begin
    WriteTextValue(MessageBox,'SOFTDMC HARDWARE REVISION TOO LOW');
    delay(3000);
    CloseGraph;
    halt(2);
 end;
  if majrevnum <> MajorRevLoc then
  begin
    WriteTextValue(MessageBox,'SOFTDMC FIRMWARE REVISION MISMATCH');
    delay(3000);
    CloseGraph;
    halt(2);
  end;
  ResetMot;
  WriteTextValue(MessageBox,'Checking for trace buffer...');
  TraceBufferSize := GetTraceBufferSize;
  if TraceBufferSize > 255 then
  begin
    HaveTraceBuffer := true;
    Str(TraceBufferSize,tbstr);
    tbstr := 'Trace buffer size = '+tbstr;
    SamplesPerVariable := TraceBufferSize div Ntraced;
    EnableTBFIFOMode;
  end
  else
  begin
    HaveTraceBuffer := false;
    if BusIntfc = false then
    begin
      WriteTextValue(MessageBox,'Serial interface and no TraceBuffer -- Bailout!');
      delay(3000);
      CloseGraph;
      halt(2);
    end;
    tbstr := ' No trace buffer';
    SamplesPerVariable := PolledModeSamplesPerVariable;
  end;
  Naxis := ReadNaxis;
  Str(Naxis,naxstr);
  if (Naxis > 8) or (Naxis < 1) then
  begin
    WriteTextValue(MessageBox,'Ridiculous # of Axis: '+naxstr+' -- Bailout!');
    delay(3000);
    CloseGraph;
    halt(2);
  end;
  for Axis := 0 to Naxis-1 do
  Begin
    InitFilterParameters(Axis);
    SavedParameters := StepParameters[0];
    Str(Axis,nString);
    WriteTextValue(MessageBox,'Motor '+nString+' initialized');
    delay(200);
  end;
  screentitle :=naxstr+' axis tuning program -- Code Rev. '+ majrev+'.'+minrev+' HW rev. '+ hrev+'  '+tbstr;
  WriteTitle(MainBox,screentitle);
  Axis := 0;                             { start out with motor 0 }
  SelectMotor(Axis);
  ClearGraph; {so our grid will be there}
end;

{*****************************************************************************}

begin                                          {main}
  if not InitGraphics then
  begin
    writeln('DMCTUNE must be run on a system with graphics capability');
    writeln('BGI driver missing? ');
    halt(2);
  end;
  InitColors;
  InitScreen;
  if GetParms then
  begin
    BusIntfc := false;
    InitializeSerInterface(TheComPort,TheBaudRate);
    delay(100);
    TossChars(TheComPort,20);
  end
  else
  begin
    BusIntfc := true;
  end;
  InitializeMOT;
  RefreshDisplayNumbers;
  WriteTextValue(MessageBox,'READY');
  KeyLoop;
  Shutdown;
  writeln('that''s all folks...');
end.
