library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity scounter is
   port ( 
	obus: out STD_LOGIC_VECTOR (7 downto 0);
	ibus: in STD_LOGIC_VECTOR (7 downto 0);
	quada: in STD_LOGIC;
	quadb: in STD_LOGIC;
	index: in STD_LOGIC;
	ccrloadcmd: in STD_LOGIC;
	ccrreadcmd: in STD_LOGIC;
	countoutreadcmd: in STD_LOGIC;
	byteselect: in STD_LOGIC_VECTOR (1 downto 0);
	countlatchcmd: in STD_LOGIC;
	clk: in STD_LOGIC
	);
end scounter;

architecture behavioral of scounter is

signal count: STD_LOGIC_VECTOR (31 downto 0);
signal countoutlatch: STD_LOGIC_VECTOR (31 downto 0);
signal quada1: STD_LOGIC;
signal quada2: STD_LOGIC;
signal quadacnt: STD_LOGIC_VECTOR (2 downto 0);
signal quadafilt: STD_LOGIC;
signal quadb1: STD_LOGIC;
signal quadb2: STD_LOGIC;
signal quadbcnt: STD_LOGIC_VECTOR (2 downto 0);
signal quadbfilt: STD_LOGIC;
signal qcountup: STD_LOGIC; 
signal qcountdown: STD_LOGIC;
signal udcountup: STD_LOGIC; 
signal udcountdown: STD_LOGIC;
signal index1: STD_LOGIC;
signal index2: STD_LOGIC;
signal doclear: STD_LOGIC;
signal clearonindex: STD_LOGIC;	-- ccr register bits...
signal quadfilter: STD_LOGIC;
signal countermode: STD_LOGIC;
signal indexpol: STD_LOGIC;
signal localclear: STD_LOGIC;
signal divisor: STD_LOGIC;
signal latchrequestd1: STD_LOGIC;
signal latchrequestd2: STD_LOGIC;
signal countcleared: STD_LOGIC;

begin
	acounter: process 	(clk, 
	 							count,
								countoutlatch,
								ibus,
								quada,quada1,quada2,quadacnt,quadafilt,
							   quadb,quadb1,quadb2,quadbcnt,quadbfilt,
								index,index1,index2, 
								clearonindex,
								indexpol,
								countlatchcmd,
								ccrloadcmd,
								ccrreadcmd,
								doclear,
								localclear,
								quadfilter,
								countermode,
								qcountup,
								qcountdown,
								udcountup,
								udcountdown,
								countoutreadcmd,
							 	byteselect,
								countcleared,
								divisor
								)

	begin
		if clk'event and clk = '1' then -- per clk stuff
			divisor <= not divisor;
			if ccrloadcmd = '1' then	
				countermode <= ibus(7);
				quadfilter <= ibus(6);
				countcleared <= ibus(5);
				clearonindex <= ibus(4);
				indexpol <= ibus(3);
				localclear <= ibus(2);
			end if;
		
			if divisor = '1' then -- divided by 2 stuff
				latchRequestd1 <= countlatchcmd;
				latchRequestd2 <= latchrequestd1;
				if quadfilter = '1' then 
					quada1 <= quadafilt;
				else quada1 <= quada;
				end if;
				quada2 <= quada1;
				if quadfilter = '1' then 
					quadb1 <= quadbfilt;
				else quadb1 <= quadb;
				end if;
				quadb2 <= quadb1;

			-- deadended counter for a input filter --
				if (quada = '1') and (quadacnt /= 7) then 
				   quadacnt <= quadacnt + 1;
				end if;
				if (quada = '0') and (quadacnt /= 0) then 
				   quadacnt <= quadacnt -1;
				end if;
				if quadacnt = 7 then
				  quadafilt<= '1';
				end if;
				if quadacnt = 0 then
				  quadafilt<= '0';
				end if;
				
				-- deadended counter for b input filter --
				if (quadb = '1') and (quadbcnt /= 7) then 
				   quadbcnt <= quadbcnt + 1;
				end if;
				if (quadb = '0') and (quadbcnt /= 0) then 
				   quadbcnt <= quadbcnt -1;
				end if;
				if quadbcnt = 7 then
				  quadbfilt<= '1';
				end if;
				if quadbcnt = 0 then
				  quadbfilt<= '0';
				end if;	
				
				index1 <= index;
				index2 <= index1;
				if (localclear = '1') or 
				((clearonindex = '1')  and (index1 = '1') and (index2 = '0') and (indexpol = '1')) or		-- rising edge of index
				((clearonindex = '1')  and (index1 = '0') and (index2 = '1') and (indexpol = '0')) then	-- falling edge of index
					doclear <= '1';					
				else
					doclear <= '0';					
				end if;		
				if latchrequestd2 = '0'  and latchrequestd1 = '1' then		-- if latchrequest rising edge, latch counter and clear request
					countoutlatch <= count;
				end if;
				if countermode = '0' and (  
				(quada2 = '0' and quada1 = '1' and quadb2 = '0' and quadb1 = '0')  or
				(quada2 = '0' and quada1 = '0' and quadb2 = '1' and quadb1 = '0')  or
				(quada2 = '1' and quada1 = '1' and quadb2 = '0' and quadb1 = '1')  or
				(quada2 = '1' and quada1 = '0' and quadb2 = '1' and quadb1 = '1')) then		   
					qcountup <= '1';
				else 
					qcountup <= '0';
				end if;		

				if (countermode = '1' and 
				quadb2 = '1' and quada2 = '0' and quada1 = '1') then
 					udcountup <= '1';
				else
					udcountup <= '0';
				end if;
			
				if countermode = '0' and ( 
				(quada2 = '0' and quada1 = '0' and quadb2 = '0' and quadb1 = '1')  or
				(quada2 = '0' and quada1 = '1' and quadb2 = '1' and quadb1 = '1')  or
				(quada2 = '1' and quada1 = '0' and quadb2 = '0' and quadb1 = '0')  or
				(quada2 = '1' and quada1 = '1' and quadb2 = '1' and quadb1 = '0')) then
					qcountdown <= '1';
				else
					qcountdown <= '0';
				end if;

				if (countermode = '1' and  
				quadb2 = '0' and quada2 = '0' and quada1 = '1') then
					udcountdown <= '1';
				else
				   udcountdown <= '0';
				end if;	

				if (qcountup = '1' or udcountup = '1') then
					count <= count + 1; 
				end if;
				if (qcountdown = '1' or udcountdown = '1') then
					count <= count - 1;
				end if;	 
 
				if doclear = '1' then
					count <= x"00000000";
					countcleared <= '1';
				end if;


				if localclear = '1' then											-- once were done clearing,, dont stick around
					localclear <= '0';
				end if;
			end if;
		end if;
		obus <= "ZZZZZZZZ";
		if (countoutreadcmd = '1') and(ccrreadcmd = '0') then
	 		case byteselect is
		 		when "00" => obus <= countoutlatch(7 downto 0);
				when "01" => obus <= countoutlatch(15 downto 8);
				when "10" => obus <= countoutlatch(23 downto 16);
				when "11" => obus <= countoutlatch(31 downto 24);
				when others => null;
			end case;
		end if;		
		
		if (ccrreadcmd = '1') and (countoutreadcmd = '0') then
				obus(7) <= countermode;
				obus(6) <= quadfilter;
				obus(5) <= countcleared; 	
				obus(4) <= clearonindex;	
				obus(3) <= indexpol;
				obus(2) <= index;
				obus(1) <= quada1;
				obus(0) <= quadb1;
		end if;
	
	end process;
end behavioral;
